/// \file animacion.cpp Animación de imágenes.
/*
	Copyright 2010 Andrés Francisco Aparicio

	This file is part of SDLgolf.

	SDLgolf is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	SDLgolf is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with SDLgolf.  If not, see <http://www.gnu.org/licenses/>.
*/

#include <iostream>
#include "animacion.h"
#include "pantalla.h"
#include "utilidades.h"


Animacion::Animacion(Rejilla *rejilla, const int *secuencia,
	const Uint32& retardo, Pantalla *pantalla)
{
	// Limpiamos la secuencia.
	secuencia_.clear();

	// Asociamos con la pantalla.
	pantalla_ = pantalla;

	// Asociamos con la rejilla.
	rejilla_ = rejilla;

	// Establecemos la posición actual de la animación al inicio de la secuencia.
	cuadro_ = 0;

	// Establecemos el retardo.
	retardo_ = retardo;
	
	// Establecemos la posición de la animación.
	rrejilla_.x = 0;
	rrejilla_.y = 0;


	// Copiamos la secuencia.
	int x;
	for(x = 0; secuencia[x] != -1; x++)
		secuencia_.push_back(secuencia[x]);

	secuencia_.push_back(secuencia[x]);

	// La secuencia de la animación no se ha realizado todavía.
	secuencia_realizada_ = false;
}

void Animacion::Dibujar()
{
	// Seleccionamos el cuadro que tenemos que dibujar.
	rejilla_ -> SeleccionarCuadro(secuencia_[cuadro_]);

	// Dibujamos.
	pantalla_ -> Dibujar(rejilla_ -> Imagen(), &(rejilla_ -> Posicion()),
		&rrejilla_);
	
	// Aumentamos la posición de la animación.
	++cuadro_;
	
	// Si hemos llegado al final de la secuencia, vovlemos al principio
	// he indicamos que la secuencia ha sido realizada.
	if(secuencia_[cuadro_] == -1)
	{
		cuadro_ = 0;
		secuencia_realizada_ = true;
	}
}

void Animacion::DibujarCuadro()
{
	// Dibujamos el cuadro actual.
	pantalla_ -> Dibujar(rejilla_ -> Imagen(), &(rejilla_ -> Posicion()),
		&rrejilla_);
}

void Animacion::Posicion(const SDL_Rect& posicion)
{
	// Establecemos la posición indicada.
	rrejilla_.x = posicion.x;
	rrejilla_.y = posicion.y;
}

int Animacion::CuadroProcesado() const
{
	// Si la posición actual es 0 significa que el último cuadro procesado
	// ha sido el anterior al valor de límite -1.
	if(cuadro_ == 0)
		return secuencia_[secuencia_.size() - 2];
	
	// Si no es así devolvemos el cuadro anterior.
	return secuencia_[cuadro_ - 1];
}

Uint32 Animacion::Retardo() const
{
	// Devolvemos el retardo de la animación.
	return retardo_;
}

int Animacion::CuadroSiguiente() const
{
	// Devolvemos el cuadro que se va a ser procesado.
	return secuencia_[cuadro_];
}

void Animacion::Reiniciar()
{
	// Reestablecemos la animación a su situación inicial.
	cuadro_ = 0;

	// Seleccionamos el primer cuadro.
	rejilla_ -> SeleccionarCuadro(secuencia_[cuadro_]);

	// La animación no ha sido realizada todavía.
	secuencia_realizada_ = false;
}

bool Animacion::Realizada() const
{
	// Devolvemos si la animación ha sido realizada.
	return secuencia_realizada_;
}

