/// \file animacion.h Animación de imágenes.
/*
	Copyright 2010 Andrés Francisco Aparicio

	This file is part of SDLgolf.

	SDLgolf is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	SDLgolf is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with SDLgolf.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef _ANIMACION_H_
#define _ANIMACION_H_

#include <SDL/SDL.h>
#include <iostream>
#include <vector>
#include "rejilla.h"

class Pantalla;


/*! \class Animacion
    \brief Animación de imágenes.

    Esta clase implementa la generación de una animación mediante la
    representación de una determinada secuencia de imágenes extraídas
    de una rejilla.
    El efecto de animación se consigue por la superposición de imágenes
    en un intervalo de tiempo.
 */
class Animacion
{
	public:

		/*! Constructor de la animación.\n
				Realiza las asociaciones con la Pantalla y la Rejilla. Copia la
				secuencia de la animación y establece el retardo de la misma.
				\brief Constructor.
				\param rejilla Rejilla de imágenes a partir de la cual se generará
							 la animación.
				\param secuencia Secuencia de números que representan los cuadros
							 de la rejilla con los que se generará la animación.
				\param retardo Retardo en milisegundos que habrá entre la
							 representación de cada cuadro de la animación.
				\param pantalla Pantalla en la que se dibujará la animación.
				\see Rejilla.
		 */
		Animacion(Rejilla *rejilla, const int *secuencia, const Uint32& retardo,
			Pantalla *pantalla);			

		/*! Dibuja el cuadro de la animación en la Pantalla y selecciona el
				siguiente cuadro de la animación.
				\brief Dibuja el cuadro y avanza en la secuencia.
				\see Rejilla::Imagen.
		 */
		void Dibujar();
		
		/*! Dibuja el cuadro actual de la animación en la Pantalla sin cambiar
				el cuadro de la animación.
				\brief Dibuja el cuadro sin avanzar.
				\see Rejilla::Imagen.
		 */
		void DibujarCuadro();
		
		/*! Cambia la posición de la animación.
				\brief Cambia la posición.
				\param posicion Posición en la que se dibujará la animación.
		 */
		void Posicion(const SDL_Rect& posicion);
		
		/*! \brief Cuadro dibujado.
				\return Último cuadro de la animación que ha sido dibujado.
		 */
		int CuadroProcesado() const;
		
		/*! \brief Cuadro a dibujar.
				\return Cuadro de la animación que se va a dibujar.
		 */
		int CuadroSiguiente() const;
		
		/*!	\brief Retardo de la animación.
				\return Retardo en milisegundos que habrá entre la
							 	representación de cada cuadro de la animación.
		 */		
		Uint32 Retardo() const;
		
		/*! Reinicia la animación al principio de la secuencia.
				\brief Reinicia la secuencia.
		 */		
		void Reiniciar();
		
		/*! \brief Animación dibujada completamente.
				\return \arg \b Verdadero En caso de haberse realizado la secuencia
												de animación.
								\arg \b Falso En caso contrario.
		 */		
		bool Realizada() const;
	
	private:
	
		Rejilla *rejilla_;	///< Rejilla de imágenes de la animación.

		SDL_Rect rrejilla_; ///< Posición de la rejilla de imágenes.
		
		int cuadro_; ///< Cuadro seleccionado de la rejilla de animación.
		
		std::vector<int> secuencia_; ///< Secuencia de la animación.
		
		Uint32 retardo_; ///< Retardo en milisegundos que habrá entre la
							 			 ///< representación de cada cuadro de la animación.
		
		Pantalla *pantalla_; ///< Pantalla del juego.
		
		bool secuencia_realizada_; ///< Indica si la animación se ha realizado.
};

#endif
