/// \file bola.cpp Implementación de la bola de golf.
/*
	Copyright 2010 Andrés Francisco Aparicio

	This file is part of SDLgolf.

	SDLgolf is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	SDLgolf is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with SDLgolf.  If not, see <http://www.gnu.org/licenses/>.
*/

#include <iostream>
#include <math.h>
#include <fstream>
#include "bola.h"
#include "utilidades.h"
#include "palo.h"
#include "partida.h"
#include "datos.h"
#include "pantalla.h"
#include "campo.h"
#include "jugador.h"
#include "constantes.h"
#include "rejilla.h"


// Establecemos la altura mínima que tendrá la bola sobre la sombra.
int Bola::altura_minima_ = 1;

Bola::~Bola()
{
	// Liberamos la superficie de la imagen de la bola.
	SDL_FreeSurface(bola_);

	// Liberamos la superficie de la imagen de la sombra.
	SDL_FreeSurface(sombra_);
	
	// Destruimos la rejilla de las bolas inactivas.
	delete inactivas_;
}

Bola::Bola(Partida *partida, Pantalla *pantalla)
{
	// Asociamos la bola con la partida actual.
	partida_ = partida;

	// Asociamos la bola con la pantalla.
	pantalla_ = pantalla;

	// Establecemos el estado de la bola a inactiva.
	estado_ = INACTIVA;

	// Cadena temporal para la generación de rutas.
	std::string tmp;

	// Cargamos la imagen de la bola.
	bola_ = Utilidades::CargarImagen(RutaImagen("bola.png"));

	// Cargamos la imagen de la sombra.
	sombra_ = Utilidades::CargarImagen(RutaImagen("sombra.png"));

	// Creamos la ruta a la rejilla de bolas inactivas.
	tmp.clear();
	tmp.append(Constantes::_IMG_BOLA_);
	tmp += "inactivas.png";
	
	// Cargamos la rejilla de las bolas inactivas
	inactivas_ = new Rejilla(tmp, 1, 6);
	inactivas_ -> SeleccionarCuadro(0);
	
	// Establecemos la posición inicial de la bola.
	rbola_.x = 0;
	rbola_.y = 0;

	// Establecemos el ancho y el alto de la imagen de la bola.
	rbola_.w = bola_ -> w;
	rbola_.h = bola_ -> h;
	
	// Establecemos la posición inicial de la sombra.
	rsombra_.x = 0;
	rsombra_.y = 0;

	// Establecemos el ancho y el alto de la imagen de la sombra.
	rsombra_.w = sombra_ -> w;
	rsombra_.h = sombra_ -> h;
	
	// Establecemos las coordenadas iniciales de la bola.
	x_bola_ = 0;
	y_bola_ = 0;
	
	// Establecemos las coordenadas iniciales de la sombra.
	x_sombra_ = 0;
	y_sombra_ = 0;
	
	// Cargamos los efectos de sonido que genera la bola.
	golpe_.Cargar(RutaSonido("golpe.ogg").c_str());
	rebote_.Cargar(RutaSonido("rebote.ogg").c_str());
	agua_.Cargar(RutaSonido("agua.ogg").c_str());
	bunker_.Cargar(RutaSonido("bunker.ogg").c_str());
	arboles_.Cargar(RutaSonido("arboles.ogg").c_str());
	hoyo_.Cargar(RutaSonido("hoyo.ogg").c_str());
}

void Bola::DibujarBola() const
{
	// Si la bola está activa o está en el bunker dibujamos la imagen activa.
	if(estado_ == ACTIVA || estado_ == BUNKER)
		pantalla_ -> Dibujar(bola_, &rbola_);

	// Si la bola está inactiva o está inactiva en el bunker, dibujamos la
	// imagen de la bola inactiva.
	else if(estado_ == INACTIVA || estado_ == BUNKER_INACTIVA)
		pantalla_ -> Dibujar(inactivas_ -> Imagen(), &(inactivas_ -> Posicion()),
			&rbola_);
}

void Bola::DibujarSombra() const
{
	// Dibujamos la sombra de la bola.
	pantalla_ -> Dibujar(sombra_, &rsombra_);
}


int Bola::Desplazar(int x, int y)
{
	// Si los efectos de sonidos están activados reproducimos el sonido del
	// golpe.
	if(partida_ -> EfectosActivos())
		golpe_.Reproducir();

	// Seleccionamos el palo actual del jugador.
	const Palo& palo = partida_ -> JugadorActivo().PaloActual();

	// A continuación creamos el triángulo de desplazamiento de la bola.

	// La longitud del cateto contiguo es igual al valor absoluto de la
	// coordenada de destino x menos la coordenada en el eje de abcisas de
	// la sombra de la bola.
	float cateto_x = abs(x - rsombra_.x);

	// La longitud del cateto opuesto es igual al valor absoluto de la
	// coordenada de destino y menos la coordenada en el eje de ordenadas de
	// la sombra de la bola.
	float cateto_y = abs(y - rsombra_.y);

	// Aplicamos el Teorema de Pitágoras. La longitud de la hipotenusa es igual
	// a la suma de los cuadrados de las longitudes de los dos catetos.
	float hipotenusa = sqrt(pow(cateto_x, 2) + pow(cateto_y, 2));

	// Aplicamos la inversa de la tangente para obtener el ángulo.
	float angulo = atan(cateto_y / cateto_x);
	
	// Tenemos que generar tantos puntos de desplazamiento como larga sea la
	// hipotenusa.
	float puntos_a_generar = hipotenusa;

	// Todavía no se ha generado ningún punto del desplazamiento.
	float puntos_generados = 0;
	
	// Establecemos la velocidad de generación de puntos de desplazamiento.
	int velocidad = sqrt(hipotenusa) * 1/3;

	// El ángulo de desvío del tiro viene determinado por el indicador de
	// efecto del jugador activo.
	float angulo_de_desvio =
		partida_ -> JugadorActivo().IndicadorActual().Efecto();//efecto;

	// Conversión a radianes del ángulo de desvío en función de la posición
	// de la bola y las coordenadas de destino.
	float angulo_desvio;
	
	// Seleccionamos la posición actual del cursor del jugador activo.
	SDL_Rect rcursor = partida_ -> JugadorActivo().CursorActual().Posicion();

	// El ángulo de desvío varía en función de la posición de la bola y
	// las coordenadas de destino.
	if(rbola_.x + rbola_.w/2 <= rcursor.x + rcursor.w/2
	&& rbola_.y + rbola_.h/2 >= rcursor.y + rcursor.h/2)
		angulo_desvio = angulo - (angulo_de_desvio * M_PI) / 180;
		
	else if(rbola_.x + rbola_.w/2 >= rcursor.x + rcursor.w/2
	&& rbola_.y + rbola_.h/2 >= rcursor.y + rcursor.h/2)
		angulo_desvio = angulo + (angulo_de_desvio * M_PI) / 180;

	else if(rbola_.x + rbola_.w/2 >= rcursor.x + rcursor.w/2
	&& rbola_.y + rbola_.h/2 <= rcursor.y + rcursor.h/2)
		angulo_desvio = angulo - (angulo_de_desvio * M_PI) / 180;

	else if(rbola_.x + rbola_.w/2 <= rcursor.x + rcursor.w/2
	&& rbola_.y + rbola_.h/2  <= rcursor.y + rcursor.h/2)
		angulo_desvio = angulo + (angulo_de_desvio * M_PI) / 180;
	
	// Establecemos el coseno del ángulo de desvío.
	float coseno_desvio = cos(angulo_desvio);

	// Establecemos el seno del ángulo de desvío.
	float seno_desvio = sin(angulo_desvio);
	
	// Vamos a generar el efecto de desvío mediante la imagen de la función
	// seno. La función seno genera una curva perfectra entre 0 y el número
	// pi, por lo que dividiremos dicho número entre la longitud de la
	// para saber el incremento que hemos de aplicar a la curva en cada
	// iteración del bucle que genera los puntos del desplazamiento de la bola.
	float incremento_curva = M_PI / hipotenusa;

	// Todavía no se ha generado ningún punto de desplazamiento, por lo que
	// el valor de la curva es 0.
	float curva = 0;

	
	// Aplicamos el valor de la gravedad. Este valor es indiferente, puesto
	// que el tiempo viene determinado por la longitud del recorrido.
	float gravedad = 1;

	// El ángulo de salida viene determinado por el palo que tenga seleccionado
	// el jugador activo.
	float angulo_salida = (palo.AnguloSalida() * M_PI) / 180;

	// Velocidad de salida del tiro parabólico.
	float velocidad_salida = sqrt((hipotenusa * gravedad)
		/ sin(2 * angulo_salida));
	
	// Tiempo que es necesario para realizar el tiro parabólico.
	float tiempo = hipotenusa / (velocidad_salida * cos(angulo_salida));
	
	// Dividimos el tiempo que necesitamos entre la hipotenusa para saber
	// en cuanto tenemos que incrementar el tiempo en cada iteración.
	float incremento_tiempo = (tiempo * velocidad) / hipotenusa;

	// Establecemos el tiempo a 0 dado que todavía no ha comenzado el
	// desplazamiento.
	float t = 0;	
	
	// Guardamos los datos del desplazamiento.
	// Esto nos permite restaurar la posición de la bola a una zona fuera de
	// colisiones en caso que se produzca dicha colisión.
	desplazamiento_.x = x;
	desplazamiento_.y = y;
	desplazamiento_.rsombra_x = rsombra_.x;
	desplazamiento_.rsombra_y = rsombra_.y;
	desplazamiento_.velocidad = velocidad;
	desplazamiento_.hipotenusa = hipotenusa;
	desplazamiento_.puntos_generados = puntos_generados;
	desplazamiento_.incremento_curva = incremento_curva;
	desplazamiento_.curva = curva;
	desplazamiento_.angulo = angulo;
	desplazamiento_.angulo_de_desvio = angulo_de_desvio;
	desplazamiento_.coseno_desvio = coseno_desvio;
	desplazamiento_.seno_desvio = seno_desvio;

	// Antes de realizar el golpe hemos realizado una animación previa en la
	// que el personaje movía el palo hasta el momento exacto de golpear la bola.
	// Ahora, hemos de continuadr dicha animación, por lo que seleccionamos
	// la que tenemos seguir dibujando.
	Animacion& animacion = partida_ -> JugadorActivo().AnimacionActual();
	
	// Si la animación no es válida salimos.
	if(!&animacion)
	{
		std::cerr << "Bola::Desplazar() Error, animación no válida." << std::endl;
		exit(1);
	}
	// Variables de control de tiempo para el bucle de la animación.
	Uint32 ta, tb;

	// Varaibles de control de tiempo para la animación en sí.
	Uint32 ta_animacion, tb_animacion;

	// Variable de control de tiempo para el retardo de la animación.
	Uint32 retardo_animacion = animacion.Retardo();

	// Variable de control para saber si la animación se ha realizado.
	bool animacion_realizada = false;
	
	// Si la velocidad es 0, realizamos lo que queda de animación (continuación
	// del método AnimacionPrevia() de la clase Jugador) y terminamos.
	if(velocidad <= 0)
	{
		// Guardamos el tiempo actual.
		ta_animacion = SDL_GetTicks();

		// Dibujamos la animación hasta que no esté completamente realizada.
		while(!animacion.Realizada())
		{
			// Guardamos el tiempo para controlar el while.
			ta = SDL_GetTicks();
			
			// Guardamos el tiempo para generar la animación correctamente.		
			tb_animacion = SDL_GetTicks();
			
			// Si el tiempo que ha pasado es mayor al retardo de la animación
			// significa que tenemos que dibujarla.
			if(tb_animacion - ta_animacion > retardo_animacion)
			{
				partida_ -> CentrarCamara();
				partida_ -> CampoActivo().Dibujar();
				partida_ -> DibujarBolasInactivas();
				this -> DibujarSombra();
				this -> DibujarBola();				
				animacion.Dibujar();
				partida_ -> CampoActivo().DibujarBandera();
				partida_ -> DibujarInterfaz();
				pantalla_ -> Actualizar();

				// Establecemos los nuevos valores para las variables de control
				// de tiempo.
				ta_animacion = SDL_GetTicks();
				tb_animacion = SDL_GetTicks();			
			}
				
			// Guardamos el tiempo.
			tb = SDL_GetTicks();

			// Si el tiempo transcurrido es menor al establecido en el juego
			// tenemos que esperar.
			if(tb - ta < Constantes::_MS_)
				SDL_Delay(Constantes::_MS_ - (tb-ta));
		
		}

		// Una vez realizada la animación por completo la reiniciamos.
		animacion.Reiniciar();
		
		// Dibujamos todo.
		partida_ -> CentrarCamara();
		partida_ -> CampoActivo().Dibujar();
		partida_ -> DibujarBolasInactivas();
		this -> DibujarSombra();
		this -> DibujarBola();
		partida_ -> JugadorActivo().Dibujar();
		partida_ -> CampoActivo().DibujarBandera();
		partida_ -> DibujarInterfaz();
		pantalla_ -> Actualizar();	
		
		// Si la bola no estaba en un bunker, establecemos su estado a activa.
		if(estado_ != BUNKER)
			estado_ = ACTIVA;
		
		// Esperamos un tiempo.
		partida_ -> Esperar(2000, true, true, false, false, false, 0, 0);

		// Terminamos.
		return 0;
	}

	// Si la velocidad es mayor a 0, tendremos que dibujar lo que falta de
	// animación. Guardamos el tiempo actual.
	ta_animacion = SDL_GetTicks();

	// El desplazamiento consta de 3 partes. Primero se describe un tiro
	// parabólico hasta llegar a las coordenadas de destino. Luego, se reduce
	// la velocidad y si ésta es suficiente la bola rebotará describiendo
	// y por último, se volverá a reducir la velocidad y si ésta es suficiente
	// la bola rodará.

	// Realizamos el bucle hasta que no se hayan generado todos los puntos
	// de desplazamiento determinados por la longitud de la hipotenusa.
	while(puntos_generados < puntos_a_generar)
	{
		// Guardamos el tiempo para controlar el bucle.
		ta = SDL_GetTicks();

		// En función de las coordenadas iniciales, vamos incrementando o
		// decrementando la posición en el eje de las x.
		if(x_sombra_ < x)
			rsombra_.x = x_sombra_ + puntos_generados * coseno_desvio;
		else
			rsombra_.x = x_sombra_ - puntos_generados * coseno_desvio;

		// Lo mismo para el eje de las y.
		if(y_sombra_ < y)
		{
			if(angulo_desvio < angulo)
			{				
				if(x_sombra_ < x)	
					rsombra_.y = y_sombra_ + puntos_generados * seno_desvio
					- (hipotenusa / 168) * angulo_de_desvio * sin(curva);
				else
					rsombra_.y = y_sombra_ + puntos_generados * seno_desvio
					+ (hipotenusa / 168) * angulo_de_desvio * sin(curva);			
			}
			else
			{
				if(x_sombra_ < x)
				{
					if(angulo_desvio > M_PI / 2)
						rsombra_.y = y_sombra_ + puntos_generados * seno_desvio
							+ (hipotenusa / 168) * angulo_de_desvio
							* sin(curva);
					else
						rsombra_.y = y_sombra_ + puntos_generados * seno_desvio
							- (hipotenusa / 168) * angulo_de_desvio
							* sin(curva);
				}
				else
				{
					if(angulo_desvio > M_PI / 2)
						rsombra_.y = y_sombra_ + puntos_generados * seno_desvio
							- (hipotenusa / 168) * angulo_de_desvio
							* sin(curva);
					else
						rsombra_.y = y_sombra_ + puntos_generados * seno_desvio
							+ (hipotenusa / 168) * angulo_de_desvio
							* sin(curva);				
				}				
			}				
		}
		else
		{		
			if(angulo_desvio < angulo)
			{
				if(x_sombra_ < x)
					rsombra_.y = y_sombra_ - puntos_generados * seno_desvio
						- (hipotenusa / 168) * angulo_de_desvio
						* sin(curva);
				else
					rsombra_.y = y_sombra_ - puntos_generados * seno_desvio
						+ (hipotenusa / 168) * angulo_de_desvio
						* sin(curva);
			}
			else
			{
				if(x_sombra_ < x)
				{
					if(angulo_desvio > M_PI / 2)
						rsombra_.y = y_sombra_ - puntos_generados * seno_desvio
							+ (hipotenusa / 168) * angulo_de_desvio
							* sin(curva);
					else
						rsombra_.y = y_sombra_ - puntos_generados * seno_desvio
							- (hipotenusa / 168) * angulo_de_desvio
							* sin(curva);
				}
				else
				{
					if(angulo_desvio > M_PI / 2)
						rsombra_.y = y_sombra_ - puntos_generados * seno_desvio
						- (hipotenusa / 168) * angulo_de_desvio
						* sin(curva);
					else
						rsombra_.y = y_sombra_ - puntos_generados * seno_desvio
						+ (hipotenusa / 168) * angulo_de_desvio
						* sin(curva);				
				}				
			}				
		}

		// Incrementamos el efecto de desvío.
		curva += velocidad * incremento_curva;

		// Establecemos la posición de la bola que viene descrita por el tiro
		// parabólico.
		rbola_.x = rsombra_.x;
		rbola_.y = rsombra_.y - t * velocidad_salida * sin(angulo_salida)
			+ pow(t, 2) * gravedad * 0.5;	

		// Incrementamos el tiempo.
		t += incremento_tiempo;

		// Incrementamos los puntos generados en función de la velocidad.
		puntos_generados += velocidad;

		// Actualizamos los datos del desplazamiento.
		desplazamiento_.puntos_generados = puntos_generados;
		desplazamiento_.curva = curva;

		// Comprobamos que las coordenadas estén dentro de los límits del campo.
		// Si no es así, lo procesamos y terminamos.
		if(partida_ -> FueraDeLimites(0, 0))
		{
			partida_ -> ProcesarColision(Campo::FDL);
			return 0;
		}
		
		// Si la altura de la bola es menor que la de los árboles, comprobamos
		// si hay colisión. En caso de haber, procesamos y terminamos.
		if(this -> AlturaBola() <= Constantes::_ALTURA_ARBOLES_)
		{
			if(palo.Nombre() != "Putter")
			{
				if(partida_ -> ComprobarColision(Partida::CENTRO,
				Campo::ARBOLES, 0, 0))
				{
					if(partida_ -> EfectosActivos())
						arboles_.Reproducir();

					partida_ -> ProcesarColision(Campo::ARBOLES);
					return 0;
				}
			}
			else
			{
				if(partida_ -> ComprobarColision(Partida::CENTRO,
				Campo::ARBOLES, 0, 0))
				{
					if(partida_ -> EfectosActivos())
						arboles_.Reproducir();

					partida_ -> ProcesarColision(Campo::ARBOLES);			
					return 0;
				}
	
				if(partida_ -> ComprobarColision(Partida::CENTRO,
				Campo::AGUA, 0, 0))
				{
					if(partida_ -> EfectosActivos())
						agua_.Reproducir();

					partida_ -> ProcesarColision(Campo::AGUA);			
					return 0;
				}
	
				if(partida_ -> ComprobarColision(Partida::CENTRO,
				Campo::BUNKER, 0, 0))
				{
					// En caso de que se produzca una colisión con el bunker guardamos
					// la posición de la bola y su sombra y a continuación procemos la
					// colisión y terminamos.
					if(partida_ -> EfectosActivos())
						bunker_.Reproducir();

					rbola_.x = rsombra_.x;
					rbola_.y = rsombra_.y - altura_minima_;

					x_bola_ = rbola_.x;
					y_bola_ = rbola_.y;

					x_sombra_ = rsombra_.x;
					y_sombra_ = rsombra_.y;

					partida_ -> ProcesarColision(Campo::BUNKER);			
					return 0;
				}
				if(partida_ -> ComprobarColision(Partida::COMPLETO,
				Campo::HOYO, 0, 0))
				{
					if(partida_ -> EfectosActivos())
						hoyo_.Reproducir();

					partida_ -> ProcesarColision(Campo::HOYO);			
					return 0;
				}			
			}
		}
		
		// Guardamos el tiempo para la animación.
		tb_animacion = SDL_GetTicks();
		
		// Si el tiempo transcurrido es mayor que el del retardo de la animación
		// todavía no se ha realizado, la dibujamos.
		if((tb_animacion - ta_animacion) > retardo_animacion
		&& !animacion_realizada)
		{
			partida_ -> CentrarCamara();
			partida_ -> CampoActivo().Dibujar();
			partida_ -> DibujarBolasInactivas();
			this -> DibujarSombra();
			this -> DibujarBola();
			animacion.Dibujar();
			partida_ -> CampoActivo().DibujarBandera();
			partida_ -> DibujarInterfaz();
			pantalla_ -> Actualizar();

			// Establecemos si la animación se ha realizado.
			animacion_realizada = animacion.Realizada();
			
			// En caso de que se haya realizado, comprobamos si la animación que
			// se ha completado es particular. Hay tres secuencias particulares
			// en las que tenemos que elegir otro cuadro final para que no se vean
			// huecos en el dibujo del personaje.
			if(animacion_realizada)
			{
				if(animacion.CuadroProcesado() / 9 == 6)
					partida_ -> JugadorActivo().SeleccionarCuadro(60);
				
				if(animacion.CuadroProcesado() / 9 == 5)
					partida_ -> JugadorActivo().SeleccionarCuadro(52);
					
				if(animacion.CuadroProcesado() / 9 == 4)
					partida_ -> JugadorActivo().SeleccionarCuadro(43);					
			}

			// Guardamos el tiempo en las variables de control.
			ta_animacion = SDL_GetTicks();
			tb_animacion = SDL_GetTicks();
		}

		// En caso de que la animación si se haya realizado, dibujamos sin más.
		else
		{
			partida_ -> CentrarCamara();
			partida_ -> CampoActivo().Dibujar();
			partida_ -> DibujarBolasInactivas();
			this -> DibujarSombra();
			this -> DibujarBola();
			animacion.DibujarCuadro();
			partida_ -> CampoActivo().DibujarBandera();
			partida_ -> DibujarInterfaz();
			pantalla_ -> Actualizar();
		}

		tb = SDL_GetTicks();
		
		// Si el tiempo transcurrido es mayor al establecido en el juego
		// tenemos que esperar.
		if(tb-ta < Constantes::_MS_)
			SDL_Delay(Constantes::_MS_ - (tb-ta));

	} // while


/******************************************************************************/


	// Reiniciamos la animación.
	animacion.Reiniciar();

	// En caso de que se haya realizado, comprobamos si la animación que
	// se ha completado es particular. Hay tres secuencias particulares
	// en las que tenemos que elegir otro cuadro final para que no se vean
	// huecos en el dibujo del personaje.	
	if(animacion.CuadroSiguiente() / 9 == 6)
		partida_ -> JugadorActivo().SeleccionarCuadro(60);

	if(animacion.CuadroSiguiente() / 9 == 5)
		partida_ -> JugadorActivo().SeleccionarCuadro(52);
	
	if(animacion.CuadroSiguiente() / 9 == 4)
		partida_ -> JugadorActivo().SeleccionarCuadro(43);

	// Comprobamos que las coordenadas estén dentro de los límits del campo.
	// Si no es así, lo procesamos y terminamos.
	if(partida_ -> FueraDeLimites(0, 0))
	{
		partida_ -> ProcesarColision(Campo::FDL);
		return 0;
	}

	// Ahora mismo la bola se encuentra en la posición de rebote. Hemos de
	// de comprobar todas las colisiones posibles. En caso de haberlas,
	// se procesarán y se terminará.

	if(partida_ -> ComprobarColision(Partida::CENTRO,
	Campo::ARBOLES, 0, 0))
	{
		if(partida_ -> EfectosActivos())
			arboles_.Reproducir();

		partida_ -> ProcesarColision(Campo::ARBOLES);			
		return 0;
	}
	
	if(partida_ -> ComprobarColision(Partida::CENTRO,
	Campo::AGUA, 0, 0))
	{
		if(partida_ -> EfectosActivos())
			agua_.Reproducir();

		partida_ -> ProcesarColision(Campo::AGUA);			
		return 0;
	}
	
	if(partida_ -> ComprobarColision(Partida::CENTRO,
	Campo::BUNKER, 0, 0))
	{
		// En caso de que se produzca una colisión con el bunker guardamos
		// la posición de la bola y su sombra y a continuación procemos la
		// colisión y terminamos.

		if(partida_ -> EfectosActivos())
			bunker_.Reproducir();

		rbola_.x = rsombra_.x;
		rbola_.y = rsombra_.y - altura_minima_;

		x_bola_ = rbola_.x;
		y_bola_ = rbola_.y;

		x_sombra_ = rsombra_.x;
		y_sombra_ = rsombra_.y;

		partida_ -> ProcesarColision(Campo::BUNKER);			
		return 0;
	}
	
	if(partida_ -> ComprobarColision(Partida::COMPLETO,
		Campo::HOYO, 0, 0))
	{
		if(partida_ -> EfectosActivos())
			hoyo_.Reproducir();

		partida_ -> ProcesarColision(Campo::HOYO);			
		return 0;
	}
	
	// Reducimos la velocidad a la mitad.
	velocidad = velocidad * 50/100;

	// En caso de que la velocidad sea menor a 0, comprobamos si estamos
	// en una secuencia de animación particular y elegimos el cuadro
	// necesario. Dibujamos todo y terminamos.
	if(velocidad <= 0)
	{
		if(animacion.CuadroSiguiente() / 9 == 6)
			partida_ -> JugadorActivo().SeleccionarCuadro(60);
		
		if(animacion.CuadroSiguiente() / 9 == 5)
			partida_ -> JugadorActivo().SeleccionarCuadro(52);

		if(animacion.CuadroSiguiente() / 9 == 4)	
			partida_ -> JugadorActivo().SeleccionarCuadro(43);

		// Establecemos la posición de la sombra.
		rsombra_.x = rbola_.x;
		rsombra_.y = rbola_.y + altura_minima_;

		// Guardamos las coordenadas iniciales.
		x_bola_ = rbola_.x;
		y_bola_ = rbola_.y;
		x_sombra_ = rsombra_.x;
		y_sombra_ = rsombra_.y;
		
		partida_ -> CentrarCamara();
		partida_ -> CampoActivo().Dibujar();
		partida_ -> DibujarBolasInactivas();
		this -> DibujarSombra();
		this -> DibujarBola();
		partida_ -> JugadorActivo().Dibujar();
		partida_ -> CampoActivo().DibujarBandera();
		partida_ -> DibujarInterfaz();
		pantalla_ -> Actualizar();

		// No hubo colisión, por lo que la bola está activa.
		estado_ = ACTIVA;

		// Esperamos.
		partida_ -> Esperar(2000, true, true, false, false, false, 0, 0);
		
		// Terminamos.
		return 0;
	}
	
	// La velocidad ha sido mayor a 0, por lo tanto existirá rebote.
	// Establecemos la distancia del rebote a 0.
	float distancia_rebote = 0;

	if(palo.Nombre() != "Putter")
	{
		// Si los efectos de sonido están activos reproducimos el sonido de
		// rebote en la superficie del campo.
		if(partida_ -> EfectosActivos())
			rebote_.Reproducir();

		// Establecemos la distancia del rebote.
		distancia_rebote = puntos_a_generar * 8/100;

		// Establecemos la nueva velocidad de salida.
		velocidad_salida = sqrt((distancia_rebote * gravedad)
			/ sin(2 * angulo_salida));

		// El tiempo viene determinado por la nueva distancia.
		tiempo = distancia_rebote / (velocidad_salida * cos(angulo_salida));

		// Calculamos el incremento del tiempo.
		incremento_tiempo = (tiempo * velocidad) / distancia_rebote;

		// Ponemos a 0 la variable de tiempo transcurrido.
		t = 0;
	
		// Este bucle genera el rebote de la bola.
		for(float i=puntos_a_generar; i < puntos_a_generar + distancia_rebote;
		i += velocidad)
		{
			// Guardamos el tiempo para el control del bucle.
			ta= SDL_GetTicks();

			// En función de las coordenadas iniciales, vamos incrementando o
			// decrementando la posición en el eje de las x.		
			if(x_sombra_ < x)
				rsombra_.x = x_sombra_ + i * coseno_desvio;
			else
				rsombra_.x = x_sombra_ - i * coseno_desvio;		

			// Lo mismo para el eje de las y.
			if(y_sombra_ < y)
			{
				if(angulo_desvio < angulo)
				{				
					if(x_sombra_ < x)	
						rsombra_.y = y_sombra_ + i * seno_desvio
						- (hipotenusa / 168) * angulo_de_desvio
						* sin(curva);
					else
						rsombra_.y = y_sombra_ + i * seno_desvio
						+ (hipotenusa / 168) * angulo_de_desvio
						* sin(curva);			
				}
				else
				{
					if(x_sombra_ < x)
					{
						if(angulo_desvio > M_PI / 2)
							rsombra_.y = y_sombra_ + i * seno_desvio
								+ (hipotenusa / 168) * angulo_de_desvio
								* sin(curva);
						else
							rsombra_.y = y_sombra_ + i * seno_desvio
								- (hipotenusa / 168) * angulo_de_desvio
								* sin(curva);
					}
					else
					{
						if(angulo_desvio > M_PI / 2)
							rsombra_.y = y_sombra_ + i * seno_desvio
								- (hipotenusa / 168) * angulo_de_desvio
								* sin(curva);
						else
							rsombra_.y = y_sombra_ + i * seno_desvio
								+ (hipotenusa / 168) * angulo_de_desvio
								* sin(curva);				
					}				
				}				
			}
			else
			{		
				if(angulo_desvio < angulo)
				{
					if(x_sombra_ < x)
						rsombra_.y = y_sombra_ - i * seno_desvio
							- (hipotenusa / 168) * angulo_de_desvio
							* sin(curva);
					else
						rsombra_.y = y_sombra_ - i * seno_desvio
							+ (hipotenusa / 168) * angulo_de_desvio
							* sin(curva);
				}
				else
				{
					if(x_sombra_ < x)
					{
						if(angulo_desvio > M_PI / 2)
							rsombra_.y = y_sombra_ - i * seno_desvio
								+ (hipotenusa / 168) * angulo_de_desvio
								* sin(curva);
						else
							rsombra_.y = y_sombra_ - i * seno_desvio
								- (hipotenusa / 168) * angulo_de_desvio
								* sin(curva);
					}
					else
					{
						if(angulo_desvio > M_PI / 2)
							rsombra_.y = y_sombra_ - i * seno_desvio
							- (hipotenusa / 168) * angulo_de_desvio
							* sin(curva);
						else
							rsombra_.y = y_sombra_ - i * seno_desvio
							+ (hipotenusa / 168) * angulo_de_desvio
							* sin(curva);				
					}				
				}				
			}

			// Determinamos la función de la bola para el tiro parabólico.
			rbola_.x = rsombra_.x;
			rbola_.y = rsombra_.y - t * velocidad_salida * sin(angulo_salida)
				+ pow(t, 2) * gravedad * 0.5;	

			// Incrementamos la curva del efecto de desvío.
			curva += velocidad * incremento_curva;

			// Incrementamos la variable de tiempo transcurrido.
			t += incremento_tiempo;	
			
			// Actualizamos los datos de desplazamiento.
			desplazamiento_.puntos_generados = i;
			desplazamiento_.curva = curva;

			// Comprobamos que las coordenadas estén dentro de los límits del campo.
			// Si no es así, lo procesamos y terminamos.
			if(partida_ -> FueraDeLimites(0, 0))
			{
				partida_ -> ProcesarColision(Campo::FDL);
				return 0;
			}

			// Si la altura de la bola es menor que la de los árboles, comprobamos
			// si hay colisión. En caso de haber, procesamos y terminamos.
			if(this -> AlturaBola() <= Constantes::_ALTURA_ARBOLES_)
			{
				if(partida_ -> ComprobarColision(Partida::CENTRO,
					Campo::ARBOLES, 0, 0))
				{
					if(partida_ -> EfectosActivos())
						arboles_.Reproducir();

					partida_ -> ProcesarColision(Campo::ARBOLES);
					return 0;
				}			
			}
			
			partida_ -> CentrarCamara();
			partida_ -> CampoActivo().Dibujar();
			partida_ -> DibujarBolasInactivas();
			this -> DibujarSombra();
			this -> DibujarBola();
			partida_ -> JugadorActivo().Dibujar();
			partida_ -> CampoActivo().DibujarBandera();
			partida_ -> DibujarInterfaz();
			pantalla_ -> Actualizar();
			
			// Guardamos el tiempo.
			tb = SDL_GetTicks();

			// Si el tiempo transcurrido es mayor que el establecido en el juego
			// tenemos que esperar.
			if(tb-ta < Constantes::_MS_)
				SDL_Delay(Constantes::_MS_ - (tb-ta));

		} // for
	} // if


/******************************************************************************/


	// En caso de que la velocidad sea menor a 0, comprobamos si estamos
	// en una secuencia de animación particular y elegimos el cuadro
	// necesario. Dibujamos todo y terminamos.
	if(partida_ -> FueraDeLimites(0, 0))
	{
		partida_ -> ProcesarColision(Campo::FDL);
		return 0;
	}

	// La bola vuelva a impactar contra la superficie del campo. Hemos
	// de comprobar todas las colisiones posibles. En caso de haberlas,
	// se procesarán y se terminará.
	if(partida_ -> ComprobarColision(Partida::CENTRO, Campo::ARBOLES, 0, 0))
	{
		if(partida_ -> EfectosActivos())
			arboles_.Reproducir();

		partida_ -> ProcesarColision(Campo::ARBOLES);			
		return 0;
	}
	
	if(partida_ -> ComprobarColision(Partida::CENTRO, Campo::AGUA, 0, 0))
	{
		if(partida_ -> EfectosActivos())
			agua_.Reproducir();

		partida_ -> ProcesarColision(Campo::AGUA);			
		return 0;
	}
	
	if(partida_ -> ComprobarColision(Partida::CENTRO, Campo::BUNKER, 0, 0))
	{
		// En caso de que se produzca una colisión con el bunker guardamos
		// la posición de la bola y su sombra y a continuación procemos la
		// colisión y terminamos.
		if(partida_ -> EfectosActivos())
			bunker_.Reproducir();

		rbola_.x = rsombra_.x;
		rbola_.y = rsombra_.y - altura_minima_;

		x_bola_ = rbola_.x;
		y_bola_ = rbola_.y;

		x_sombra_ = rsombra_.x;
		y_sombra_ = rsombra_.y;

		partida_ -> ProcesarColision(Campo::BUNKER);			
		return 0;
	}
	
	if(partida_ -> ComprobarColision(Partida::COMPLETO, Campo::HOYO, 0, 0))
	{
		if(partida_ -> EfectosActivos())
			hoyo_.Reproducir();

		partida_ -> ProcesarColision(Campo::HOYO);			
		return 0;
	}	
	
	// Hemos terminado el rebote. La bola vuelve a golpear contra la superficie
	// del campo, así que volvemos a reducir la velocidad.
	velocidad = velocidad * 60/100;

	// Establecemos la distancia rodando a 0.
	float distancia_rodando = 0;
	
	// Si la velocidad es nula, guardamos las coordenadas y terminamos.
	if(velocidad <= 0)
	{
		// Si estamos en una secuencia particular, elegimos el cuadro necesario.
		if(animacion.CuadroSiguiente() / 9 == 6)
			partida_ -> JugadorActivo().SeleccionarCuadro(60);
	
		if(animacion.CuadroSiguiente() / 9 == 5)
			partida_ -> JugadorActivo().SeleccionarCuadro(52);
		
		if(animacion.CuadroSiguiente() / 9 == 4)
			partida_ -> JugadorActivo().SeleccionarCuadro(43);	

		// Guardamos la posición de la sombra.
		rsombra_.x = rbola_.x;
		rsombra_.y = rbola_.y + altura_minima_;

		// Guardamos las coordenadas iniciales.
		x_bola_ = rbola_.x;
		y_bola_ = rbola_.y;
		x_sombra_ = rsombra_.x;
		y_sombra_ = rsombra_.y;

		// Dibujamos todo.
		partida_ -> CentrarCamara();
		partida_ -> CampoActivo().Dibujar();
		partida_ -> DibujarBolasInactivas();
		this -> DibujarSombra();
		this -> DibujarBola();
		partida_ -> JugadorActivo().Dibujar();
		partida_ -> CampoActivo().DibujarBandera();
		partida_ -> DibujarInterfaz();
		pantalla_ -> Actualizar();

		// No hubo colisión, así que el estado de la bola es activa.
		estado_ = ACTIVA;

		// Esperamos.
		partida_ -> Esperar(2000, true, true, false, false, false, 0, 0);
		
		// Terminamos.
		return 0;		
	}
	
	if(palo.Nombre() != "Putter")
	{	
		// Si los efectos de sonido están activos reproducimos el sonido de
		// rebote en la superficie del campo.
		if(partida_ -> EfectosActivos())
			rebote_.Reproducir();

		// Establecemos la distancia en la que la bola rodará.
		distancia_rodando = puntos_a_generar * 2/100;

		for(float i = puntos_a_generar + distancia_rebote;
		i < puntos_a_generar + distancia_rebote + distancia_rodando;
		i += velocidad)
		{
			// Guardamos el tiempo para el control del bucle.
			ta= SDL_GetTicks();	

			// En función de las coordenadas iniciales, vamos incrementando o
			// decrementando la posición en el eje de las x.	
			if(x_sombra_ < x)
				rsombra_.x = x_sombra_ + i * coseno_desvio;
			else
				rsombra_.x = x_sombra_ - i * coseno_desvio;		

			// Lo mismo para el eje de las y.
			if(y_sombra_ < y)
			{
				if(angulo_desvio < angulo)
				{				
					if(x_sombra_ < x)	
						rsombra_.y = y_sombra_ + i * seno_desvio
						- (hipotenusa / 168) * angulo_de_desvio
						* sin(curva);
					else
						rsombra_.y = y_sombra_ + i * seno_desvio
						+ (hipotenusa / 168) * angulo_de_desvio
						* sin(curva);			
				}
				else
				{
					if(x_sombra_ < x)
					{
						if(angulo_desvio > M_PI / 2)
							rsombra_.y = y_sombra_ + i * seno_desvio
								+ (hipotenusa / 168) * angulo_de_desvio
								* sin(curva);
						else
							rsombra_.y = y_sombra_ + i * seno_desvio
								- (hipotenusa / 168) * angulo_de_desvio
								* sin(curva);
					}
					else
					{
						if(angulo_desvio > M_PI / 2)
							rsombra_.y = y_sombra_ + i * seno_desvio
								- (hipotenusa / 168) * angulo_de_desvio
								* sin(curva);
						else
							rsombra_.y = y_sombra_ + i * seno_desvio
								+ (hipotenusa / 168) * angulo_de_desvio
								* sin(curva);				
					}				
				}				
			}
			else
			{		
				if(angulo_desvio < angulo)
				{
					if(x_sombra_ < x)
						rsombra_.y = y_sombra_ - i * seno_desvio
							- (hipotenusa / 168) * angulo_de_desvio
							* sin(curva);
					else
						rsombra_.y = y_sombra_ - i * seno_desvio
							+ (hipotenusa / 168) * angulo_de_desvio
							* sin(curva);
				}
				else
				{
					if(x_sombra_ < x)
					{
						if(angulo_desvio > M_PI / 2)
							rsombra_.y = y_sombra_ - i * seno_desvio
								+ (hipotenusa / 168) * angulo_de_desvio
								* sin(curva);
						else
							rsombra_.y = y_sombra_ - i * seno_desvio
								- (hipotenusa / 168) * angulo_de_desvio
								* sin(curva);
					}
					else
					{
						if(angulo_desvio > M_PI / 2)
							rsombra_.y = y_sombra_ - i * seno_desvio
							- (hipotenusa / 168) * angulo_de_desvio
							* sin(curva);
						else
							rsombra_.y = y_sombra_ - i * seno_desvio
							+ (hipotenusa / 168) * angulo_de_desvio
							* sin(curva);				
					}				
				}				
			}

			// La posición de la bola será igual a la de la sombra, ya que está
			// rodando.
			rbola_.x = rsombra_.x;
			rbola_.y = rsombra_.y;
						
			// Incrementamos el efecto de desvío.
			curva += velocidad * incremento_curva;

			// Incrementamos el tiempo transcurrido.
			t += incremento_tiempo;	

			// Actualizamos los datos del desplazamiento.
			desplazamiento_.puntos_generados = i;
			desplazamiento_.curva = curva;

			// Comprobamos que las coordenadas estén dentro de los límits del campo.
			// Si no es así, lo procesamos y terminamos.
			if(partida_ -> FueraDeLimites(0, 0))
			{
				partida_ -> ProcesarColision(Campo::FDL);
				return 0;
			}

			// La bola está rodando sobre la superficie del campo. Hemos
			// de comprobar todas las colisiones posibles. En caso de haberlas,
			// se procesarán y se terminará.
			if(partida_ -> ComprobarColision(Partida::CENTRO, Campo::ARBOLES, 0, 0))
			{
				if(partida_ -> EfectosActivos())
					arboles_.Reproducir();

				partida_ -> ProcesarColision(Campo::ARBOLES);			
				return 0;
			}
	
			if(partida_ -> ComprobarColision(Partida::CENTRO, Campo::AGUA, 0, 0))
			{
				if(partida_ -> EfectosActivos())
					agua_.Reproducir();

				partida_ -> ProcesarColision(Campo::AGUA);			
				return 0;
			}
	
			if(partida_ -> ComprobarColision(Partida::CENTRO, Campo::BUNKER, 0, 0))
			{
				// En caso de que se produzca una colisión con el bunker guardamos
				// la posición de la bola y su sombra y a continuación procemos la
				// colisión y terminamos.
				if(partida_ -> EfectosActivos())
					bunker_.Reproducir();

				rbola_.x = rsombra_.x;
				rbola_.y = rsombra_.y - altura_minima_;

				x_bola_ = rbola_.x;
				y_bola_ = rbola_.y;

				x_sombra_ = rsombra_.x;
				y_sombra_ = rsombra_.y;

				partida_ -> ProcesarColision(Campo::BUNKER);			
				return 0;
			}
	
			if(partida_ -> ComprobarColision(Partida::COMPLETO, Campo::HOYO, 0, 0))
			{
				if(partida_ -> EfectosActivos())
					hoyo_.Reproducir();

				partida_ -> ProcesarColision(Campo::HOYO);			
				return 0;
			}	
	
			// Dibujamos todo.
			partida_ -> CentrarCamara();
			partida_ -> CampoActivo().Dibujar();
			partida_ -> DibujarBolasInactivas();
			this -> DibujarSombra();
			this -> DibujarBola();
			partida_ -> JugadorActivo().Dibujar();
			partida_ -> CampoActivo().DibujarBandera();
			partida_ -> DibujarInterfaz();
			pantalla_ -> Actualizar();

			// Guardamos el tiempo.						
			tb = SDL_GetTicks();

			// Si el tiempo transcurrido es mayor que el establecido en el juego
			// tenemos que esperar.		
			if(tb-ta < Constantes::_MS_)
				SDL_Delay(Constantes::_MS_ - (tb-ta));
		} // for
	} // if


/******************************************************************************/


	// Actualizamos la posición de la sombra.
	rsombra_.x = rbola_.x;
	rsombra_.y = rbola_.y + altura_minima_;
	
	// Dibujamos todo.
	partida_ -> CentrarCamara();
	partida_ -> CampoActivo().Dibujar();
	partida_ -> DibujarBolasInactivas();
	this -> DibujarSombra();
	this -> DibujarBola();
	partida_ -> JugadorActivo().Dibujar();
	partida_ -> CampoActivo().DibujarBandera();
	partida_ -> DibujarInterfaz();
	pantalla_ -> Actualizar();

	// Actualizamos las coordenadas iniciales.
	x_bola_ = rbola_.x;
	y_bola_ = rbola_.y;
	x_sombra_ = rsombra_.x;
	y_sombra_ = rsombra_.y;

	// No hubo colisión, así que la bola está activa.
	estado_ = ACTIVA;

	// Esperamos.
	partida_ -> Esperar(2000, true, true, false, false, false, 0, 0);

	// Terminamos.
	return 0;
}

const SDL_Rect& Bola::PosicionBola() const
{
	// Devolvemos la posición de la bola.
	return rbola_;
}

const SDL_Rect& Bola::PosicionSombra() const
{
	// Devolvemos la posición de la sombra.
	return rsombra_;
}

void Bola::RestaurarPosicion()
{
	// Decrementamos para eliminar el último incremento y volvemos a
	// decrementar para restaurar la posición.

	// Decrementamos los puntos generados.
	float i = desplazamiento_.puntos_generados
		- desplazamiento_.velocidad;

	// Decrementamos la curva.
	float j = desplazamiento_.curva
		- desplazamiento_.velocidad * desplazamiento_.incremento_curva;

	// Determinamos si las coordenadas iniciales eran menores.
	bool x_inicial_menor = desplazamiento_.rsombra_x <= rsombra_.x;
	bool y_inicial_menor = desplazamiento_.rsombra_y <= rsombra_.y;

	// Esta variable determina si se ha rebaso las coordenadas de inicio
	// del desplazamiento. En caso de que eso ocurra detendremos el bucle.
	bool rebasado = false;

	while(partida_ -> ComprobarColision(Partida::RESTAURACION,
	Campo::TODOS, 0, 0))
	{ 
		if(x_sombra_ < desplazamiento_.x)
			rsombra_.x = x_sombra_ + i * desplazamiento_.coseno_desvio;
		else
			rsombra_.x = x_sombra_ - i * desplazamiento_.coseno_desvio;

		if(y_sombra_ < desplazamiento_.y)
			if(desplazamiento_.angulo_de_desvio < desplazamiento_.angulo)
				rsombra_.y = y_sombra_ + i * desplazamiento_.seno_desvio -
					(desplazamiento_.hipotenusa / 168)
						* desplazamiento_.angulo_de_desvio * sin(j);
			else
				rsombra_.y = y_sombra_ + i * desplazamiento_.seno_desvio
					+ (desplazamiento_.hipotenusa / 168)
					* desplazamiento_.angulo_de_desvio * sin(j);				
		else
			rsombra_.y = y_sombra_ - i * desplazamiento_.seno_desvio
				- (desplazamiento_.hipotenusa / 168)
				* desplazamiento_.angulo_de_desvio * sin(j);
		
			rbola_.x = rsombra_.x;
			rbola_.y = rsombra_.y - altura_minima_;

			// Si hemos llegado a la posición original o la hemos rebasado, paramos.
			if((x_inicial_menor && rsombra_.x <= desplazamiento_.rsombra_x)
				|| (y_inicial_menor && rsombra_.y <= desplazamiento_.rsombra_y)
				|| (!x_inicial_menor && rsombra_.x >= desplazamiento_.rsombra_x)
				|| (!y_inicial_menor && rsombra_.y >= desplazamiento_.rsombra_y))
			{
				// Guardamos las coordenadas.
				rsombra_.x = desplazamiento_.rsombra_x;
				rsombra_.y = desplazamiento_.rsombra_y;
				
				// Actualizamos la variable de control.
				rebasado = true;
				
				// Salimos del bucle.
				break;			
			}
			
			// Decrementamos los puntos de desplazamiento generados.
			i -= 1;
			
			// Decrementamos la curva.
			j -= 1 * desplazamiento_.incremento_curva;
		
	}
	
	// Si hemos rebasado las coordendas iniciales y la bola no estaba en
	// un bunker, establecemos su estado a activa.
	if(rebasado)
	{
		if(estado_ != BUNKER)
			estado_ = ACTIVA;
	}
	else
	// En caso contrario, establecemos el estado a activa.
		estado_ = ACTIVA;
	
	// Actualizamos la posición de la sombra.
	rbola_.x = rsombra_.x;
	rbola_.y = rsombra_.y - altura_minima_;
	
	// Actualizamos las coordenadas iniciales.
	x_bola_ = rbola_.x;
	y_bola_ = rbola_.y;
	x_sombra_ = rsombra_.x;
	y_sombra_ = rsombra_.y;
}

int Bola::AlturaBola() const
{
	// Devolvemos la altura a la que se encuentra la bola.
	return rsombra_.y - rbola_.y;
}

int Bola::DistanciaHoyo(const SDL_Rect& hoyo) const
{
	// Rectángulo de distancia.
	float cateto_x, cateto_y, hipotenusa;
	
	// Calculamos los catetos.
	cateto_x = abs(hoyo.x - (rbola_.x + rbola_.w));
	cateto_y = abs(hoyo.y - (rbola_.y + rbola_.h));

	// Aplicamos el Teorema de Pitágoras.
	hipotenusa = sqrt(pow(cateto_x, 2) + pow(cateto_y, 2));
	
	// Devolvemos la distancia en relación al factor de conversión.
	return hipotenusa / Constantes::_FACTOR_DE_CONVERSION_;
}

int Bola::Ancho() const
{
	// Devolvemos el ancho de la imagen de la bola.
	return bola_ -> w;
}

int Bola::Alto() const
{
	// Devolvemos el alto de la imagen de la bola.
	return bola_ -> h;
}

void Bola::Posicion(const SDL_Rect& posicion)
{
	// Establecemos la nueva posición de la bola.
	rbola_.x = posicion.x;
	rbola_.y = posicion.y - altura_minima_;

	// Establecemos la nueva posición de la sombra.
	rsombra_.x = posicion.x;
	rsombra_.y = posicion.y;
	
	// Actualizamos las coordenadas iniciales de la bola.
	x_bola_ = rbola_.x;
	y_bola_ = rbola_.y;

	// Actualizamos las coordenadas iniciales de la sombra.	
	x_sombra_ = rsombra_.x;
	y_sombra_ = rsombra_.y;
	
}

int Bola::Estado() const
{
	// Devolvemos el estado actual de la bola.
	return estado_;
}
void Bola::Estado(int estado)
{
	// Establecemos el estado de la bola.
	estado_ = estado;
}

void Bola::Color(int color)
{
	// Seleccionamos el cuadro de la bola inactiva en función del color.
	inactivas_ -> SeleccionarCuadro(color);
}

std::string Bola::RutaImagen(const char *imagen)
{
	// Creamos la ruta a la imagen.
	std::string ruta = Constantes::_IMG_BOLA_;
	std::string img = imagen;
	
	// La devolvemos.
	return ruta += img;
}

std::string Bola::RutaSonido(const char *sonido)
{
	// Creamos la ruta al efecto de sonido.
	std::string ruta = Constantes::_SND_EFECTOS_;
	std::string snd = sonido;
	
	// La devolvemos.
	return ruta += snd;
}
