/// \file bola.h Implementación de la bola de golf.
/*
	Copyright 2010 Andrés Francisco Aparicio

	This file is part of SDLgolf.

	SDLgolf is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	SDLgolf is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with SDLgolf.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef _BOLA_H_
#define _BOLA_H_

#include <SDL/SDL.h>
#include "sonido.h"
#include "datos.h"

class Palo;
class Partida;
class Pantalla;
class Rejilla;

/*! \class Bola
    \brief Bola de golf.

    Esta es una de las clases más importantes del juego.
    
    Se encarga de la
    implementación de la física de la bola de golf y realiza todos los
    cálculos necesarios para simular el desplazamiento de dicha bola
    sobre la superficie de juego.
 */
class Bola 
{
	public:
		
		/*! Constructor de la bola de golf.\n
				Realiza las asociaciones con la Pantalla y la Partida. Se cargan las
				imágenes de la bola, la sombra y la Rejilla de bolas inactivas.\n
				De la misma forma, se cargan los efectos de Sonido que produce la bola
				al caer al agua, al golpear contra el bunker o al caer en los
				árboles.\n
				El estado inicial de la bola es inactivo.
				
				\brief Constructor.
				\param partida Partida en curso.
				\param pantalla Pantalla donde se dibujará la bola.
				\see Estados.
		 */
		Bola(Partida *partida, Pantalla *pantalla);
		
		/*! Destructor de la bola.\n
				Destruye las imágenes y la Rejilla de bolas inactivas.
				
				\brief Destructor.
		 */
		~Bola();
		
		/*! Realiza el desplazamiento de la bola hasta las coordenadas (x, y).\n
				La bola llegará hasta dichas coordenadas y si tiene suficiente
				inercia rebotará o rodará.
				
				\brief Desplaza la bola.
				\param x Coordenada x hasta la que se desplazará la bola.
				\param y Coordenada y hasta la que se desplazará la bola.
				\return \b 0 cuando el lanzamiento ha finalizado.
		 */
		int Desplazar(int x, int y);

		/*! Calcula la altura a la que se encuentra la bola. La altura viene dada
				por la diferencia en el <em> eje de las y </em> entre la imagen
				de la bola y su sombra.
				
				\brief Altura de la bola.
				\return La altura a la que se encuentra la bola con respecto al suelo.
				\see PosicionBola.
				\see PosicionSombra.
		 */
		int AlturaBola() const;
		
		/*! Ancho de la imagen de la bola.
				\brief Ancho de la bola.
				\return El ancho de la imagen de la bola.
		 */
		int Ancho() const;
		
		/*! Alto de la imagen de la bola.
				\brief Alto de la bola.
				\return El alto de la imagen de la Bola.
		 */
		int Alto() const;
		
		/*! La función calcula la distancia euclídea entre la posición de la bola
				y el hoyo del Campo.
				\brief Distancia al hoyo.
				\param hoyo Localización del hoyo.
				\return La distancia en línea recta que hay desde la bola al hoyo.
				\see PosicionBola
				\see Campo::PosicionHoyo
		 */		
		int DistanciaHoyo(const SDL_Rect& hoyo) const;
		
		/*! \brief Rectángulo de posición de la bola.
				\return Rectángulo de posición de la bola.
		 */
		const SDL_Rect& PosicionBola() const;

		/*! \brief Rectángulo de posición de la sombra.
				\return Rectángulo de posición de la sombra.
		 */
		const SDL_Rect& PosicionSombra() const;

		/*! Establece una nueva posición para la bola.
				\brief Cambiar posición.
				\param posicion Nueva posición de la bola.
		 */	
		void Posicion(const SDL_Rect& posicion);
		
		/*! Restaura la posición de la bola a una zona libre de colisiones.
				\brief Restaurar a zona sin colisiones.
				\see Campo::ElementosCampo.
				\see Partida::ComprobarColision.
		 */			
		void RestaurarPosicion();
		
		/*! Dibuja la bola en la Pantalla.
				\brief Dibuja la bola.
				\see Pantalla::Dibujar.
		 */			
		void DibujarBola() const;

		/*! Dibuja la sombra en la Pantalla.
				\brief Dibuja la sombra.
				\see Pantalla::Dibujar.
		 */	
		void DibujarSombra() const;


		/*! \enum Estados
		  	Estados en los que puede encontrarse la bola.
		  	
		  	\brief Estados de la bola.

				\var ACTIVA
		 		La bola está actualmente en juego.

				\var INACTIVA
		 		La bola está a la espera de entrar en juego.
		 		
		 		\var BUNKER
		 		La bola está \e ACTIVA y además se encuentra en un bunker.
		 		
		 		\var BUNKER_INACTIVA
				La bola está \e INACTIVA y además se encuentra en un bunker.
				
		 		\var FINALIZADA
				La bola ha entrado en el hoyo del campo.
		 */
		
		enum Estados {ACTIVA, INACTIVA, BUNKER, BUNKER_INACTIVA, FINALIZADA};


		/*! \brief Estado de la bola.
				\return Estado actual de la bola.
				\see Estados.
		 */
		int Estado() const;

		/*! Establece un nuevo estado para la bola.
				\brief Cambia el estado.
				\param estado Estado que pasará a tener la bola.
				\see Estados.
		 */
		void Estado(int estado);

		/*! Establece el color de la bola inactiva.
				\brief Cambia el color.
				\param color Color que tendrá la bola inactiva.
		 */		
		void Color(int color);
		
	private:

		/*! Genera una ruta desde el directorio de imágenes de la bola.
				\brief Genera la ruta a una imagen.
				\param imagen Nombre de la imagen.
				\return Ruta a la imagen.
		 */				
		std::string RutaImagen(const char *imagen);

		/*! Genera una ruta a un sonido desde el directorio de efectos de sonido.
				\brief Genera ruta a sonido.
				\param imagen Nombre del sonido.
				\return Ruta al sonido.
		 */			
		std::string RutaSonido(const char *sonido);
	
		static int altura_minima_; ///< Altura mínima que puede tener la Bola
															 ///< sobre su sombra.

		int estado_; ///< Estado actual de la bola.
		
		Sonido rebote_, ///< Sonido que genera la Bola al golpear en el Campo.
					 golpe_, ///< Sonido que genera la Bola al ser golpeada por un Palo.
					 agua_, ///< Sonido que genera la Bola al caer al agua del Campo.
					 bunker_, ///< Sonido que genera la Bola al caer al bunker.
					 arboles_, ///< Sonido que genera la Bola al caer a los árboles
					 hoyo_; ///< Sonido que genera la Bola al entrar en el hoyo.

		Datos::Bola desplazamiento_;	///< Datos necesarios para restaurar la
																	///< bola a una zona libre de colisión.
	
		SDL_Surface *bola_, ///< Imagen de la Bola.
								*sombra_; ///< Imagen de la sombra de la Bola.
		
		Rejilla *inactivas_; ///< Imágenes de la Bola cuando está inactiva.
		
		SDL_Rect rbola_, ///< Rectángulo de posición de la Bola.
						 rsombra_; ///< Rectángulo de posición de la sombra de la Bola.
		
		int x_bola_, ///< Posición inicial de la Bola en el eje de abscisas.
				y_bola_; ///< Posición inicial de la Bola en el eje de ordenadas.
		
		int x_sombra_, ///< Posición inicial la sombra de la Bola en el
									 ///< eje de abscisas.
				y_sombra_; ///< Posición inicial de la sombra de la Bola en
									 ///< el eje de ordenadas.
		
		Partida *partida_; ///< Partida que está actualmente en juego.
		
		Pantalla *pantalla_; ///< Pantalla del Juego.
		
};

#endif
