/// \file campo.cpp Campo de juego.
/*
	Copyright 2010 Andrés Francisco Aparicio

	This file is part of SDLgolf.

	SDLgolf is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	SDLgolf is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with SDLgolf.  If not, see <http://www.gnu.org/licenses/>.
*/

#include <SDL/SDL.h>
#include <SDL/SDL_image.h>
#include <iostream>
#include <fstream>
#include "tinyxml.h"
#include "campo.h"
#include "utilidades.h"
#include "datos.h"
#include "camara.h"
#include "pantalla.h"
#include "partida.h"
#include "rejilla.h"
#include "animacion.h"
#include "constantes.h"


Campo::Campo(std::string nombre, std::string hoyo, std::string par,
	SDL_Rect* posicion_salida, SDL_Rect* posicion_hoyo,
	std::string colores, std::string ruta, Pantalla *pantalla)
{
	// Establecemos el nombre del campo.
	nombre_ = nombre;

	// Guardamos la ruta del campo.
	ruta_ = ruta;

	// Establecemos el número de hoyo.
	numero_hoyo_ = hoyo;

	// Establecemos el par del campo.
	par_ = par;

	// Asociamos con la pantalla.
	pantalla_ = pantalla;
	
	// Generamos la ruta de la imagen del campo.
	std::string tmp = Constantes::_IMG_CAMPO_ + ruta + "/" + hoyo + ".png";
	
	// Cargamos la imagen de la ruta del campo.
	campo_ = Utilidades::CargarImagen(tmp);
	
	// Establecemos las coordenadas.
	rcampo_.x = 0;
	rcampo_.y = 0;

	// Guardamos el ancho y el alto de la imagen.
	rcampo_.w = campo_ -> w;
	rcampo_.h = campo_ -> h;

	// Determinamos las coordenadas para la posición de salida.
	rsalida_.x = posicion_salida -> x;
	rsalida_.y = posicion_salida -> y;

	// Establecemos las coordenadas del hoyo.
	rhoyo_.x = posicion_hoyo -> x;
	rhoyo_.y = posicion_hoyo -> y;

	// Documento xml.
	TiXmlDocument colores_xml;

	// Cargamos el fichero de colores de colisión.
	std::string ruta_colores = "./cfg/" + colores;
	if(!colores_xml.LoadFile(ruta_colores))
	{
		std::cerr<< "Campo::Campo() Error: No se puede abrir el fichero"
			<< " de colores." << std::endl;
		exit(1);
	}
	
	// Establecemos la raiz del documento.
	TiXmlElement *raiz = colores_xml.RootElement();

	// Establecemos los elementos del campo como hijos de la raiz.
	TiXmlElement* elementos = raiz -> FirstChildElement();

	// Establecemos el color como hijo del elemento.
	TiXmlElement* color =
		elementos -> FirstChildElement();
	
	// Nombre del elemento a procesar.
	std::string elemento = elementos -> Attribute("nombre");
	
	// Recorremos todos los elementos.
	do
	{
		// Recorremos todos los colores.
		do
		{
			// Modelo de color RGB.
			int r, g, b;
			
			// Leemos el valor del color rojo.
			r = atoi(color -> Attribute("r"));

			// Leemos el valor del color verde.
			g = atoi(color -> Attribute("g"));
			
			// Leemos el valor del color blanco.
			b = atoi(color -> Attribute("b"));
			
			// Añadimos el color de colisión del agua.
			if(elemento == "agua")
				agua_.push_back(SDL_MapRGB(campo_ -> format, r, g, b));
			
			// Añadimos el color de colisión de los árboles.
			if(elemento == "arboles")
				arboles_.push_back(SDL_MapRGB(campo_ -> format, r, g, b));

			// Añadimos el color de colisión del bunker.
			if(elemento == "bunker")
				bunker_.push_back(SDL_MapRGB(campo_ -> format, r, g, b));

			// Añadimos el color de colisión del hoyo.				
			if(elemento == "hoyo")
				hoyo_.push_back(SDL_MapRGB(campo_ -> format, r, g, b));
			
			// Seleccionamos el siguiente color.
			color = color -> NextSiblingElement();
		}
		while(color);
		
		// Seleccionamos el siguiente elemento.
		elementos = elementos -> NextSiblingElement();
		
		// Hay más elementos.
		if(elementos != 0)
		{
			// Seleccionamos el primer color.
			color = elementos -> FirstChildElement();
			
			// Establecemos el nombre del elemento.
			elemento = elementos -> Attribute("nombre");
		}
		
	}
	while(elementos);
	
	// Insertamos los colores de colisión en el map.
	elementos_.insert(std::pair< int, std::vector<Uint32> >(ARBOLES, arboles_));
	elementos_.insert(std::pair< int, std::vector<Uint32> >(AGUA, agua_));
	elementos_.insert(std::pair< int, std::vector<Uint32> >(BUNKER, bunker_));
	elementos_.insert(std::pair< int, std::vector<Uint32> >(HOYO, hoyo_));
	
	// Creamos la ruta a la rejilla de la bandera.
	tmp.clear();
	tmp.append(Constantes::_IMG_ANIMACIONES_);
	tmp += "bandera.png";
	
	// Cargamos la rejilla de la bandera.
	rejilla_ = new Rejilla(tmp, 1, 4);
	
	// Generamos la secuencia de la animación.
	int secuencia[] = {0, 1, 2, 3, 1, 0, 2, 3, -1};
	
	// Creamos la animación.
	bandera_ = new Animacion(rejilla_, secuencia, 100, pantalla_);
	
	// Establecemos el ancho y el alto de la bandera.
	rbandera_.w = rejilla_ -> Posicion().w;
	rbandera_.h = rejilla_ -> Posicion().h;
	
	// Establecemos sus coordenadas.
	rbandera_.x = rhoyo_.x - 4;
	rbandera_.y = rhoyo_.y - 35;
	
	// Actualizamos su posición.
	bandera_ -> Posicion(rbandera_);
	
	// La bandera se dibuja al iniciar el campo.
	dibujar_bandera_ = true;
	
	// Guardamos el tiempo para el control de la animación.
	t_bandera = SDL_GetTicks();
	
}

Campo::~Campo()
{
	// Destruimos la imagen del campo.
	SDL_FreeSurface(campo_);
	delete rejilla_;
	delete bandera_;
}

const SDL_Rect& Campo::PosicionHoyo() const
{
	// Devolvemos la posición del hoyo.
	return rhoyo_;
}

const std::map< int, std::vector<Uint32> > & Campo::Elementos() const
{
	// Devolvemos los elementos de colisión.
	return elementos_;
}

int Campo::Ancho() const
{
	// Devolvemos el ancho del campo.
	return campo_ -> w;
}

int Campo::Alto() const
{
	// Devolvemos el alto del campo.
	return campo_ -> h; 
}

std::string Campo::Nombre() const
{
	// Devolvemos el nombre del campo.
	return nombre_;
}

std::string Campo::Ruta() const
{
	// Devolvemos la ruta en la que se encuentra la imagen del campo.
	return ruta_;
}

std::string Campo::NumeroHoyo() const
{
	// Devolvemos el número de hoyo.
	return numero_hoyo_;
}

int Campo::Par() const
{
	// Devolvemos el par del campo.
	return atoi(par_.c_str());
}

const SDL_Rect& Campo::LugarSalida() const
{
	// Devolvemos la posición de salida del campo.
	return rsalida_;
}

void Campo::Dibujar()
{
	// Dibujamos el campo.
	pantalla_ -> Dibujar(campo_);
}

Uint32 Campo::Pixel(int x, int y) const
{
	// Devolvemos el color del pixel (x, y) de la superficie del campo.
	return Utilidades::ObtenerPixel(x, y, campo_);
}

const std::vector<Uint32>& Campo::Arboles() const
{
	// Devolvemos los colores de colisión de los árboles.
	return arboles_;
}
const std::vector<Uint32>& Campo::Agua() const
{
	// Devolvemos los colores de colisión del agua.
	return agua_;
}

const std::vector<Uint32>& Campo::Bunker() const
{
	// Devolvemos los colores de colisión del bunker.
	return bunker_;
}

const std::vector<Uint32>& Campo::Hoyo() const
{
	// Devolvemos los colores de colisión del hoyo.
	return hoyo_;
}

const SDL_Rect& Campo::Posicion() const
{
	// Devolvemos la posición del campo.
	return rcampo_;
}

void Campo::DibujarBandera()
{
	if(dibujar_bandera_)
	{
		if(SDL_GetTicks() - t_bandera > bandera_ -> Retardo())
		{
			//pantalla_ -> Dibujar(campo_, &rbandera_, &rbandera_);
			bandera_ -> Dibujar();
			//pantalla_ -> ActualizarPosicion(&rbandera_);
			//pantalla_ -> Actualizar();
			t_bandera = SDL_GetTicks();
		}
		else
			bandera_ -> DibujarCuadro();
	}
}

void Campo::DesactivarBandera()
{
	dibujar_bandera_ = false;
}
