/// \file campo.h Campo de juego.
/*
	Copyright 2010 Andrés Francisco Aparicio

	This file is part of SDLgolf.

	SDLgolf is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	SDLgolf is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with SDLgolf.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef _CAMPO_H_
#define _CAMPO_H_

#include <SDL/SDL.h>
#include <SDL/SDL_image.h>
#include <vector>
#include <map>
#include "datos.h"


class Pantalla;
class Rejilla;
class Animacion;

/*! \class Campo
    \brief Campo de golf.

    El campo de golf es la superficie en la que se desarrollará la partida.
    
    Está compuesto por diferentes elementos con los que puede colisionar la
    bola de golf.
    
    Todo campo de golf tiene una posición de salida para los
    jugadores y un hoyo en el que habrá que colar la bola.
 */
class Campo
{
	public:
	
		/*!	Establece todos los atributos relacionados con el cambo (nombre,
				número de hoyo, par del campo...) y carga los colores de colisión
				de los diferentes elementos del campo.\n
				De igual forma, se crea la animación de la bandera.
				
				\brief Constructor.
				\param nombre Nombre del campo.
				\param hoyo Número de hoyo del campo.
				\param par Par del hoyo.
				\param posicion_salida Posición de salida de los jugadores.
				\param posicion_hoyo Posición en la que se encuentra el hoyo.
				\param colores Nombre del fichero XML que contiene los colores
							 de colisión.
				\param ruta Ruta a las imágenes del campo.
				\param pantalla Pantalla del Juego.
				\see ElementosCampo.
				\see DibujarBandera.
		 */
		Campo(std::string nombre, std::string hoyo, std::string par,
			SDL_Rect* posicion_salida, SDL_Rect* posicion_hoyo, std::string colores,
			std::string ruta, Pantalla *pantalla);

		/*! Destruye la imagen del campo, la rejilla de imágenes de la bandera y
				la animación de la bandera.
				\brief Destructor.
		 */
		~Campo();

		/*! \brief Posición del hoyo.
				\return Posición del hoyo del campo.
		 */
		const SDL_Rect& PosicionHoyo() const;

		/*! \brief Posición de salida.
				\return Posición de salida para los jugadores.
		 */
		const SDL_Rect& LugarSalida() const;

		/*! \brief Posición.
				\return Posición del campo.
		 */
		const SDL_Rect& Posicion() const;

		/*! \brief Elementos.
				\return Elementos que componen el campo y con los que puede colisionar
								la Bola.
				\see Partida::ComprobarColision.
				\see ElementosCampo.
		 */
		const std::map< int, std::vector<Uint32> >& Elementos() const;
		
		/*! \brief Ancho del terreno.
				\return Ancho de la superficie del campo.
		 */
		int Ancho() const;

		/*! \brief Alto del terreno.
				\return Alto de la superficie del campo.
		 */
		int Alto() const;

		/*! \brief Ruta a las imágenes.
				\return Ruta a las imágenes del campo.
		 */
		std::string Ruta() const;

		/*! \brief Nombre del campo.
				\return Nombre del campo.
		 */
		std::string Nombre() const;

		/*! \brief Número de hoyo.
				\return Número de hoyo del campo.
		 */
		std::string NumeroHoyo() const;

		/*! \brief Par del hoyo.
				\return Par del hoyo.
		 */
		int Par() const;

		/*! Dibuja el campo en la Pantalla.
				\brief Dibujar campo.
		 */
		void Dibujar();
		
		/*!	Obtiene el color de un determinado píxel.
				\brief Obtener Píxel.
				\param x Coordenada x del píxel.
				\param y Coordenada y del píxel.
				\return Color del Píxel en la posición (x, y) de la superficie
								del Campo.
		 */		
		Uint32 Pixel(int x, int y) const;
		
		/*! \brief Árboles.
				\return Colores de los árboles.
				\see ElementosCampo.
		 */		
		const std::vector<Uint32>& Arboles() const;

		/*! \brief Agua.
				\return Colores del agua.
				\see ElementosCampo.
		 */
		const std::vector<Uint32>& Agua() const;

		/*! \brief Bunker.
				\return Colores del bunker.
				\see ElementosCampo.
		 */
		const std::vector<Uint32>& Bunker() const;

		/*! \brief Hoyo.
				\return Colores del hoyo.
				\see ElementosCampo.
		 */
		const std::vector<Uint32>& Hoyo() const;

		/*! \enum ElementosCampo
		  	Elementos que componen el campo de golf con los que puede colisionar
		  	la Bola.
		  	
		  	\brief Elementos de colisión del campo.

				\var ARBOLES
		 		Árboles presentes en el terreno de juego.

				\var AGUA
		 		Zonas de agua.
		 		
		 		\var BUNKER
		 		Zonas de bunker.
		 		
		 		\var HOYO
				Hoyo del campo.
				
		 		\var TODOS
				Todos los elementos.
				
		 		\var FDL
				Fuera de límites.
		 */
		enum ElementosCampo { ARBOLES, AGUA, BUNKER, HOYO, TODOS, FDL};


		/*! Dibuja la animación de la bandera.
				\brief Dibuja la bandera.
		 */
		void DibujarBandera();

		/*! Desactiva la bandera para evitar que se siga dibujando.
				\brief Quita la bandera.
		 */
		void DesactivarBandera();
			
	private:
		std::string nombre_;	///< Nombre del Campo.
		
		std::string ruta_;	///< Ruta a las imágenes del Campo.
		
		std::string numero_hoyo_; ///< Número de hoyo.
		
		std::string par_; ///< Par del Campo.

		SDL_Surface *campo_;	///< Imagen de la superficie del Campo.

		SDL_Rect rcampo_;	///< Rectángulo de posición de la imagen del Campo.

		SDL_Rect rhoyo_;	///< Rectángulo de posición del hoyo.
		
		SDL_Rect rsalida_;	///< Rectángulo de posición de la salida de los
												///< jugadores.

		std::map< int, std::vector<Uint32> > elementos_; ///< Elementos del Campo.

		std::vector<Uint32> arboles_;	///< Colores de los árboles.
		
		std::vector<Uint32> agua_;	///< Colores del agua.
		
		std::vector<Uint32> bunker_;	///< Colores del bunker.
		
		std::vector<Uint32> hoyo_;	///< Colores del hoyo.

		Pantalla *pantalla_;	///< Pantalla del Juego.
		
		Uint32 t_bandera;	///< Control de tiempo para la animación de la bandera.
		
		Rejilla *rejilla_;	///< Rejilla de imágenes de la bandera.
		
		Animacion *bandera_;	///< Animación de la bandera.
		
		SDL_Rect rbandera_;	///< Posición de la bandera.
		
		bool dibujar_bandera_;	///< Variable de control para el dibujo de
														///< la bandera.
};

#endif
