/// \file cursor.cpp Cursor de dirección de la bola.
/*
	Copyright 2010 Andrés Francisco Aparicio

	This file is part of SDLgolf.

	SDLgolf is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	SDLgolf is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with SDLgolf.  If not, see <http://www.gnu.org/licenses/>.
*/

#include <SDL/SDL.h>
#include <math.h>
#include <iostream>
#include "cursor.h"
#include "utilidades.h"
#include "pantalla.h"
#include "jugador.h"
#include "bola.h"
#include "campo.h"
#include "constantes.h"

// Establecemos la velocidad del cursor.
float Cursor::velocidad_ = 2;

// Establecemos el radio del cursor.
float Cursor::radio_ = 70;

// Establecemos el incremento que se producirá al mover el cursor.
float Cursor::incremento_ = M_PI / 180;

// Establecemos el ángulo completo.
float Cursor::angulo_completo_ = M_PI * 2;

// Establecemos el ángulo nulo.
float Cursor::angulo_nulo_ = 0;

Cursor::Cursor(Jugador *jugador, Pantalla *pantalla)
{
	// Creamos la ruta a la imagen del cursor.
	std::string tmp;
	tmp.append(Constantes::_IMG_INTERFAZ_);
	tmp += "cursor.png";
	
	// Cargamos la imagen del cursor.
	cursor_ = Utilidades::CargarImagen(tmp);
	
	// Asociamos con el jugador.
	jugador_ = jugador;

	// Asociamos con la pantalla;
	pantalla_ = pantalla;
	
	// Guardamos la anchura y la altura del cursor.
	rcursor_.w = cursor_ -> w;
	rcursor_.h = cursor_ -> h;

	// Establecemos las coordenadas iniciales del cursor.
	rcursor_.x = 0;
	rcursor_.y = 0;
}

Cursor::~Cursor()
{
	// Destruimos la imagen del cursor.
	SDL_FreeSurface(cursor_);
}

void Cursor::Orientar()
{
	// Catetos del triángulo de posición del cursor.
	float cateto_x;
	float cateto_y;
	
	// Bola activa.
	Bola& b = jugador_ -> BolaEnJuego();

	// Posición del hoyo.
	const SDL_Rect& hoyo = jugador_ -> CampoActual().PosicionHoyo();
	
	// La bola está a la izquierda del hoyo.
	if(b.PosicionBola().x + b.Ancho() / 2 < hoyo.x)
		cateto_x = hoyo.x - (b.PosicionBola().x + b.Ancho() / 2);
	// La bola está a la derecha del hoyo.
	else
		cateto_x = (b.PosicionBola().x + b.Ancho() / 2) - hoyo.x ;

	// La bola está a abajo del hoyo.
	if(b.PosicionBola().y + b.Alto() / 2 > hoyo.y)
		cateto_y = (b.PosicionBola().y + b.Alto() / 2) - hoyo.y;
	// La bola está a arriba del hoyo.
	else
		cateto_y = hoyo.y - (b.PosicionBola().y + b.Alto() / 2);

	// El ángulo del cursor será igual al que forma la bola con el hoyo es
	// igual a la arcotangente del cateto opuesto partido del cateto contiguo.
	angulo_ = atan(cateto_y / cateto_x);
	
	// La posición del cursor en el eje de abcisas es igual al producto del
	// coseno del ángulo por el radio.
	px_ = radio_ * cos(angulo_);

	// La posición del cursor en el eje de ordenadas es igual al producto del
	// seno del ángulo por el radio.
	py_ = radio_ * sin(angulo_);

	// La bola está a la izquierda del hoyo. Establecemos la posición del cursor
	// para el eje de las x.
	if(b.PosicionBola().x + b.Ancho() / 2 < hoyo.x)
		rcursor_.x = b.PosicionBola().x + b.Ancho() / 2 + px_ - cursor_ -> w / 2;
	// La bola está a la derecha del hoyo.
	else
		rcursor_.x = b.PosicionBola().x + b.Ancho() / 2 - px_ - cursor_ -> w / 2;
	
	// La bola está abajo del hoyo. Establecemos la posición del cursor
	// para el eje de las y.	
	if(b.PosicionBola().y + b.Alto() / 2  > hoyo.y)
		rcursor_.y = b.PosicionBola().y + b.Alto() / 2 - py_ - cursor_ -> h / 2;
	// La bola está a arriba del hoyo.
	else
		rcursor_.y = b.PosicionBola().y + b.Alto() / 2 + py_ - cursor_ -> h / 2;
}

void Cursor::Decrementar()
{
	// Referencia a la bola activa.
	Bola& b = jugador_ -> BolaEnJuego();

	// Posición del hoyo.
	const SDL_Rect& hoyo = jugador_ -> CampoActual().PosicionHoyo();
	
	// Decrementamos el ángulo.
	angulo_ -= velocidad_ * incremento_;
	
	// Si llegamos a 0 grados, establecemos un ángulo de 360 grados.
	if(angulo_ <= angulo_nulo_)
		angulo_ = angulo_completo_ - (1 * M_PI / 180);

	// La posición del cursor en el eje de abcisas es igual al producto del
	// coseno del ángulo por el radio.
	px_ = radio_ * cos(angulo_);

	// La posición del cursor en el eje de ordenadas es igual al producto del
	// seno del ángulo por el radio.
	py_ = radio_ * sin(angulo_);

	// La bola está a la izquierda del hoyo. Establecemos la posición del cursor
	// para el eje de las x.
	if(b.PosicionBola().x + b.Ancho() / 2 < hoyo.x)
		rcursor_.x = b.PosicionBola().x + b.Ancho() / 2 + px_ - cursor_ -> w / 2;
	// La bola está a la derecha del hoyo.
	else
		rcursor_.x = b.PosicionBola().x + b.Ancho() / 2 - px_ - cursor_ -> w / 2;
	
	// La bola está abajo del hoyo. Establecemos la posición del cursor
	// para el eje de las y.		
	if(b.PosicionBola().y + b.Alto() / 2  > hoyo.y)
		rcursor_.y = b.PosicionBola().y + b.Alto() / 2 - py_ - cursor_ -> h / 2;
	// La bola está a arriba del hoyo.
	else
		rcursor_.y = b.PosicionBola().y + b.Alto() / 2 + py_ - cursor_ -> h / 2;
}

void Cursor::Incrementar()
{
	// Referencia a la bola activa.
	Bola& b = jugador_ -> BolaEnJuego();

	// Posición del hoyo.
	const SDL_Rect& hoyo = jugador_ -> CampoActual().PosicionHoyo();

	// Incrementamos el ángulo.
	angulo_ += velocidad_ * incremento_;
		
	// Si llegamos a 360 grados, establecemos un ángulo de 0 grados.
	if(angulo_ >= angulo_completo_)
		angulo_ = angulo_nulo_ + (1 * M_PI / 180);
		
	// La posición del cursor en el eje de abcisas es igual al producto del
	// coseno del ángulo por el radio.
	px_ = radio_ * cos(angulo_);

	// La posición del cursor en el eje de ordenadas es igual al producto del
	// seno del ángulo por el radio.
	py_ = radio_ * sin(angulo_);

	// La bola está a la izquierda del hoyo. Establecemos la posición del cursor
	// para el eje de las x.
	if(b.PosicionBola().x + b.Ancho() / 2 < hoyo.x)
		rcursor_.x = b.PosicionBola().x + b.Ancho() / 2 + px_ - cursor_ -> w / 2;
	// La bola está a la derecha del hoyo.
	else
		rcursor_.x = b.PosicionBola().x + b.Ancho() / 2 - px_ - cursor_ -> w / 2;
	
	// La bola está abajo del hoyo. Establecemos la posición del cursor
	// para el eje de las y.	
	if(b.PosicionBola().y + b.Alto() / 2  > hoyo.y)
		rcursor_.y = b.PosicionBola().y + b.Alto() / 2 - py_ - cursor_ -> h / 2;
	else
	// La bola está a arriba del hoyo.
		rcursor_.y = b.PosicionBola().y + b.Alto() / 2 + py_ - cursor_ -> h / 2;
}

void Cursor::Derecha()
{
	// Referencia a la bola activa.
	Bola& b = jugador_ -> BolaEnJuego();

	// Posición del hoyo.
	const SDL_Rect& hoyo = jugador_ -> CampoActual().PosicionHoyo();

	// La bola está a la izquierda del hoyo.	
	if(b.PosicionBola().x + b.Ancho() / 2 < hoyo.x)
		// La bola está abajo del hoyo.
		if(b.PosicionBola().y + b.Alto() / 2  > hoyo.y)
			Decrementar();
		// La bola está arriba del hoyo.
		else
			Incrementar();
	// La bola está a la derecha del hoyo.
	else
		// La bola está abajo del hoyo.
		if(b.PosicionBola().y + b.Alto() / 2  > hoyo.y)
			Incrementar();
		// La bola está arriba del hoyo.
		else
			Decrementar();
}

void Cursor::Izquierda()
{
	// Referencia a la bola activa.
	Bola& b = jugador_ -> BolaEnJuego();

	// Posición del hoyo.
	const SDL_Rect& hoyo = jugador_ -> CampoActual().PosicionHoyo();

	// La bola está a la izquierda del hoyo.	
	if(b.PosicionBola().x + b.Ancho() / 2 < hoyo.x)
		// La bola está abajo del hoyo.
		if(b.PosicionBola().y + b.Alto() / 2  > hoyo.y)
			Incrementar();
		// La bola está arriba del hoyo.
		else
			Decrementar();
	// La bola está a la derecha del hoyo.
	else
		// La bola está abajo del hoyo.
		if(b.PosicionBola().y + b.Alto() / 2  > hoyo.y)
			Decrementar();
		// La bola está arriba del hoyo.
		else
			Incrementar();
}

const SDL_Rect& Cursor::Posicion() const
{
	// Devolvemos la posición del cursor.
	return rcursor_;
}

float Cursor::Angulo() const
{
	// Devolvemos el ángulo que forma el cursor con la bola.

	if(angulo_ > 0 && angulo_ <= M_PI / 2)
		return angulo_;
	else if(angulo_ > M_PI / 2 && angulo_ <= M_PI)
		return M_PI - angulo_;
	else if(angulo_ > M_PI && angulo_ <= M_PI * 3 / 2)
		return angulo_ - M_PI;
	else if(angulo_ > M_PI * 3 / 2 && angulo_ <= 2 * M_PI)
		return 2 * M_PI - angulo_;
	else
		return angulo_;
}

void Cursor::Dibujar() const
{
	// Dibujamos el cursor.
	pantalla_ -> Dibujar(cursor_, &rcursor_);		
}

int Cursor::Cuadrante() const
{
	// Referencia a la bola activa.
	Bola& b = jugador_ -> BolaEnJuego();

	// Posición del hoyo.
	const SDL_Rect& hoyo = jugador_ -> CampoActual().PosicionHoyo();

	// La bola está a la izquierda del hoyo.
	if(b.PosicionBola().x + b.Ancho() / 2 < hoyo.x)
	{
		// La bola está abajo del hoyo.
		if(b.PosicionBola().y + b.Alto() / 2  > hoyo.y)
		{
			if(angulo_ >= 0 && angulo_ <= M_PI / 2)
				return 1;
			else if(angulo_ >= M_PI / 2 && angulo_ <= M_PI)
				return 2;
			else if(angulo_ >= M_PI && angulo_ <= M_PI * 3 / 2)
				return 3;
			else if(angulo_ >= M_PI * 3 / 2 && angulo_ <= 2 * M_PI)
				return 4;
			else
				return 0;	
		}
		// La bola está arriba del hoyo.
		else
		{
			if(angulo_ >= 0 && angulo_ <= M_PI / 2)
				return 4;
			else if(angulo_ >= M_PI / 2 && angulo_ <= M_PI)
				return 3;
			else if(angulo_ >= M_PI && angulo_ <= M_PI * 3 / 2)
				return 2;
			else if(angulo_ >= M_PI * 3 / 2 && angulo_ <= 2 * M_PI)
				return 1;
			else
				return 0;	
		}
	}
	// La bola ésta a la derecha del hoyo.
	else
	{
		// La bola está abajo del hoyo.
		if(b.PosicionBola().y + b.Alto() / 2 > hoyo.y)
		{
			if(angulo_ >= 0 && angulo_ <= M_PI / 2)
				return 2;
			else if(angulo_ >= M_PI / 2 && angulo_ <= M_PI)
				return 1;
			else if(angulo_ >= M_PI && angulo_ <= M_PI * 3 / 2)
				return 4;
			else if(angulo_ >= M_PI * 3 / 2 && angulo_ <= 2 * M_PI)
				return 3;
			else
				return 0;
		}
		// La bola está arriba del hoyo.
		else
		{
			if(angulo_ >= 0 && angulo_ <= M_PI / 2)
				return 3;
			else if(angulo_ >= M_PI / 2 && angulo_ <= M_PI)
				return 4;
			else if(angulo_ >= M_PI && angulo_ <= M_PI * 3 / 2)
				return 1;
			else if(angulo_ >= M_PI * 3 / 2 && angulo_ <= 2 * M_PI)
				return 2;
			else
				return 0;
		}
	}
}

