/// \file cursor.h Cursor de dirección la bola.
/*
	Copyright 2010 Andrés Francisco Aparicio

	This file is part of SDLgolf.

	SDLgolf is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	SDLgolf is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with SDLgolf.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef _CURSOR_H_
#define _CURSOR_H_

#include <SDL/SDL_image.h>

class Jugador;
class Pantalla;

/** \class Cursor
    \brief Cursor de dirección.

    El cursor de dirección permite al jugador visualizar hacia dónde está
    apuntando.
    
    El jugador podrá establecer la dirección que desee girando
    el cursor hacia la derecha o hacia la izquierda.
    
    La dirección
    que indique el cursor será la que tome la bola de golf cuando sea
    golpeada.
*/
class Cursor
{
	public:

		/*! Realiza las asociaciones con el Jugador y la Pantalla y establece
				el tamaño y las coordenadas iniciales del cursor.
				\brief Constructor.
				\param jugador Jugador al que pertenece el Cursor.
				\param pantalla Pantalla donde se dibujará el Cursor.
		 */
		Cursor(Jugador *jugador, Pantalla *pantalla);

		/*! Destruye la imagen del cursor.
				\brief Destructor.
		 */
		~Cursor();
		
		/*! Gira el cursor a la derecha.
				\brief Gira a la derecha.
		 */		
		void Derecha();

		/*! Gira el cursor a la izquierda.
				\brief Gira a la izquierda.
		 */		
		void Izquierda();
		
		/*! Orienta el cursor en dirección al hoyo del Campo.
				\brief Orienta al hoyo.
				\see Campo::PosicionHoyo.
		 */		
		void Orientar();

		/*! \brief Posición del cursor.
				\return Posición del cursor.
		 */
		const SDL_Rect& Posicion() const;

		/*! \brief Ángulo del cursor.
				\return Ángulo que forma el cursor con la Bola.
		 */
		float Angulo() const;

		/*! Dibuja el cursor en la Pantalla.
				\brief Dibuja el cursor.
		 */	
		void Dibujar() const;
		
		/*! \brief Cuadrante.
				\return Cuadrante en el que se encuentra el cursor.
		 */		
		int Cuadrante() const;
		
	private:

		/// Incrementamos el ángulo del Cursor.
		void Incrementar();

		/// Decrementamos el ángulo del Cursor.
		void Decrementar();

		static float incremento_;	///< Incremento del ángulo del Cursor.
		
		static float velocidad_;	///< Velocidad del Cursor
		
		static float angulo_completo_;	///< Ángulo completo.
		
		static float angulo_nulo_;	///< Ángulo nulo.
		
		static float radio_;	///< Radio del Cursor.

		SDL_Surface * cursor_;	///< Imagen del Cursor.
		
		SDL_Rect rcursor_;	///< Posición del Cursor.

		int px_;	///< Coordenada x del Cursor.
		
		int py_;	///< Coordenada y del Cursor.
		
		float angulo_;	///< Incremento del ángulo del Cursor.

		Jugador *jugador_;	///< Jugador con el que se relaciona el Cursor.
		
		Pantalla *pantalla_;	///< Pantalla del Juego.
};

#endif
