/// \file humano.cpp Humano.
/*
	Copyright 2010 Andrés Francisco Aparicio

	This file is part of SDLgolf.

	SDLgolf is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	SDLgolf is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with SDLgolf.  If not, see <http://www.gnu.org/licenses/>.
*/

#include <math.h>
#include "humano.h"
#include "partida.h"
#include "campo.h"
#include "pantalla.h"
#include "constantes.h"


Humano::Humano(std::string nombre, int color, Bola& bola, Partida *partida,
	Pantalla *pantalla)
{
	// El jugador está inactivo.
	estado_ = INACTIVO;

	// Establecemos el tipo de jugador a humano.
	tipo_ = HUMANO;

	// Establecemos el par del jugador a 0.
	par_ = 0;
	
	// Establecemos el nombre del jugador.
	nombre_ = nombre;

	// Guardamos el color del jugador.
	color_ = color;

	// Asociamos con la partida en juego.
	partida_ = partida;

	// Asociamos con la pantalla.
	pantalla_ = pantalla;

	// Asociamos con la bola.
	bola_ = &bola;
	
	// Creamos la lista de palos y los insertamos.
	palos_.push_back(Palo("Madera 1", 240, 45));
	palos_.push_back(Palo("Madera 3", 220, 46));
	palos_.push_back(Palo("Madera 5", 200, 47));
	palos_.push_back(Palo("Hierro 3", 180, 48));
	palos_.push_back(Palo("Hierro 4", 170, 49));
	palos_.push_back(Palo("Hierro 5", 160, 50));
	palos_.push_back(Palo("Hierro 6", 150, 52));
	palos_.push_back(Palo("Hierro 7", 140, 53));
	palos_.push_back(Palo("Hierro 8", 130, 54));
	palos_.push_back(Palo("Hierro 9", 120, 55));
	palos_.push_back(Palo("Pitching Wedge", 100, 60));
	palos_.push_back(Palo("Sand Wedge", 80, 70));
	palos_.push_back(Palo("Putter", 40, 1));
	palo_actual_ = 0;
	
	// Creamos y asociamos el nuevo cursor.
	cursor_ = new Cursor(this, pantalla_);

	// Establecemos el color del jugador.
	std::string nombre_color;
	
	switch(color)
	{
		case 0: nombre_color = "rojo"; break;
		case 1: nombre_color = "marron"; break;
		case 2: nombre_color = "amarillo"; break;
		case 3: nombre_color = "magenta"; break;
		case 4: nombre_color = "azul"; break;
		case 5: nombre_color = "cyan"; break;
		default:
			std::cerr << "Humano(): Error, color inválido." << std::endl;
			exit(1);
	}
	
	// Generamos la ruta en función del color del jugador.
	std::string ruta;
	ruta.append(Constantes::_IMG_JUGADOR_);
	ruta += nombre_color + ".png";
	
	// Creamos y asociamos la nueva rejilla de imágenes.
	rejilla_ = new Rejilla(ruta, 8, 9);

	// Establecemos el ancho y el alto de la rejilla.
	rjugador_.w = rejilla_ -> Posicion().w;
	rjugador_.h = rejilla_ -> Posicion().y;

	// Establecemos las coordenadas iniciales de la rejilla.
	rjugador_.x = 0;
	rjugador_.y = 0;
	
	// Creamos y asociamos el nuevo indicador de fuerza y efecto.
	indicador_ = new Indicador(this, pantalla_);

	// Guardamos las secuencias de animación.
	int s0[] = {0, 1, 3, 3, 1, 2, 0, 4, 5, 6, 6, 0, -1};
	int s1[] = {9, 10, 11, 12, 10, 10, 13, 14, 15, 16, 16, 9, -1};
	int s2[] = {18, 19, 20, 20, 19, 21, 18, 22, 23, 23, 18, -1};
	int s3[] = {27, 28, 29, 30, 30, 29, 28, 27, 31, 32, 33, 27, -1};
	// *** Secuencias particulares
	int s4[] = {36, 37, 38, 38, 39, 36, 40, 41, 42, 42, -1};
	int s5[] = {45, 46, 47, 47, 48, 49, 50, 51, 51, -1};
	int s6[] = {54, 55, 56, 56, 55, 55, 57, 58, 59, 59, 59, -1 };
	// ***
	int s7[] = {63, 64, 65, 66, 65, 64, 63, 68, 69, 70, 70, 63, -1};	

	// Creamos cada una de las secuencias y las guardamos.
	animaciones_.push_back(new Animacion (rejilla_, s0, 70, pantalla_));
	animaciones_.push_back(new Animacion (rejilla_, s1, 70, pantalla_));
	animaciones_.push_back(new Animacion (rejilla_, s2, 70, pantalla_));		
	animaciones_.push_back(new Animacion (rejilla_, s3, 70, pantalla_));
	animaciones_.push_back(new Animacion (rejilla_, s4, 70, pantalla_));
	animaciones_.push_back(new Animacion (rejilla_, s5, 70, pantalla_));
	animaciones_.push_back(new Animacion (rejilla_, s6, 70, pantalla_));
	animaciones_.push_back(new Animacion (rejilla_, s7, 70, pantalla_));	
}

int Humano::GolpearBola()
{
	// Leemos del indicador la fuerza que ha establecido el jugador.	
	float fuerza = indicador_ -> Fuerza();
	
	// Dividimos la fuerza entre 10 y la multiplicamos por el alcance máximo
	// del palo para obtener el alcance equivalente relativo a la fuerza.
	float alcance_maximo = PaloActual().AlcanceMaximo() * (fuerza / 10);

	// Leemos el ángulo del cursor.
	float angulo = cursor_ -> Angulo(); 

	// Coordenadas de destino.
	int x, y;
	
	// Obtenemos las coordenadas de destino en función de la posición de
	// la bola y el ángulo que forma con el cursor.
	if(bola_ -> PosicionBola().x + bola_ -> Ancho()/2  < 
		cursor_ -> Posicion().x + cursor_ -> Posicion().w/2)
		x = bola_ -> PosicionSombra().x + alcance_maximo * cos(angulo);
	else
		x = bola_ -> PosicionSombra().x - alcance_maximo * cos(angulo);

	if(cursor_ -> Posicion().y + cursor_ -> Posicion().h/2<
		bola_ -> PosicionBola().y + bola_ -> Alto()/2)
		y = bola_ -> PosicionSombra().y - alcance_maximo * sin(angulo);
	else
		y = bola_ -> PosicionSombra().y + alcance_maximo * sin(angulo);

	// Incrementamos el par del jugador.
	IncrementarPar();
	
	// Dibujamos todo.
	partida_ -> ActualizarPar();
	partida_ -> CampoActivo().Dibujar();
	partida_ -> DibujarBolasInactivas();
	bola_ -> DibujarBola();
	this -> Dibujar();
	partida_ -> CampoActivo().DibujarBandera();
	partida_ -> DibujarInterfaz();
	pantalla_ -> Actualizar();

	// LLevamos a cabo el desplazamiento.
	return bola_ -> Desplazar(x, y);
}

