/// \file indicador.cpp Indicador de fuerza y ángulo de tiro.
/*
	Copyright 2010 Andrés Francisco Aparicio

	This file is part of SDLgolf.

	SDLgolf is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	SDLgolf is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with SDLgolf.  If not, see <http://www.gnu.org/licenses/>.
*/

#include <iostream>
#include "indicador.h"
#include "pantalla.h"
#include "animacion.h"
#include "jugador.h"
#include "constantes.h"

Indicador::Indicador(Jugador *jugador, Pantalla *pantalla)
{
	// Establecemos el valor para el efecto a 0.
	efecto_ = 0;

	// Establecemos la fuerza a 0.
	fuerza_ = 0;
	
	// Asociamos con el jugador.
	jugador_ = jugador;

	// Asociamos con la pantalla.
	pantalla_ = pantalla;

	// Cadena temporal para la generación de rutas.
	std::string tmp;
	
	// Creamos la ruta a la imagen del indicador de fuerza.
	tmp.append(Constantes::_IMG_INDICADOR_);
	tmp += "fuerza.png";

	// Creamos y asociamos la nueva rejilla del indicador de fuerza.
	rejilla_fuerza_ = new Rejilla(tmp, 11, 1);

	// Creamos la ruta a la imagen del indicador de efecto.
	tmp.clear();
	tmp.append(Constantes::_IMG_INDICADOR_);
	tmp += "efecto.png";

	// Creamos y asociamos la nueva rejilla del indicador de efecto.
	rejilla_efecto_ = new Rejilla(tmp, 11, 1);

	// Secuencia de animación para el indicador de fuerza.
	int secuencia_fuerza[] = 
		{0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 9, 8, 7, 6, 5, 4, 3, 2, 1, -1};

	// Secuencia de animación para el indicador de efecto.
	int secuencia_efecto[] =
		{0, 1, 2, 3, 4, 5, 4, 3, 2, 1, 0, 6, 7, 8, 9, 10, 9, 8, 7, 6, -1};

	// Creamos y asociamos la animación del indicador de fuerza.
	afuerza_ = new Animacion (rejilla_fuerza_, secuencia_fuerza, 60, pantalla_);

	// Creamos y asociamos la animación del indicador de fuerza.
	aefecto_ = new Animacion (rejilla_efecto_, secuencia_efecto, 60, pantalla_);
}

Indicador::~Indicador()
{
	// Destruimos la animación de el indicador de fuerza.
	delete afuerza_;

	// Destruimos la animación de el indicador de efecto.
	delete aefecto_;

	// Destruimos la rejilla de el indicador de fuerza.
	delete rejilla_fuerza_;

	// Destruimos la rejilla de el indicador de efecto.
	delete rejilla_efecto_;
}

void Indicador::Efecto(int efecto)
{
	// Establecemos el valor del efecto.
	efecto_ = efecto;
}

void Indicador::Fuerza(int fuerza)
{
	// Establecemos el valor de la fuerza.
	fuerza_ = fuerza;
}

int Indicador::Efecto() const
{
	// Devolvemos el valor del indicador de efecto.
	return efecto_;
}

int Indicador::Fuerza() const
{
	// Devolvemos el valor del indicador de fuerza.
	return fuerza_;
}

Animacion& Indicador::AnimacionFuerza()
{
	// Devolvemos la animación del indicador de fuerza.
	return *afuerza_;
}

Animacion& Indicador::AnimacionEfecto()
{
	// Devolvemos la animación del indicador de efecto.
	return *aefecto_;
}

void Indicador::ActualizarPosicion()
{
	// Variables de posición para los indicadores.
	SDL_Rect posicion_fuerza, posicion_efecto;

	// Variables de coordenadas para los desplazamientos de los indicadores
	// en función de el cuadro del jugador.
	int desplazamiento_x = 0;
	int desplazamiento_y = 0;

	// Elegimos los desplazamientos en función del cuadro del jugador.
	switch(jugador_ -> CuadroActual())
	{
		case 0:		desplazamiento_x = -30; desplazamiento_y = 2;		break;
		case 9: 	desplazamiento_x = -26; desplazamiento_y = 7;		break;
		case 18:	desplazamiento_x = -18; desplazamiento_y = 16;	break;				
		case 27:	desplazamiento_x = -9;	desplazamiento_y = 6;		break;			
		case 36:	desplazamiento_x = -8;	desplazamiento_y = 2;		break;
		case 45:	desplazamiento_x = -17;	desplazamiento_y = 2;		break;			
		case 54:	desplazamiento_x = -20;	desplazamiento_y = 2;		break;					
		case 63:	desplazamiento_x = -25;	desplazamiento_y = 2;		break;					
		default:  break;
	}
	
	// Establecemos la nueva coordenada x para el indicador de fuerza.
	posicion_fuerza.x = jugador_ -> BolaEnJuego().PosicionBola().x
		+ desplazamiento_x;

	// Establecemos la nueva coordenada y para el indicador de fuerza.
	posicion_fuerza.y = jugador_ -> BolaEnJuego().PosicionBola().y
			+ jugador_ -> BolaEnJuego().Alto() + desplazamiento_y;

	// Establecemos la nueva coordenada x para el indicador de efecto.
	posicion_efecto.x = posicion_fuerza.x - 2;

	// Establecemos la nueva coordenada y para el indicador de efecto.
	posicion_efecto.y = posicion_fuerza.y + 8;	
	
	// Actualizamos la posición para el indicador de fuerza.
	afuerza_ -> Posicion(posicion_fuerza);

	// Actualizamos la posición para el indicador de efecto.
	aefecto_ -> Posicion(posicion_efecto);
}
