/// \file indicador.h Indicador de fuerza y ángulo de tiro.
/*
	Copyright 2010 Andrés Francisco Aparicio

	This file is part of SDLgolf.

	SDLgolf is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	SDLgolf is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with SDLgolf.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef _INDICADOR_H_
#define _INDICADOR_H_

#include "animacion.h"

class Jugador;
class Pantalla;

/*! \class Indicador
    \brief Indicador de fuerza y efecto.

    La clase indicador se encarga de mostrar al usuario los niveles de fuerza
    y efecto que se aplicarán al lanzamiento.
    
    Para ello, se mostrarán sendas
    animaciones con las que el usuario podrá interactuar para determinar
    los niveles de fuerza y efectos deseados con los que se realizará
    el lanzamiento de la bola de golf.
 */
class Indicador
{
	public:

		/*! Realiza las asociaciones con el Jugador y la Pantalla.\n
				Genera las animaciones para los indicadores de fuerza y efecto.
				\brief Constructor.
				\param jugador Jugador al que pertenece el Indicador.
				\param pantalla Pantalla donde se dibujará el Indicador.
		 */
		Indicador(Jugador *jugador, Pantalla *pantalla);

		/*! Destruye las animaciones y las rejillas de imágenes de los indicadores
				de fuerza y efecto.
				\brief Destructor.
		 */
		~Indicador();
		
		/*! Establece el valor de efecto que se ha aplicado a la Bola en el
				lanzamiento.
				\brief Efecto de tiro.
				\param efecto Valor del efecto aplicado.
				\see Jugador::GolpearBola.
		 */		
		void Efecto(int efecto);

		/*! Establece el valor de la fuerza que se ha aplicado a la Bola en el
				lanzamiento.
				\brief Establece la fuerza de lanzamiento.
				\param fuerza Valor de la fuerza aplicada.
				\see Jugador::GolpearBola.
		 */
		void Fuerza(int fuerza);

		/*! Devuelve la Animacion del indicador de fuerza.
				\brief Animación del indicador de fuerza.
				\return Animación de la barra de fuerza.
		 */
		Animacion& AnimacionFuerza();

		/*! Devuelve la Animacion del indicador de efecto.
				\brief Animación del indicador de efecto.
				\return Animación de la barra de efecto.
		 */
		Animacion& AnimacionEfecto();

		/*! \brief Efecto del lanzamiento.
				\return Valor del efecto aplicado a la Bola.
		 */
		int Efecto() const;

		/*! \brief Fuerza del lanzamiento.
				\return Valor de la fuerza aplicada a la Bola.
		 */
		int Fuerza() const;

		/*! Actualiza la posición de los indicadores en función del cuadro
				del Jugador que esté seleccionado.
				\brief Actualiza la posición del indicador.
		 */
		void ActualizarPosicion();
	
	private:
	
		int efecto_;	///< Efecto aplicado a la Bola.
		
		int fuerza_;	///< Fuerza aplicada a la Bola.
		
		Animacion *afuerza_;	///< Animación de la barra de fuerza.
		
		Animacion *aefecto_;	///< Animación de la barra de efecto.
		
		Rejilla *rejilla_fuerza_;	///< Rejilla de la barra de fuerza.
		
		Rejilla *rejilla_efecto_;	///< Rejilla de la barra de efecto.
		
		Jugador *jugador_;	///< Jugador con el que se relaciona el Indicador.
		
		Pantalla *pantalla_;	///< Pantalla del Juego.
};

#endif
