/// \file interfaz.cpp Interfaz de juego.
/*
	Copyright 2010 Andrés Francisco Aparicio

	This file is part of SDLgolf.

	SDLgolf is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	SDLgolf is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with SDLgolf.  If not, see <http://www.gnu.org/licenses/>.
*/

#include <string>
#include <sstream>
#include "interfaz.h"
#include "pantalla.h"
#include "utilidades.h"
#include "partida.h"
#include "jugador.h"
#include "campo.h"
#include "constantes.h"
#include "rejilla.h"

Interfaz::Interfaz(Partida *partida, Pantalla *pantalla)
{
	// Asociamos con la pantalla.
	pantalla_ = pantalla;

	// Asociamos con la partida en juego.
	partida_ = partida;

	// El mapa está desactivado por defecto.
	dibujar_mapa_ = false;

	// La interfaz no se ha generado aún. Las imágenes son nulas.
	barra_ = 0;
	campo_miniatura_ = 0;
	campo_nombre_ = 0;
	campo_hoyo_ = 0;
	campo_par_ = 0;
	jugador_nombre_ = 0;
	jugador_par_ = 0;	
	palo_nombre_ = 0;
	palo_distancia_max_ = 0;
	distancia_hoyo_ = 0;
	campo_mapa_ = 0;
	cursor_mapa_ = 0;
	bandera_mapa_ = 0;
	
	// Cargamos la rejilla de las imágenes de los jugadores en el mapa.
	rejilla_jugador_mapa_ = new Rejilla(RutaImagen("jugador_mapa.png"), 1, 6);

	// Cargamos la rejilla de las imágenes de los palos.	
	rejilla_palos_ = new Rejilla(RutaImagen("palos.png"), 2, 7);
}

Interfaz::~Interfaz()
{
	// Destruimos todas las imágenes.
	SDL_FreeSurface(barra_);
	SDL_FreeSurface(campo_miniatura_);
	SDL_FreeSurface(campo_nombre_);
	SDL_FreeSurface(campo_hoyo_);
	SDL_FreeSurface(campo_par_);
	SDL_FreeSurface(jugador_nombre_);
	SDL_FreeSurface(jugador_par_);
	SDL_FreeSurface(palo_nombre_);
	SDL_FreeSurface(palo_distancia_max_);
	SDL_FreeSurface(distancia_hoyo_);
	SDL_FreeSurface(campo_mapa_);
	SDL_FreeSurface(cursor_mapa_);
	SDL_FreeSurface(bandera_mapa_);
	
	// Destruimos la rejilla de los jugadores.
	delete rejilla_jugador_mapa_;
	
	// Destruimos la rejilla de las imágenes de los palos.
	delete rejilla_palos_;
}

void Interfaz::Dibujar()
{
	// Dibujamos todas las imágenes por pantalla.
	pantalla_ -> DibujarDirectamente(barra_, &rbarra_);
	pantalla_ -> DibujarDirectamente(campo_miniatura_, &rcampo_miniatura_);
	pantalla_ -> DibujarDirectamente(campo_nombre_, &rcampo_nombre_);
	pantalla_ -> DibujarDirectamente(campo_hoyo_, &rcampo_hoyo_);	
	pantalla_ -> DibujarDirectamente(campo_par_, &rcampo_par_);
	pantalla_ -> DibujarDirectamente(jugador_nombre_, &rjugador_nombre_);
	pantalla_ -> DibujarDirectamente(rejilla_palos_ -> Imagen(),
		&(rejilla_palos_ -> Posicion()), &rpalo_miniatura_);
	pantalla_ -> DibujarDirectamente(palo_nombre_, &rpalo_nombre_);
	pantalla_ -> DibujarDirectamente(jugador_par_, &rjugador_par_);
	pantalla_ -> DibujarDirectamente(palo_distancia_max_, &rpalo_distancia_max_);
	pantalla_ -> DibujarDirectamente(distancia_hoyo_, &rdistancia_hoyo_);
	
	// Si es necesario dibujar el mapa, obtenemos las coordenadas y lo
	// dibujamos.
	if(dibujar_mapa_)
	{
		// Establecemos la coordenada y para el cursor en miniatura.	
		rcursor_mapa_.x = rcampo_mapa_.x + 2
			+ partida_ -> JugadorActivo().CursorActual().Posicion().x
			* (rcampo_mapa_.w - 4) / partida_ -> CampoActivo().Ancho();	

		// Establecemos la coordenada y para el cursor en miniatura.		
		rcursor_mapa_.y = rcampo_mapa_.y + 2
			+ partida_ -> JugadorActivo().CursorActual().Posicion().y
			* (rcampo_mapa_.h - 4) / partida_ -> CampoActivo().Alto();

		// Establecemos la coordenada x para la bandera en miniatura.
		rbandera_mapa_.x = rcampo_mapa_.x + 2 +
			partida_ -> CampoActivo().PosicionHoyo().x *
			(rcampo_mapa_.w - 4) / partida_ -> CampoActivo().Ancho();

		// Establecemos la coordenada y para la bandera en miniatura.		
		rbandera_mapa_.y = rcampo_mapa_.y + 2 +
			+ partida_ -> CampoActivo().PosicionHoyo().y * (rcampo_mapa_.h - 4)
			/ partida_ -> CampoActivo().Alto() - 8;

		// Dibujamos el mapa, el jugador en miniatura y el cursor en miniatura.
		pantalla_ -> DibujarDirectamente(campo_mapa_, &rcampo_mapa_);
		pantalla_ -> DibujarDirectamente(rejilla_jugador_mapa_ -> Imagen(),
			&(rejilla_jugador_mapa_ -> Posicion()), &rjugador_mapa_);
		pantalla_ -> DibujarDirectamente(cursor_mapa_, &rcursor_mapa_);
		pantalla_ -> DibujarDirectamente(bandera_mapa_, &rbandera_mapa_);
	}
}

void Interfaz::Generar()
{
	// Destruimos todas las imágenes porque tenemos que generarlas de nuevo.
	SDL_FreeSurface(barra_);
	SDL_FreeSurface(campo_miniatura_);	
	SDL_FreeSurface(campo_nombre_);
	SDL_FreeSurface(campo_hoyo_);
	SDL_FreeSurface(campo_par_);
	SDL_FreeSurface(jugador_nombre_);
	SDL_FreeSurface(jugador_par_);
	SDL_FreeSurface(palo_nombre_);
	SDL_FreeSurface(palo_distancia_max_);
	SDL_FreeSurface(distancia_hoyo_);
	SDL_FreeSurface(campo_mapa_);
	SDL_FreeSurface(cursor_mapa_);
	SDL_FreeSurface(bandera_mapa_);

	// Variable para la conversión de int a string.
	std::stringstream ss;

	// Ruta a las imágenes del campo.
	std::string ruta_campo = Constantes::_IMG_CAMPO_;

	// Cargamos la imagen de la barra de la interfaz.
	barra_ = Utilidades::CargarImagen(RutaImagen("barra.png"));

	// Establecemos el ancho y el alto.
	rbarra_.w = barra_ -> w;
	rbarra_.h = barra_ -> h;

	// Establecemos las coordenadas x e y.
	rbarra_.x = 0;
	rbarra_.y = pantalla_ -> Alto() - rbarra_.h;
	
	// Cargamos la imagen del mapa.
	campo_mapa_ = Utilidades::CargarImagen(ruta_campo
		+ partida_ -> CampoActivo().Ruta()
		+ "/" + partida_ -> CampoActivo().NumeroHoyo() + "_mapa.png");

	// Guardamos el ancho y el alto de la imagen del mapa.
	rcampo_mapa_.w = campo_mapa_ -> w;
	rcampo_mapa_.h = campo_mapa_ -> h;

	// Determinamos la coordenada x para el mapa.
	if(partida_ -> JugadorActivo().BolaEnJuego().PosicionBola().x
	< partida_ -> CampoActivo().Ancho() / 2)
		rcampo_mapa_.x = pantalla_ -> Ancho() - campo_mapa_ -> w - 20;
	else
		rcampo_mapa_.x = 20;

	// Determinamos la coordenada y para el mapa.
	rcampo_mapa_.y = (pantalla_ -> Alto() - rbarra_.h - campo_mapa_ -> h) / 2;


	// Seleccionamos el jugador en miniatura para el mapa en función de
	// su color.
	rejilla_jugador_mapa_ ->
		SeleccionarCuadro(partida_ -> JugadorActivo().Color());

	// Guardamos el ancho y el alto de la miniatura del jugador.
	rjugador_mapa_.w = rejilla_jugador_mapa_ -> Posicion().w;
	rjugador_mapa_.h = rejilla_jugador_mapa_ -> Posicion().h;
	
	// Establecemos la coordenada x para la miniatura del jugador.
	rjugador_mapa_.x = rcampo_mapa_.x + 2
		+ partida_ -> JugadorActivo().BolaEnJuego().PosicionBola().x
		* (rcampo_mapa_.w - 4) / partida_ -> CampoActivo().Ancho();

	// Establecemos la coordenada y para la miniatura del jugador.		
	rjugador_mapa_.y = rcampo_mapa_.y + 2
		+ partida_ -> JugadorActivo().BolaEnJuego().PosicionBola().y
		* (rcampo_mapa_.h - 4) / partida_ -> CampoActivo().Alto();
	
	// Cargamos la imagen del cursor en miniatura.
	cursor_mapa_ = Utilidades::CargarImagen(RutaImagen("cursor_mapa.png"));
	rcursor_mapa_.w = cursor_mapa_ -> w;
	rcursor_mapa_.h = cursor_mapa_ -> h;	

	// Establecemos la coordenada x para el cursor en miniatura.
	rcursor_mapa_.x = rcampo_mapa_.x + 2
		+ partida_ -> JugadorActivo().CursorActual().Posicion().x
		* (rcampo_mapa_.w - 4) / partida_ -> CampoActivo().Ancho();	

	// Establecemos la coordenada y para el cursor en miniatura.		
	rcursor_mapa_.y = rcampo_mapa_.y + 2
		+ partida_ -> JugadorActivo().CursorActual().Posicion().y
		* (rcampo_mapa_.h - 4) / partida_ -> CampoActivo().Alto();

	// Cargamos la imagen de la bandera en miniatura.
	bandera_mapa_ = Utilidades::CargarImagen(RutaImagen("bandera_mapa.png"));
	rbandera_mapa_.w = bandera_mapa_ -> w;
	rbandera_mapa_.h = bandera_mapa_ -> h;

	// Establecemos la coordenada x para la bandera en miniatura.
	rbandera_mapa_.x = rcampo_mapa_.x + 2 +
		partida_ -> CampoActivo().PosicionHoyo().x *
		(rcampo_mapa_.w - 4) / partida_ -> CampoActivo().Ancho();

	// Establecemos la coordenada y para la bandera en miniatura.		
	rbandera_mapa_.y = rcampo_mapa_.y + 2 +
		+ partida_ -> CampoActivo().PosicionHoyo().y * (rcampo_mapa_.h - 4)
		/ partida_ -> CampoActivo().Alto() - 8;

	// Cargamos la imagen de la miniatura del campo.
	campo_miniatura_ = Utilidades::CargarImagen(ruta_campo
		+ partida_ -> CampoActivo().Ruta()
		+ "/" + partida_ -> CampoActivo().NumeroHoyo() + "_miniatura.png");

	// Guardamos el ancho y el alto de la imagen.
	rcampo_miniatura_.w = campo_miniatura_ -> w;
	rcampo_miniatura_.h = campo_miniatura_ -> h;

	// Establecemos las coordenadas para la miniatura del campo.
	rcampo_miniatura_.x = 10;
	rcampo_miniatura_.y = rbarra_.y + (barra_ -> h - campo_miniatura_ -> h) / 2;
	
	// Creamos el texto del nombre del campo.
	campo_nombre_ = Utilidades::CrearTexto(partida_
		-> CampoActivo().Nombre().c_str(), Constantes::_FUENTE_, 18,
			Utilidades::Color(Utilidades::BLANCO));

	// Guardamos el ancho y el alto del texto.
	rcampo_nombre_.w = campo_nombre_ -> w;
	rcampo_nombre_.h = campo_nombre_ -> h;

	// Establecemos las coordenadas para el nombre del campo.
	rcampo_nombre_.x = rcampo_miniatura_.x + rcampo_miniatura_.w + 5;
	rcampo_nombre_.y = rcampo_miniatura_.y;

	// Cadena temporal.
	std::string tmp;

	// Limpiamos la cadena.
	tmp.clear();

	// Creamos la cadena del número de hoyo.
	tmp = "Hoyo " + partida_ -> CampoActivo().NumeroHoyo();

	// Generamos el texto del número de hoyo.
	campo_hoyo_ = Utilidades::CrearTexto(tmp.c_str(), Constantes::_FUENTE_, 14,
		Utilidades::Color(Utilidades::BLANCO));

	// Guardamos el ancho y el alto del texto.
	rcampo_hoyo_.w = campo_hoyo_ -> w;
	rcampo_hoyo_.h = campo_hoyo_ -> h;

	// Establecemos las coordenadas para el número de hoyo.
	rcampo_hoyo_.x = rcampo_nombre_.x;	
	rcampo_hoyo_.y = rcampo_nombre_.y + 20;

	// Borramos la salida.
	ss.str("");

	// Imprimimos el par del campo.
	ss << partida_ -> CampoActivo().Par();

	// Limpiamos la cadena.
	tmp.clear();

	// Generamos el par del campo.
	tmp = "Par " + ss.str();

	// Creamos el texto del par del campo.
	campo_par_ = Utilidades::CrearTexto(tmp.c_str(), Constantes::_FUENTE_, 14,
		Utilidades::Color(Utilidades::BLANCO));

	// Guardamos el ancho y el alto del texto.
	rcampo_par_.w = campo_par_ -> w;
	rcampo_par_.h = campo_par_ -> h;

	// Establecemos las coordenadas para el texto del par del campo.
	rcampo_par_.x = rcampo_nombre_.x;	
	rcampo_par_.y = rcampo_hoyo_.y + 15;

	// Creamos el texto del nombre del jugador.
	jugador_nombre_ = Utilidades::CrearTexto(partida_
		-> JugadorActivo().Nombre().c_str(), Constantes::_FUENTE_, 18,
		 Utilidades::Color(Utilidades::BLANCO));

	// Guardamos el ancho y el alto del texto.
	rjugador_nombre_.w = jugador_nombre_ -> w;
	rjugador_nombre_.h = jugador_nombre_ -> h;

	// Establecemos las coordenadas para el nombre del jugador.
	rjugador_nombre_.x = (pantalla_ -> Ancho() / 2) - jugador_nombre_ -> w / 2;
	rjugador_nombre_.y = rcampo_miniatura_.y;

	// Guardamos el palo actual.
	std::string palo_actual_nombre =
		partida_ -> JugadorActivo().PaloActual().Nombre();
	
	// Seleccionamos la imagen del palo.
	rejilla_palos_
		-> SeleccionarCuadro(partida_ -> JugadorActivo().IndicePaloActual());

	// Guardamos el ancho y el alto de la imagen.
	rpalo_miniatura_.w = rejilla_palos_ -> Posicion().w;
	rpalo_miniatura_.h = rejilla_palos_ -> Posicion().h;

	// Establecemos las coordenadas de la imagen del palo.
	rpalo_miniatura_.x = rbarra_.w - rpalo_miniatura_.w - 10;
	rpalo_miniatura_.y = rcampo_miniatura_.y;

	// Generamos el texto con el nombre del palo.
	palo_nombre_ = Utilidades::CrearTexto(partida_ ->
		JugadorActivo().PaloActual().Nombre().c_str(),
		Constantes::_FUENTE_, 18, Utilidades::Color(Utilidades::BLANCO));

	// Guardamos el ancho y el alto del texto.
	rpalo_nombre_.w = palo_nombre_ -> w;
	rpalo_nombre_.h = palo_nombre_ -> h;

	// Establecemos las coordenadas del nombre del palo.
	rpalo_nombre_.x = rpalo_miniatura_.x - rpalo_nombre_.w - 5;
	rpalo_nombre_.y = rcampo_nombre_.y;

	// Borramos la salida.
	ss.str("");

	// Imprimimos el par del jugador.
	ss << partida_ -> JugadorActivo().Par();

	// Limpiamos la cadena.
	tmp.clear();

	// Generamos la cadena con el par del jugador.
	tmp = "Par " + ss.str();

	// Creamos el texto del par del jugador.
	jugador_par_ = Utilidades::CrearTexto(tmp.c_str(), Constantes::_FUENTE_, 14,
		Utilidades::Color(Utilidades::BLANCO));

	// Guardamos el ancho y el alto del texto.
	rjugador_par_.w = jugador_par_ -> w;
	rjugador_par_.h = jugador_par_ -> h;

	// Establecemos las coordenadas del par del jugador.
	rjugador_par_.x = (pantalla_ -> Ancho() / 2) - rjugador_par_.w / 2;
	rjugador_par_.y = rcampo_hoyo_.y;
	
	// Borramos la salida.
	ss.str("");

	// Imprimimos el alcance del palo.
	ss << partida_ -> JugadorActivo().PaloActual().AlcanceReal();

	// Limpiamos la cadena.
	tmp.clear();

	// Generamos la cadena con el alcance.
	tmp = "Maxima: " + ss.str();

	// Creamos el texto con el alcance.
	palo_distancia_max_ = Utilidades::CrearTexto(tmp.c_str(),
		Constantes::_FUENTE_, 14, Utilidades::Color(Utilidades::BLANCO));

	// Guardamos el ancho y el alto del texto.
	rpalo_distancia_max_.w = palo_distancia_max_ -> w;
	rpalo_distancia_max_.h = palo_distancia_max_ -> h;

	// Establecemos las coordenadas del texto del alcance.
	rpalo_distancia_max_.x = rpalo_miniatura_.x - rpalo_distancia_max_.w - 5;
	rpalo_distancia_max_.y = rcampo_hoyo_.y;

	// Borramos la salida.
	ss.str("");

	// Imprimimos la distancia al hoyo.
	ss << partida_ -> JugadorActivo().DistanciaHoyo();

	// Limpiaos la cadena.
	tmp.clear();

	// Generamos la cadena con la distancia al hoyo.
	tmp = "Al hoyo: " + ss.str();

	// Creamos el texto con la distancia al hoyo.
	distancia_hoyo_ = Utilidades::CrearTexto(tmp.c_str(),
		Constantes::_FUENTE_, 14, Utilidades::Color(Utilidades::BLANCO));

	// Guardamos el ancho y el alto del texto.
	rdistancia_hoyo_.w = distancia_hoyo_ -> w;
	rdistancia_hoyo_.h = distancia_hoyo_ -> h;

	// Establecemos las coordenadas de la distancia al hoyo.
	rdistancia_hoyo_.x = rpalo_miniatura_.x - rdistancia_hoyo_.w - 5;
	rdistancia_hoyo_.y = rcampo_par_.y;
}

void Interfaz::ActivarMapa()
{
	// Activamos el mapa.
	dibujar_mapa_ = true;
}

void Interfaz::DesactivarMapa()
{
	// Desactivamos el mapa.
	dibujar_mapa_ = false;
}

std::string Interfaz::RutaImagen(const char *imagen)
{
	std::string ruta = Constantes::_IMG_INTERFAZ_;
	std::string img = imagen;
	return ruta += img;
}

void Interfaz::ActualizarPalo()
{
	SDL_FreeSurface(palo_nombre_);
	SDL_FreeSurface(palo_distancia_max_);

	// Seleccionamos la imagen del palo.
	rejilla_palos_
		-> SeleccionarCuadro(partida_ -> JugadorActivo().IndicePaloActual());

	// Generamos el texto con el nombre del palo.
	palo_nombre_ = Utilidades::CrearTexto(partida_ ->
		JugadorActivo().PaloActual().Nombre().c_str(),
		Constantes::_FUENTE_, 18, Utilidades::Color(Utilidades::BLANCO));

	// Guardamos el ancho y el alto del texto.
	rpalo_nombre_.w = palo_nombre_ -> w;
	rpalo_nombre_.h = palo_nombre_ -> h;

	// Establecemos las coordenadas del nombre del palo.
	rpalo_nombre_.x = rpalo_miniatura_.x - rpalo_nombre_.w - 5;
	rpalo_nombre_.y = rcampo_nombre_.y;

	// Variable para la conversión de int a string.
	std::stringstream ss;

	// Imprimimos el alcance del palo.
	ss << partida_ -> JugadorActivo().PaloActual().AlcanceReal();

	// Generamos la cadena con el alcance.
	std::string tmp = "Maxima: " + ss.str();

	// Creamos el texto con el alcance.
	palo_distancia_max_ = Utilidades::CrearTexto(tmp.c_str(),
		Constantes::_FUENTE_, 14, Utilidades::Color(Utilidades::BLANCO));

	// Guardamos el ancho y el alto del texto.
	rpalo_distancia_max_.w = palo_distancia_max_ -> w;
	rpalo_distancia_max_.h = palo_distancia_max_ -> h;

	// Establecemos las coordenadas del texto del alcance.
	rpalo_distancia_max_.x = rpalo_miniatura_.x - rpalo_distancia_max_.w - 5;
	rpalo_distancia_max_.y = rcampo_hoyo_.y;

}

void Interfaz::ActualizarPar()
{
	SDL_FreeSurface(jugador_par_);

	// Variable para la conversión de int a string.
	std::stringstream ss;
	
	// Imprimimos el par del jugador.
	ss << partida_ -> JugadorActivo().Par();

	// Generamos la cadena con el par del jugador.
	std::string tmp = "Par " + ss.str();

	// Creamos el texto del par del jugador.
	jugador_par_ = Utilidades::CrearTexto(tmp.c_str(), Constantes::_FUENTE_, 14,
		Utilidades::Color(Utilidades::BLANCO));

	// Guardamos el ancho y el alto del texto.
	rjugador_par_.w = jugador_par_ -> w;
	rjugador_par_.h = jugador_par_ -> h;
}

void Interfaz::Actualizar()
{
	SDL_FreeSurface(distancia_hoyo_);
	SDL_FreeSurface(jugador_nombre_);

	// Determinamos la coordenada x para el mapa.
	if(partida_ -> JugadorActivo().BolaEnJuego().PosicionBola().x
	< partida_ -> CampoActivo().Ancho() / 2)
		rcampo_mapa_.x = pantalla_ -> Ancho() - campo_mapa_ -> w - 20;
	else
		rcampo_mapa_.x = 20;

	// Determinamos la coordenada y para el mapa.
	rcampo_mapa_.y = (pantalla_ -> Alto() - rbarra_.h - campo_mapa_ -> h) / 2;

	// Variable para la conversión de int a string.
	std::stringstream ss;

	// Imprimimos la distancia al hoyo.
	ss << partida_ -> JugadorActivo().DistanciaHoyo();
	
	// Generamos la cadena con la distancia al hoyo.
	std::string tmp = "Al hoyo: " + ss.str();

	// Creamos el texto con la distancia al hoyo.
	distancia_hoyo_ = Utilidades::CrearTexto(tmp.c_str(),
		Constantes::_FUENTE_, 14, Utilidades::Color(Utilidades::BLANCO));

	// Guardamos el ancho y el alto del texto.
	rdistancia_hoyo_.w = distancia_hoyo_ -> w;
	rdistancia_hoyo_.h = distancia_hoyo_ -> h;

	// Establecemos las coordenadas de la distancia al hoyo.
	rdistancia_hoyo_.x = rpalo_miniatura_.x - rdistancia_hoyo_.w - 5;
	rdistancia_hoyo_.y = rcampo_par_.y;
	
	// Creamos el texto del nombre del jugador.
	jugador_nombre_ = Utilidades::CrearTexto(partida_
		-> JugadorActivo().Nombre().c_str(), Constantes::_FUENTE_, 18,
		 Utilidades::Color(Utilidades::BLANCO));

	// Guardamos el ancho y el alto del texto.
	rjugador_nombre_.w = jugador_nombre_ -> w;
	rjugador_nombre_.h = jugador_nombre_ -> h;

	// Establecemos las coordenadas para el nombre del jugador.
	rjugador_nombre_.x = (pantalla_ -> Ancho() / 2) - jugador_nombre_ -> w / 2;
	rjugador_nombre_.y = rcampo_miniatura_.y;

	// Seleccionamos el jugador en miniatura para el mapa en función de
	// su color.
	rejilla_jugador_mapa_ ->
		SeleccionarCuadro(partida_ -> JugadorActivo().Color());

	// Guardamos el ancho y el alto de la miniatura del jugador.
	rjugador_mapa_.w = rejilla_jugador_mapa_ -> Posicion().w;
	rjugador_mapa_.h = rejilla_jugador_mapa_ -> Posicion().h;
	
	// Establecemos la coordenada x para la miniatura del jugador.
	rjugador_mapa_.x = rcampo_mapa_.x + 2
		+ partida_ -> JugadorActivo().BolaEnJuego().PosicionBola().x
		* (rcampo_mapa_.w - 4) / partida_ -> CampoActivo().Ancho();

	// Establecemos la coordenada y para la miniatura del jugador.		
	rjugador_mapa_.y = rcampo_mapa_.y + 2
		+ partida_ -> JugadorActivo().BolaEnJuego().PosicionBola().y
		* (rcampo_mapa_.h - 4) / partida_ -> CampoActivo().Alto();
	
	// Actualizamos el palo seleccionado.
	ActualizarPalo();
	
	// Actualizamos el par del jugador.
	ActualizarPar();
}
