/// \file interfaz.h Interfaz de juego.
/*
	Copyright 2010 Andrés Francisco Aparicio

	This file is part of SDLgolf.

	SDLgolf is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	SDLgolf is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with SDLgolf.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef _INTERFAZ_H_
#define _INTERFAZ_H_

#include <SDL/SDL_image.h>

class Pantalla;
class Partida;
class Rejilla;

/*! \class Interfaz
    \brief Interfaz del juego.

		La interfaz informa al jugador sobre los datos más importantes de la
		partida.
		
		En ella, se indica el nombre del campo y el número de hoyo
		en el que se está jugando, así como el par de éste último.
		
		Por otra parte,
		se muestra el nombre y el par del jugador activo, el nombre y el alcance
		del palo de golf que tiene seleccionado y la distancia a la que se
		encuentra el jugador con respecto al hoyo.
		
		También permite al jugador visualizar el mapa del campo en el que se
		está jugando.
 */
class Interfaz
{
	public:

		/*! Realiza las asociaciones con la Partida y la Pantalla.\n
				Establece los punteros de las imágenes como nulos y desactiva el
				mapa del campo. Crea la rejilla de imágenes para la representación
				de los jugadores en el mapa en función de su color.
				\brief Constructor.
				\param partida Partida que se está desarrollando.
				\param pantalla Pantalla donde se dibujará la interfaz.
				\see Jugador::Color.
		 */	
		Interfaz(Partida *partida, Pantalla *pantalla);
		
		/*! Destruye todas las imágenes de la Interfaz y la rejilla de los
				jugadores en el mapa.
				\brief Destructor.
		 */		
		~Interfaz();
		
		/*! Dibuja la Interfaz en la Pantalla y actualiza la posición del
				mapa en función de la posición del Jugador.
				\brief Dibuja la interfaz.
		 */		
		void Dibujar();
		
		/*! Genera la interfaz por completo, carga todas las imágenes y genera
				todos los datos.
				\brief Genera la interfaz.
				\see Jugador::PaloActual.
				\see Jugador::Par.
		 */			
		void Generar();
		
		/*! Activa el mapa para que sea dibujado.
				\brief Activa el mapa del Campo.
		 */			
		void ActivarMapa();

		/*! Desactiva el mapa para que no sea dibujado.
				\brief Desactiva el mapa del Campo.
		 */	
		void DesactivarMapa();

		/*! \brief Actualiza la imagen, el nombre y el alcance del palo
				seleccionado.
		 */			
		void ActualizarPalo();

		/*! \brief Actualiza el par del jugador.
		 */			
		void ActualizarPar();

		/*! \brief Actualiza todos los datos e imágenes de la interfaz, salvo
							 el nombre del campo, el número de hoyo y la imagen en
							 miniatura del campo.
		 */		
		void Actualizar();
	
	private:

		/*! Genera una ruta desde el directorio de imágenes de la interfaz.
				\brief Genera la ruta a una imagen.
				\param imagen Nombre de la imagen.
				\return Ruta a la imagen.
		 */	
		std::string RutaImagen(const char *imagen);
	
		Pantalla *pantalla_;	///< Pantalla del Juego.
		
		Partida *partida_;	///< Partida que está actualmente en juego.

		bool dibujar_mapa_;	///< Variable de control para dibujar el mapa.

		SDL_Surface *barra_;	///< Imagen de la barra de la Interfaz.
		
		SDL_Rect rbarra_;	///< Posición de la barra de la Interfaz.
		
		SDL_Surface *campo_miniatura_;	///< Imagen de la miniatura del Campo.
		
		SDL_Rect rcampo_miniatura_;	///< Posición de la miniatura del Campo.
		
		SDL_Surface *campo_nombre_;	///< Imagen del nombre del Campo.
		
		SDL_Rect rcampo_nombre_;	///< Posición del nombre del Campo.
		
		SDL_Surface *campo_hoyo_;	///< Imagen del número de hoyo.
		
		SDL_Rect rcampo_hoyo_;	///< Posición del número de hoyo.
		
		SDL_Surface *campo_par_;	///< Imagen del par del hoyo.
		
		SDL_Rect rcampo_par_;	///< Posición del par del hoyo.
		
		SDL_Surface *jugador_nombre_;	///< Imagen del nombre del Jugador.
		
		SDL_Rect rjugador_nombre_;	///< Posición del nombre del Jugador.
		
		SDL_Surface *jugador_par_;	///< Imagen del par del Jugador.
		
		SDL_Rect rjugador_par_;	///< Posición del par del Jugador.
		
		SDL_Surface *palo_nombre_;	///< Imagen del nombre del Palo.
		
		SDL_Rect rpalo_nombre_;	///< Posición del nombre del Palo.
	
		Rejilla *rejilla_palos_;	///< Rejilla de imágenes de los palos.
		
		SDL_Rect rpalo_miniatura_;	///< Posición del Palo.
		
		SDL_Surface *palo_distancia_max_;	///< Imagen del alcance del Palo.
		
		SDL_Rect rpalo_distancia_max_;	///< Posición del alcance del Palo.
			
		SDL_Surface *distancia_hoyo_;	///< Imagen de la distancia al hoyo.
		
		SDL_Rect rdistancia_hoyo_;	///< Posición de la distancia al hoyo.
		
		SDL_Surface *campo_mapa_;	///< Imagen del mapa del Campo.
		
		SDL_Rect rcampo_mapa_;	///< Posición del mapa del Campo.
		
		Rejilla *rejilla_jugador_mapa_;	///< Rejilla del Jugador en el mapa.
		
		SDL_Rect rjugador_mapa_;	///< Posición del Jugador en el mapa.
		
		SDL_Surface *cursor_mapa_;	///< Imagen del Cursor en el mapa.
		
		SDL_Rect rcursor_mapa_;	///< Posición del Cursor en el mapa.
		
		SDL_Surface *bandera_mapa_;	///< Imagen de la bandera en el mapa.
		
		SDL_Rect rbandera_mapa_;	///< Posición de la bandera en el mapa.
};

#endif
