/// \file jugador.cpp Jugador.
/*
	Copyright 2010 Andrés Francisco Aparicio

	This file is part of SDLgolf.

	SDLgolf is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	SDLgolf is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with SDLgolf.  If not, see <http://www.gnu.org/licenses/>.
*/

#include <SDL/SDL.h>
#include <SDL/SDL_image.h>
#include <iostream>
#include <math.h>
#include "jugador.h"
#include "partida.h"
#include "pantalla.h"
#include "campo.h"
#include "utilidades.h"
#include "camara.h"
#include "constantes.h"


Jugador::~Jugador()
{

	// Destruimos la rejilla del jugador.
	delete rejilla_;

	// Destruimos el cursor.
	delete cursor_;

	// Destruimos los indicadores de fuerza y efecto.
	delete indicador_;
	
	// Destruimos todas las animaciones.
	for(std::vector<Animacion*>::iterator i = animaciones_.begin();
		i != animaciones_.end(); i++)
		delete *i;
}

void Jugador::Dibujar() const
{
	// Dibujamos el jugador.
	pantalla_ -> Dibujar(rejilla_ -> Imagen(), &(rejilla_ -> Posicion()),
		&rjugador_);
}

const Palo& Jugador::PaloActual() const
{
	// Devolvemos el palo actual.
	return palos_.at(palo_actual_);
}

void Jugador::PaloSiguiente()
{
	// Cambiamos al siguiente palo.
	// Si la bolá esta en un bunker no podemos cambiar de palo.
	if(bola_ -> Estado() != Bola::BUNKER)
	{
		if(palo_actual_ == 0)
			palo_actual_ = palos_.size() - 1;
		else
			palo_actual_--;
	}
}

void Jugador::PaloAnterior()
{
	// Cambiamos al palo anterior.
	// Si la bolá esta en un bunker no podemos cambiar de palo.
	if(bola_ -> Estado() != Bola::BUNKER)
	{
		if(palo_actual_ == palos_.size() - 1)
			palo_actual_ = 0;
		else
			++palo_actual_;
	}
}

void Jugador::IncrementarPar()
{
	// Incrementamos el par del jugador.
	++par_;
}

void Jugador::EstablecerEstado(int estado)
{
	// Establecemos el estado del jugador.
	estado_ = estado;
}

int Jugador::EstadoActual() const
{
	// Devolvemos el estado del jugador.
	return estado_;
}

Bola& Jugador::BolaEnJuego() const
{
	// Devolvemos la bola del jugador.
	return *bola_;
}

int Jugador::DistanciaHoyo() const
{
	// Devolvemos la distancia a la que está la bola respecto al hoyo.
	return bola_ -> DistanciaHoyo(partida_ -> CampoActivo().PosicionHoyo());
}

void Jugador::OrientarCursor()
{
	// Orientamos el cursor al hoyo.
	cursor_ -> Orientar();
}

void Jugador::CursorDerecha()
{
	// Giramos el cursor a la derecha.
	cursor_ -> Derecha();
}

void Jugador::CursorIzquierda()
{
	// Giramos el cursor a la izquierda.
	cursor_ -> Izquierda();
}

std::string Jugador::Nombre()
{
	// Devolvemos el nombre del jugador.
	return nombre_;
}

void Jugador::Nombre(std::string nombre)
{
	// Establecemos el nombre del jugador.
	nombre_ = nombre;
}

int Jugador::Par() const
{
	// Devolvemos el par del jugador.
	return par_;
}

void Jugador::Equipo(int equipo)
{
	// Establecemos el equipo al que pertenece el jugador.
	equipo_ = equipo;
}

int Jugador::Equipo() const
{
	// Devolvemos el equipo al que pertenece el jugador.
	return equipo_;
}

void Jugador::Reiniciar()
{
	// Ponemos el par a 0 y el estado a inactivo.
	par_ = 0;
	estado_ = INACTIVO;
	
	// Seleccionamos Madera 1.
	SeleccionarPalo("Madera 1");
}

void Jugador::Par(int par)
{
	// Establecemos el par del jugador.
	par_ = par;
}

Cursor& Jugador::CursorActual()
{
	// Devolvemos el cursor del jugador.
	return *cursor_;
}

Indicador& Jugador::IndicadorActual()
{
	// Devolvemos el indicador del jugador.
	return *indicador_;
}

int Jugador::NumeroDePalos() const
{
	// Devolvemos el número de palos que tiene el jugador.
	return palos_.size();
}

void Jugador::ActualizarCuadro()
{
	// En función del cuadrante en el que esté el cursor, hemos de elegir
	// un cuadro u otro.
	int cuadrante = cursor_ -> Cuadrante();

	// Ángulo que forma el cursor con la bola.
	float angulo = cursor_ -> Angulo() * 180 / M_PI;

	// Cuadro de la rejilla del jugador.
	int cuadro = 0;
	
	// El ángulo es mayor que 70.
	if(angulo > 70)
	{
		switch(cuadrante)
		{
			case 1:	cuadro = 0; break;
			case 2: cuadro = 0; break;
			case 3: cuadro = 36; break;
			case 4: cuadro = 36; break;
			default: break;
		}
	}
	
	// El ángulo es menor que 70 y mayor que 20.
	if(angulo < 70 && angulo > 20)
	{
		switch(cuadrante)
		{
			case 1:	cuadro = 63; break;
			case 2: cuadro = 9; break;
			case 3: cuadro = 27; break;
			case 4: cuadro = 45; break;
			default: break;
		}	
	}
	
	// El ángulo es menor que 20.
	if(angulo < 20)
	{
		switch(cuadrante)
		{
			case 1:	cuadro = 54; break;
			case 2: cuadro = 18; break;
			case 3: cuadro = 18; break;
			case 4: cuadro = 54; break;
			default: break;
		}		
	}

	// Si el cuadrante es correcto, realizamos el cambio de cuadro del jugador.
	if(cuadrante != 0)
	{
		rejilla_ -> SeleccionarCuadro(cuadro);
		this -> ActualizarPosicion();
	}
	
	// Al cambiar de cuadro, tenemos que actualizar la posición del indicador.
	indicador_ -> ActualizarPosicion();
}

void Jugador::ActualizarPosicion()
{
	// Desplazamiento de la imagen del jugador en función del cuadro.
	int desplazamiento_x = 0;
	int desplazamiento_y = 0;

	// Determinamos en que fila de la rejilla nos encontramos.
	int fila_rejilla = rejilla_ -> CuadroActual() / 9;

	// Elegimos los desplazamientos en función de la fila.
	switch(fila_rejilla)
	{
		case 0: desplazamiento_x = -29;	desplazamiento_y = -32; break;
		case 1: desplazamiento_x = -23; desplazamiento_y = -28; break;
		case 2: desplazamiento_x = -13; desplazamiento_y = -22; break;
		case 3: desplazamiento_x = -5;	desplazamiento_y = -30; break;
		case 4: desplazamiento_x = 0;		desplazamiento_y = -32; break;
		case 5: desplazamiento_x = -12; desplazamiento_y = -38; break;
		case 6: desplazamiento_x = -17; desplazamiento_y = -37; break;
		case 7: desplazamiento_x = -23; desplazamiento_y = -38; break;
		default: break;
	}
	
	// Actualizamos las coordenadas de posición de la imagen del jugador.
	rjugador_.x = bola_ -> PosicionBola().x + desplazamiento_x;
	rjugador_.y = bola_ -> PosicionBola().y + desplazamiento_y;
}

void Jugador::AnimacionPrevia()
{
	// Variable de control de tiempo para el bucle principal.
	Uint32 ta, tb;

	// Variable de control de tiempo para la animación.
	Uint32 ta_animacion, tb_animacion;
	
	// Animación a procesar.
	Animacion *animacion;

	// Retardo de la animación a procesar.
	Uint32 retardo;

	// Cuadro en el que pararemos.
	int parada;

	// Indice de la animación que tenemos que dibujar.
	int indice;

	// Determinamos la animación que tenemos que llevar a cabo en función
	// del cuadro de la rejilla que estemos dibujando. De la misma forma,
	// establecemos cuantos cuadros tenemos que dibujar antes de parar.
	switch(rejilla_ -> CuadroActual())
	{
		case 0: 	indice = 0; parada = 7; break;
		case 9: 	indice = 1; parada = 7; break;		
		case 18:	indice = 2; parada = 7; break;
		case 27:	indice = 3; parada = 8; break;
		case 36:	indice = 4; parada = 6; break;
		case 45:	indice = 5; parada = 5; break;
		case 54:	indice = 6; parada = 7; break;
		case 63:	indice = 7; parada = 7; break;
		default:	indice = 0; parada = 0; break;
	}

	// Seleccionamos la animación.
	animacion = animaciones_[indice];
	
	// Tomamos el retardo de la animación.
	retardo = animaciones_[indice] -> Retardo();

	// Establecemos la posición en la que se dibujará la animación.
	animacion -> Posicion(rjugador_);
	
	// Variable del bucle.
	int x = 0;
	
	// Tiempo para el control de la animación.
	ta_animacion = SDL_GetTicks();
	
	// Mientras que no hayamos llegado al cuadro de parada ejecutamos.
	while(x < parada)
	{
		// Guardamos el tiempo para el control del bucle.
		ta = SDL_GetTicks();

		// Guardamos el tiempo para el control de la animación.
		tb_animacion = SDL_GetTicks();
		
		// Si el tiempo que ha pasado es mayor que el retardo, dibujamos.
		if(tb_animacion - ta_animacion > retardo)
		{
			// Dibujamos.
			partida_ -> CampoActivo().Dibujar();
			partida_ -> DibujarBolasInactivas();
			bola_ -> DibujarSombra();
			bola_ -> DibujarBola();
			animacion -> Dibujar();
			partida_ -> CampoActivo().DibujarBandera();
			partida_ -> DibujarInterfaz();
			pantalla_ -> Actualizar();

			// Incrementamose el cuadro.
			++x;

			// Tomamos las nuevas referencias de tiempo.
			ta_animacion = SDL_GetTicks();
			tb_animacion = SDL_GetTicks();
		}

		// Guardamos el tiempo.
		tb = SDL_GetTicks();
		
		// Si el tiempo transcurrido es menor que el establecido en el juego
		// tenemos que esperar.
		if(tb - ta < Constantes::_MS_)
			SDL_Delay(Constantes::_MS_ - (tb - ta));
	}
}

Animacion& Jugador::AnimacionActual()
{
	// Determinamos la animación actual en función de la fila de la rejilla
	// en la que nos encontremos.
	switch(rejilla_ -> CuadroActual() / 9)
	{
		case 0:	return *(animaciones_[0]);
		case 1: return *(animaciones_[1]);
		case 2: return *(animaciones_[2]);
		case 3: return *(animaciones_[3]);
		case 4: return *(animaciones_[4]);
		case 5: return *(animaciones_[5]);
		case 6: return *(animaciones_[6]);
		case 7: return *(animaciones_[7]);
		default: break;
	}
	return *(animaciones_[0]);
}

int Jugador::CuadroActual() const
{
	// Devolvemos el cuadro actual de la rejilla.
	return rejilla_ -> CuadroActual();
}

void Jugador::SeleccionarCuadro(int cuadro)
{
	// Seleccionamos el cuadro de la rejilla.
	rejilla_ -> SeleccionarCuadro(cuadro);
}

Campo& Jugador::CampoActual() const
{
	// Devolvemos el campo en el que se encuentra el jugador.
	return partida_ -> CampoActivo();
}

void Jugador::SeleccionarPalo(std::string palo)
{
	// Seleccionamos el palo.
	while(palos_[palo_actual_].Nombre() != palo)
	{
		if(palo_actual_ == 0)
			palo_actual_ = palos_.size() - 1;
		else
			palo_actual_--;
	}
}

int Jugador::Color() const
{
	// Devolvemos el color del jugador.
	return color_;
}

int Jugador::Tipo() const
{
	// Devolvemos el tipo del jugador.
	return tipo_;
}

int Jugador::IndicePaloActual() const
{
	return palo_actual_;
}

