/// \file jugador.h Jugador.
/*
	Copyright 2010 Andrés Francisco Aparicio

	This file is part of SDLgolf.

	SDLgolf is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	SDLgolf is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with SDLgolf.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef _JUGADOR_H_
#define _JUGADOR_H_

#include <SDL/SDL.h>
#include <SDL/SDL_image.h>
#include <iostream>
#include <deque>
#include <vector>
#include <string>
#include "datos.h"
#include "bola.h"
#include "campo.h"
#include "cursor.h"
#include "palo.h"
#include "utilidades.h"
#include "indicador.h"
#include "rejilla.h"
#include "animacion.h"


class Partida;

/*! \class Jugador
    \brief Clase virtual.

    Los jugadores pueden ser personas o estar controlados por el ordenador.
    
    Esta clase engloba todos los métodos necesarios para la implementación
    de los jugadores.
    
    Todos los jugadores tienen un nombre, un color y
		pueden pertenecer a un equipo cuando se está jugando una partida por
		parejas. De la misma forma, los jugadores de un mismo equipo podrán
		compartir la bola de golf. A parte de esto, el jugador cuenta con una
		lista de palos de golf que podrá utilizar durante la partida.
 */
class Jugador
{
	public:

		/*! Destruye la Rejilla, el Cursor, el Indicador de fuerza y efecto y
				todas las animaciones.
				\brief Destructor.
		 */
		~Jugador();
		
		/*! El Jugador golpea la Bola.
				\brief Golpeo de la Bola.
				\return 0 cuando el lanzamiento ha finalizado.
		 */			
		virtual int GolpearBola() = 0;

		/*! \brief Tipo de jugador.
				\return Tipo de jugador.
				\see Tipos.
		 */
		int Tipo() const;

		/*! Dibuja el cuadro del Jugador en la Pantalla.
				\brief Dibuja el jugador.
				\see CuadroActual.
		 */
		void Dibujar() const;

		/*! Actualiza la posición del Jugador en función del cuadro de la
				Rejilla que esté seleccionado.
				\brief Actualiza la posición dependiendo de la imagen de la rejilla.
		 */		
		void ActualizarPosicion();

		/*! Actualiza el cuadro de la Rejilla en función de la posición del
				Cursor.
				\brief Actualiza la imagen del jugador dependiendo de donde apunte
							 el Cursor.
		 */	
		void ActualizarCuadro();

		/*! Selecciona un determinado cuadro en la Rejilla.
				\brief Selecciona un cuadro de la Rejilla.
				\param cuadro Número de cuadro a seleccionar.
		 */
		void SeleccionarCuadro(int cuadro);

		/*! \brief Cuadro del jugador seleccionado.
				\return Cuadro actual seleccionado en la Rejilla.
		 */
		int CuadroActual() const;

		/*! Lleva a cabo la secuencia de animación del jugador que se
				corresponde con el intervalo de tiempo entre que el jugador prepara
				el tiro hasta que golpea la Bola.
				\brief Animación previa al lanzamiento.
		 */
		void AnimacionPrevia();

		/*! Devuelve la Animacion del jugador.
				\brief Animación del jugador.
				\return Animación actual del jugador en función del cuadro de la
								Rejilla.
		 */
		Animacion& AnimacionActual();

		/*! \brief Establece el nombre del jugador.
				\param nombre Nombre del jugador.
		 */	
		void Nombre(std::string nombre);

		/*! \brief Nombre del jugador.
				\return Nombre del jugador.
		 */
		std::string Nombre();

		/*! \brief Establece el estado del jugador.
				\param estado Estado del Jugador.
				\see Estados.
		 */
		void EstablecerEstado(int estado);

		/*! \brief Selecciona el Palo siguiente al actual.
				\see PaloActual.
		 */
		void PaloSiguiente();

		/*! \brief Selecciona el Palo anterior al actual.
				\see PaloActual.
		 */
		void PaloAnterior();

		/*! \brief Incrementa en una unidad el par del Jugador.
				\see Par.
		 */
		void IncrementarPar();

		/*! \brief Orienta el Cursor del jugador al hoyo del Campo.
				\see Cursor::Orientar.
		 */
		void OrientarCursor();

		/*! \brief Gira el Cursor del jugador a la derecha.
				\see Cursor::Derecha.
		 */
		void CursorDerecha();

		/*! \brief Gira el Cursor del jugador a la izquierda.
				\see Cursor::Izquierda.
		 */
		void CursorIzquierda();

		/*! \brief Establece el par del jugador a 0 y su estado a inactivo.
				\see Par.
				\see Estados.
		 */
		void Reiniciar();

		/*! \brief Estado del jugador.
				\return Estado del jugador.
				\see Estados.
		 */
		int EstadoActual() const;

		/*! \brief Bola del jugador.
				\return Bola del jugador.
		 */
		Bola& BolaEnJuego() const;

		/*! \brief Campo en el que se está jugando.
				\return Campo en el que se desarrolla la Partida actual.
		 */
		Campo& CampoActual() const;

		/*! \brief Palo seleccionado por el jugador.
				\return Palo que el Jugador tiene seleccionado actualmente.
		 */
		const Palo& PaloActual() const;

		/*! \brief Palo seleccionado por el jugador.
				\return Palo que el Jugador tiene seleccionado actualmente.
		 */
		int IndicePaloActual() const;

		/*! \brief Cursor del jugador.
				\return Cursor del Jugador.
		 */
		Cursor& CursorActual();

		/*! \brief Indicador de fuerza y efecto del jugador.
				\return Indicador del Jugador.
		 */
		Indicador& IndicadorActual();
		
		/*! \brief Número de palos disponibles.
				\return Número de palos disponibles.
		 */
		int NumeroDePalos() const;

		/*! \brief Selecciona el Palo indicado.
				\param palo Nombre del palo que queremos seleccionar.
		 */
		void SeleccionarPalo(std::string palo);
		
		/*! \brief Color del jugador.
				\return Color del jugador.
		 */
		int Color() const;
			
		/*! \brief Establece el equipo al que pertenece el jugador
				\param equipo Número de equipo del jugador.
		 */
		void Equipo(int equipo);

		/*! \brief Número de equipo del jugador.
				\return Número de equipo del jugador.
		 */
		int Equipo() const;
		
		/*! \brief Establece el par del jugador
				\param par Par del Jugador.
		 */
		void Par(int par);

		/*! \brief Par del jugador.
				\return Par del Jugador.
		 */
		int Par() const;

		/*! \brief Distancia al hoyo en línea recta.
				\return Distancia al hoyo del Campo a la que se encuentra el Jugador.
		 */
		int DistanciaHoyo() const;

		/*! \enum Estados
		  	Estados en los que puede encontrarse el jugador.
		  	
		  	\brief Estados del jugador.

				\var ACTIVO
		 		El jugador está jugando.
		 		
		 		\var INACTIVO
		 		El jugador está a la espera de entrar en juego.
		 		
		 		\var FINALIZADO
		 		El jugador ha efectuado hoyo.

		 */	
		enum Estados { ACTIVO, INACTIVO, FINALIZADO };

		/*! \enum Tipos
		  	Tipos de jugadores a los que puede pertenecer un jugador.
		  	
		  	\brief Tipos de jugadores.

				\var HUMANO
		 		El jugador es una persona.

				\var IA_BAJA
		 		El jugador es el ordenador y la inteligencia artificial es baja.

				\var IA_MEDIA
		 		El jugador es el ordenador y la inteligencia artificial es media.
		 		
				\var IA_ALTA
		 		El jugador es el ordenador y la inteligencia artificial es alta.		 		
		 */	
		enum Tipos { HUMANO, IA_BAJA, IA_MEDIA, IA_ALTA };
		
	protected:

		int tipo_;	///< Tipo de Jugador.
		
		Rejilla *rejilla_;	///< Rejilla de imágenes del Jugador.
		
		SDL_Rect rjugador_;	///< Posición de la imagen del Jugador.

		std::string nombre_;	///< Nombre del Jugador.
		
		int equipo_;	///< Equipo al que pertenece el Jugador.
		
		int par_;	///< Par del Jugador.
		
		int color_;	///< Color del Jugador.
		
		int estado_;	///< Estado actual del Jugador.
		
		std::vector<Animacion *> animaciones_;	///< Animaciones del Jugador.
		
		Bola *bola_;	///< Bola asociada al Jugador.
		
		std::deque<Palo> palos_;	///< Lista de palos del Jugador.
		
		unsigned palo_actual_;	///< Palo seleccionado actualmente.
		
		Cursor *cursor_;	///< Cursor del Jugador.
		
		Indicador *indicador_;	///< Indicador de fuerza y efecto del Jugador.
		
		Partida *partida_;	///< Partida que está actualmente en juego.
		
		Pantalla *pantalla_;	///< Pantalla del Juego.
};

#endif
