/// \file matchplay.h Modo de juego Matchplay.
/*
	Copyright 2010 Andrés Francisco Aparicio

	This file is part of SDLgolf.

	SDLgolf is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	SDLgolf is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with SDLgolf.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef _MATCHPLAY_H_
#define _MATCHPLAY_H_

#include "partida.h"
#include "datos.h"


/*! \class Matchplay
    \brief Modo de juego Matchplay.

    Implementación del modo de juego Matchplay.
    
    En el modo Matchplay, dos o más jugadores se enfrentan por parejas
    en todos los hoyos de un campo.
    
    El equipo o el jugador que gane un hoyo
    sumará un punto. El equipo o el jugador que pierda el hoyo no sumará nada.
    Si los equipos o los jugadores empatan un hoyo, tampoco sumarán puntos.
    
    Los miembros de un mismo equipo podrán compartir la bola. Si se comparte
    la bola, los jugadores también compartirán el par.
    
    Ganará la partida el equipo o el jugador que más puntos haya conseguido.
 */
class Matchplay: public Partida
{
	public:

		/*! Realiza las asociaciones con el Menu y la Pantalla.\n
				Crea y asigna las bolas de golf y los jugadores de los diferentes
				equipos.\n
				Inicializa las puntuaciones a 0 y carga el primer Campo.\n
				Coloca las Bolas sobre el Campo, actualiza la lista de bolas inactivas
				y activa el primer Jugador.\n
				Crea las animaciones de las colisiones de la Bola con los elementos
				del Campo.
				
				\brief Constructor.
				\param jugadores Datos de los jugadores que participan en la Partida.
				\param campos Datos de los campos en los que se desarrollará la
							 Partida.
				\param modo Modo de juego para Matchplay.
				\param menu Menú del Juego.
				\param pantalla Pantalla del Juego.
				\see Modos.
				\see Campo::LugarSalida.
				\see Campo::ElementosCampo.
				\see Datos::Jugador.
				\see Datos::Campo.
		 */
		Matchplay(std::vector<Datos::Jugador>& jugadores,
			std::vector<Datos::Campo>& campos, int modo, Menu *menu,
			Pantalla *pantalla);

		/*! \brief Inicia la partida.
				\return \arg \b 0 hemos salido de la Partida desde el Menu de pausa.
								\arg \b -1 hemos cerrado la ventana del Juego.
		 */
		int Iniciar();

		/*! Pasa al siguiente hoyo, destruyendo el actual y modificando el
				campo activo.
				\brief Pasa al siguiente hoyo.
				\return \arg \b Verdadero en caso de haber más hoyos.
								\arg \b Falso en caso de no haber más hoyos.
		 */
		bool SiguienteHoyo();

		/*! Finaliza los jugadores del mismo equipo que posean el mismo par
				que el jugador activo o un par menor en una unidad, puesto que
				no podrán mejorar la puntuación del equipo.
				\brief Finaliza los jugadores del mismo par.
				\param par Par que posee el jugador activo.
				\param equipo Equipo al que pertenece el jugador activo.
				\see Jugador::Par.
		 */
		void FinalizarMismoPar(int par, int equipo);

		/*! \brief Finaliza a todos los jugadores de un equipo.
				\param equipo Equipo a finalizar.
				\see Jugador::Equipo.
		 */
		void FinalizarEquipo(int equipo);

		/*! Establece el mismo par a los jugadores que estén en el mismo equipo.
				\brief Establece el mismo par a un equipo.
				\param par Par que posee el jugador activo.
				\param equipo Equipo al que pertenece el jugador activo.
				\see Jugador::Par.
				\see Jugador::Equipo.
		 */
		void EstablecerMismoPar(int par, int equipo);
		
		/*! \brief Reinicia todos los jugadores de la Partida.
				\see Jugador::Reiniciar()
		 */
		void ReiniciarJugadores();

		/*! Actualiza las puntuaciones de los equipos. \n
				El equipo que posea el menor par sumará un punto. Si los equipos
				tienen el mismo par no suman nada.
				\brief Actualiza las puntuaciones.
		 */
		void ActualizarPuntuaciones();

		/*! \enum Modos

		  	\brief Modos de juego para Matchplay.

				\var BOLAS_INDEPENDIENTES
		 		Cada jugador tiene su propia Bola.

				\var BOLAS_COMPARTIDAS
		 		Los jugadores del mismo equipo comparten la Bola.
		 */	
		enum Modos { BOLAS_INDEPENDIENTES, BOLAS_COMPARTIDAS };

	private:

		std::vector<Datos::Campo>
			datos_campos_;	///< Datos de los campos de la Partida.
		
		int modo_;	///< Modo de Matchplay.
		
		
		unsigned int hoyo_actual_;	///< Hoyo en el que se está jugando.
};

#endif
