/// \file menu.h Menú del juego.
/*
	Copyright 2010 Andrés Francisco Aparicio

	This file is part of SDLgolf.

	SDLgolf is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	SDLgolf is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with SDLgolf.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef _MENU_H_
#define _MENU_H_

#include "partida.h"
#include "datos.h"
#include "sonido.h"

class Pantalla;
class Partida;
class Rejilla;
class Jugador;
class Musica;

/*! \class Menu
    \brief Menú del juego.

   	El menú del juego permite al jugador seleccionar los diferentes modos
   	de juego, las opciones, obtener información sobre SDLgolf o salir
   	de la aplicación.
   	Esta clase incluye todos los submenús, como los de elección de jugador,
   	elección de campo, pantallas de resumen y menú de pausa.
 */
class Menu
{
	public:

		/*! Carga las imágenes principales del menú del juego, la música, los
				efectos de sonido y el fichero de nombres.
				\brief Constructor.
				\param pantalla Pantalla del Juego.
		 */
		Menu(Pantalla *pantalla);

		/*! Destruye todas las imágenes, rejillas y el vector del fichero
				de nombres.
				\brief Destructor.
		 */
		~Menu();

		/*! \brief Muestra el menú principal del Juego.
				\return \b 0 Si se sale del juego o se cierra la ventana.
		 */		
		int Principal();

		/*! \brief Muestra el menú de elección de jugadores.
				\param modo Modo de juego.
				\return \arg \b -1 Si se ha cerrado la ventana del Juego.
								\arg \b 0 En otro caso.
				\see Modos.
		 */	
		int ElegirJugadores(int modo);

		/*! \brief Muestra el menú de elección de pareja para el modo Matchplay.
				\return \arg \b -1 Si se ha cerrado la ventana del Juego.
								\arg \b 1 Si se ha pulsado el botón de volver.
								\arg \b 0 En otro caso.
		 */	
		int ElegirPareja();

		/*! \brief Muestra el menú de elección de Campo.
				\param modo Modo de juego.
				\return \arg \b -1 Si se ha cerrado la ventana del Juego.
								\arg \b 1 Si se ha pulsado el botón de volver.
								\arg \b 0 En otro caso.
				\see Modos.
		 */
		int ElegirCampo(int modo);

		/*! \brief Inicia la partida en función del modo de juego.
				\param modo Modo de juego.
				\return \arg \b -1 Si se ha cerrado la ventana del Juego.
								\arg \b 1 Si se ha pulsado el botón de volver.
								\arg \b 0 En otro caso.
				\see Modos.
		 */
		int IniciarPartida(int modo);

		/*! \brief Muestra el menú de opciones.
				\return \arg \b -1 Si se ha cerrado la ventana del Juego.
								\arg \b 0 En otro caso.
				\see GuardarOpciones.
		 */	
		int Opciones();

		/*! \brief Muestra el menú de información.
				\return \arg \b -1 Si se ha cerrado la ventana del Juego.
								\arg \b 0 En otro caso.
		 */	
		void Acercade();

		/*! \brief Muestra el menú de finalización de partida.
				\param modo Modo de juego.
				\return \arg \b -1 Si se ha cerrado la ventana del Juego.
								\arg \b 0 En otro caso.
				\see Modos.
		 */	
		int Finalizar(int modo);

		/*! \brief Ejecuta el menú de pausa.
				\return \arg \b -1 Si se sale de la Partida.
								\arg \b 1 Si se ha cerrado la ventana del Juego.
								\arg \b 0 Si se continua la Partida.
		 */
		int Pausa();		

		/*! \brief Dibuja el menú principal.
		 */	
		void DibujarPrincipal() const;

		/*! \brief Dibuja el menú de elección de jugadores.
				\param modo Modo de juego.
				\see Modos.
		 */	
		void DibujarElegirJugadores(int modo) const;

		/*! \brief Dibuja el menú de elección de modo de Matchplay.
				\see Matchplay::Modos.
		*/	
		void DibujarElegirModoMatchplay() const;

		/*! \brief Dibuja el menú de elección de pareja.
				\see Jugador::Equipo.
		 */	
		void DibujarElegirPareja() const;

		/*! \brief Dibuja el menú de elección de campo.
				\param modo Modo de juego.
				\see Modos.
		 */	
		void DibujarElegirCampo(int modo) const;

		/*! \brief Dibuja el menú de acerca de.
		 */
		void DibujarAcercaDe() const;

		/*! \brief Dibuja el menú de opciones.
		 */
		void DibujarOpciones() const;
		
		/*! \brief Indica el estado de los efectos de sonido.
				\return \arg \b Verdadero si los efectos de sonido están activados.
								\arg \b Falso si los efectos de sonido están desactivados.
		 */
		bool EfectosActivos() const;

		/*! \brief Indica el estado de la música.
				\return \arg \b Verdadero si la música está activada.
								\arg \b Falso si la música está desactivada.
		 */
		bool MusicaActiva() const;

		/*! El nombre es seleccionado de una lista de nombres cargada previamente
				en memoria.
				\brief Genera un nombre aleatorio para el jugador indicado.
				\param jugador Número de jugador.
				\param modo Modo de juego.
				\see Utilidades::NumeroAleatorio.
		 */
		void JugadorAleatorio(int jugador, int modo);
		
		/*! \brief Número de jugadores.
				\return Número de jugadores creados que participarán en la Partida.
		 */
		int Jugadores() const;

		/*! Elimina las imágenes de los nombres, tipos y colores de los jugadores
				creados.
				\brief Elimina datos de la elección de jugador.
		 */		
		void EliminarDatosElegirJugador();

		/*! Elimina la imagen del campo, el nombre del campo y el número de hoyo.
				\brief Elimina datos de la elección de campo.
		 */	
		void EliminarDatosElegirCampo();

		/*! \brief Elimina las imágenes de los textos.
		 */	
		void EliminarDatosElegirPareja();

		/*! \brief Elimina la imagen del nombre del jugador.
				\param jugador Número de jugador.
		 */	
		void EliminarNombre(int jugador);
		
		/*! \brief Crea la imagen del tipo del jugador.
				\param jugador Número de jugador.
				\param tipo Tipo de jugador.
				\see Jugador::Tipos.
		 */	
		void Tipo(int jugador, int tipo);

		/*! \brief Establece el siguiente tipo para el jugador.
				\param jugador Número de jugador.
				\see Jugador::Tipos.
		 */	
		void SiguienteTipo(int jugador);

		/*! \brief Elimina la imagen del tipo del jugador.
				\param jugador Número de jugador.
		 */	
		void EliminarTipo(int jugador);
		
		/*! \brief Selecciona de la rejilla de imágenes el color del jugador.
				\param jugador Número de jugador.
				\param color Color del jugador.
				\see Jugador::Tipos.
		 */	
		void EstablecerColor(int jugador, int color);

		/*! \brief Establece el siguiente color para el jugador.
				\param jugador Número de jugador.
		 */
		void SiguienteColor(int jugador);

		/*! \brief Elimina la imagen del color del jugador.
				\param jugador Número de jugador.
		 */	
		void EliminarColor(int jugador);
		
		/*! Captura texto del teclado y genera la imagen del nombre del jugador.
				\brief Captura las pulsaciones del teclado.
				\param jugador Número de jugador.
				\param modo Modo de juego.
				\return \arg \b 1 Si se ha cerrado la ventana del Juego.
								\arg \b 0 En otro caso.
				\see Modos.
		 */
		int CapturarTexto(int jugador, int modo);

		/*! Lee las opciones del juego del fichero de configuración y establece
				los parámetros.
				\brief Lee las opciones.
		 */
		void LeerOpciones();

		/*! Guarda las opciones del juego en el fichero de configuración.
				\brief Guarda las opciones.
		 */
		void GuardarOpciones();

		/*! \enum Modos

		  	\brief Modos de juego disponibles.

				\var PRACTICAR
		 		Uno o varios jugadores pueden practicar un determinado hoyo de un
		 		campo.

				\var MATCHPLAY
		 		Los jugadores se enfrentan por parejas realizando todos los hoyos de
		 		un campo.\n
		 		El equipo (o el jugador, en caso de haber dos contrincantes)
		 		que gane el mayor número de hoyos ganará la partida.\n
		 		Si un equipo o un jugador gana un hoyo, sumará 1 punto. Si empatan,
		 		ambos sumarán 0 puntos.
		 		
		 		\var STROKEPLAY
		 		Los jugadores realizan todos los hoyos de un campo. El jugador que
		 		tenga en el cómputo total el menor par, ganará la partida.\n
		 		Las puntuaciones de los jugadores para cada hoyo se obtendrán restando
		 		al par del jugador el par del hoyo.
		 		
		 */	
		 
		enum Modos {PRACTICAR, MATCHPLAY, STROKEPLAY};

	private:

		/*! Genera una ruta desde el directorio de imágenes del menú.
				\brief Genera ruta a imagen.
				\param imagen Nombre de la imagen.
				\return Ruta a la imagen.
		 */	
		std::string RutaImagen(const char *imagen);

		/*! Genera una ruta a un sonido desde el directorio de efectos de sonido.
				\brief Genera ruta a sonido.
				\param imagen Nombre del sonido.
				\return Ruta al sonido.
		 */					
		std::string RutaSonido(const char *sonido);

		int njugadores_;	///< Número de jugadores.

		int ncolores_;	///< Número de colores.

		int ntipos_;	///< Número de tipos.

		char *fichero_;	///< Fichero de nombres para la generación aleatoria.

		int longitud_;	///< Longitud del fichero.
		
		bool efectos_;	///< Estado de los efectos de sonido.

		bool musica_;	///< Estado de la música del juego.
		
		int tipo_matchplay_;	///< Tipo de Matchplay que se va a jugar.

		SDL_Rect rfondo_;	///< Posición de las imágenes de fondo del menú.

		SDL_Surface *principal_;	///< Imagen del menú principal.
		
		SDL_Surface *practicar_;	///< Imagen del menú practicar.

		SDL_Surface *practicar_campo_;	///< Imagen del menú de elección de 
																		///< campo del modo practicar.

		SDL_Surface *matchplay_;	///< Imagen del menú matchplay.
		
		SDL_Surface *matchplay_campo_;	///< Imagen del menú de elección de 
																		///< campo del modo matchplay.

		SDL_Surface *strokeplay_;	///< Imagen del menú strokeplay.

		SDL_Surface *strokeplay_campo_;	///< Imagen del menú de elección de 
																		///< campo del modo strokeplay.

		SDL_Surface *selector_;	///< Imagen del selector del menú.

		SDL_Rect rselector_;	///< Posición del selector del menú.
		
		std::vector<SDL_Surface *> sjugadores_;	///< Imágenes de los nombres de
																						///< los jugadores.

		std::vector<SDL_Rect> rjugadores_;	///< Posición de las imágenes de
																				///< los nombres de los jugadores.

		std::vector<std::string> jugadores_;	///< Cadenas de texto con los
																					///< nombres de los jugadores.
		
		std::vector<SDL_Surface *> stipos_;	///< Imágenes de los tipos de
																				///< los jugadores.

		std::vector<SDL_Rect> rtipos_; 	///< Posición de las imágenes de
																		///< los tipos de los jugadores.		

		std::vector<int> tipos_;	///< Cadenas de texto con los
															///< nombres de los jugadores.
		
		Rejilla *rejilla_colores_;	///< Imágenes de los colores de
																///< los jugadores.

		std::vector<SDL_Rect> rcolores_;	///< Posición de las imágenes de
																			///< los colores de los jugadores.

		std::vector<int> colores_;	///< Cadenas de texto con los
																///< colores de los jugadores.

		SDL_Surface *imagen_hoyo_;	///< Imagen del campo.
		
		SDL_Rect rimagen_hoyo_;	///< Posición de la imagen del campo.
		
		SDL_Surface *nombre_campo_;	///< Imagen del nombre del campo.

		SDL_Rect rnombre_campo_;	///< Posición del nombre del campo.

		SDL_Surface *numero_hoyo_;	///< Posición del numero de hoyo del campo.

		SDL_Rect rnumero_hoyo_;	///< Posición del nombre del campo.

		SDL_Surface *pausa_;	///< Imagen del menú de pausa.

		SDL_Surface *mpausa_;	///< Imagen del cuadro de opciones menú de pausa.

		SDL_Rect rmpausa_;	///< Posición del cuadro de opciones del menú de pausa.

		SDL_Surface *mseguro_;	///< Imagen del cuadro de confirmación del
													///< menú de pausa.

		SDL_Rect rmseguro_;	///< Posición del cuadro de confirmación del
												///< menú de pausa.

		SDL_Surface *matchplay_pareja_;	///< Imagen del menú de elección de
																		///< pareja del modo matchplay.
		
		SDL_Surface *matchplay_modo_;	///< Imagen del menú de tipo del
																	///< modo matchplay.

		std::vector<SDL_Surface *> stexto_pareja_;	///< Imágenes de los nombres
																								///< de las parejas.

		std::vector<SDL_Rect> rtexto_pareja_;	///< Posición de las imágenes de
																					///< los nombres de las parejas.

		SDL_Surface *bolas_compartidas_;	///< Imagen del texto de bolas
																			///< compartidas.

		SDL_Rect rbolas_compartidas_;	///< Posición de la imagen del texto
																	///< de bolas compartidas.

		SDL_Surface *bolas_independientes_;	///< Imagen del texto de bolas
																				///< independientes.

		SDL_Rect rbolas_independientes_;	///< Posición de la imagen del texto
																			///< de bolas independientes.
		
		SDL_Surface *matchplay_final_;	///< Imagen del menú de finalización
																		///< de Matchplay.
		
		SDL_Surface *strokeplay_final_;	///< Imagen del menú de finalización
																		///< de Strokeplay.


		SDL_Surface *practicar_final_;	///< Imagen del menú de finalización
																		///< de Practicar.

		SDL_Surface *acercade_;	///< Imagen del menú acercade.

		SDL_Surface *opciones_;	///< Imagen del menú de opciones.
		
		SDL_Surface *controles_;	///< Imagen de los controles del juego.

		SDL_Surface *tefectos_;	///< Imagen del texto del estado de los efectos
														///< de sonido.
		
		SDL_Rect rtefectos_;	///< Posición de la imagen del texto del estado
													///< de los efectos de sonido.

		SDL_Surface *tmusica_;	///< Imagen del texto del estado de la música
														///< del juego.

		SDL_Rect rtmusica_;		///< Posición de la imagen del texto del estado
													///< de la música del juego.

		SDL_Surface *tcontroles_;	///< Imagen del texto de los controles del
															///< juego.

		SDL_Rect rtcontroles_;	///< Posición del texto de los controles del
														///< juego.

		std::vector<Datos::Jugador>
			datos_jugadores_;	///< Datos de los jugadores que participaran en
												///< la partida.
																									
		std::vector<Datos::Campo>
			datos_campos_; ///< Datos de los campos en los que se jugará
										///< la partida.
		
		Sonido seleccionar_;	///< Sonido que se produce al seleccionar
													///< una opción.

		Sonido aceptar_;	///< Sonido que se produce al aceptar
												///< una opción.

		Sonido cancelar_;	///< Sonido que se produce al cancelar
										///< una opción.

		Sonido cambiar_;	///< Sonido que se produce al cambiar
											///< una opción.

		Musica musica_fondo_;	///< Música de fondo.

		Teclado teclado_;	///< Teclado del menú.
		Pantalla *pantalla_;	///< Pantalla del juego.
		Partida *partida_;	///< Partida que se creará.
		
};

#endif
