/// \file ordenador.cpp Inteligencia artificial.
/*
	Copyright 2010 Andrés Francisco Aparicio

	This file is part of SDLgolf.

	SDLgolf is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	SDLgolf is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with SDLgolf.  If not, see <http://www.gnu.org/licenses/>.
*/

#include <cmath>
#include <iostream>
#include <vector>
#include <algorithm>
#include "ordenador.h"
#include "partida.h"
#include "campo.h"
#include "utilidades.h"
#include "constantes.h"
#include "pantalla.h"



Ordenador::Ordenador(std::string nombre, int color, int tipo,
	Bola& bola, Partida *partida, Pantalla *pantalla)
{
	// Establecemos el estado del jugador a inactivo.
	estado_ = INACTIVO;

	// El par del jugador es 0.
	par_ = 0;

	// Guardamos el tipo del jugador.
	tipo_ = tipo;

	// Establecemos el nombre del jugador.
	nombre_ = nombre;

	// Establecemos el color del jugador.
	color_ = color;

	// Asociamos con la partida.
	partida_ = partida;

	// Asociamos con la pantalla.
	pantalla_ = pantalla;

	// Asociamos con la bola.
	bola_ = &bola;

	// Creamos la lista de palos y los insertamos.	
	palos_.push_back(Palo("Madera 1", 240, 45));
	palos_.push_back(Palo("Madera 3", 220, 46));
	palos_.push_back(Palo("Madera 5", 200, 47));
	palos_.push_back(Palo("Hierro 3", 180, 48));
	palos_.push_back(Palo("Hierro 4", 170, 49));
	palos_.push_back(Palo("Hierro 5", 160, 50));
	palos_.push_back(Palo("Hierro 6", 150, 52));
	palos_.push_back(Palo("Hierro 7", 140, 53));
	palos_.push_back(Palo("Hierro 8", 130, 54));
	palos_.push_back(Palo("Hierro 9", 120, 55));
	palos_.push_back(Palo("Pitching Wedge", 100, 60));
	palos_.push_back(Palo("Sand Wedge", 80, 70));
	palos_.push_back(Palo("Putter", 40, 1));
	palo_actual_ = 0;

	// Creamos y asociamos el nuevo cursor.	
	cursor_ = new Cursor(this, pantalla_);

	// Establecemos el color del jugador.
	std::string nombre_color;
	
	switch(color)
	{
		case 0: nombre_color = "rojo"; break;
		case 1: nombre_color = "marron"; break;
		case 2: nombre_color = "amarillo"; break;
		case 3: nombre_color = "magenta"; break;
		case 4: nombre_color = "azul"; break;
		case 5: nombre_color = "cyan"; break;
		default:
			std::cerr << "Ordenador::Ordenador(): Error, color inválido."
				<< std::endl;
			exit(1);
	}

	// Generamos la ruta en función del color del jugador.
	std::string ruta;
	ruta.append(Constantes::_IMG_JUGADOR_);
	ruta += nombre_color + ".png";

	// Creamos y asociamos la nueva rejilla de imágenes.	
	rejilla_ = new Rejilla(ruta, 8, 9);

	// Establecemos el ancho y el alto de la rejilla.
	rjugador_.w = rejilla_ -> Posicion().w;
	rjugador_.h = rejilla_ -> Posicion().y;

	// Establecemos las coordenadas iniciales de la rejilla.
	rjugador_.x = 0;
	rjugador_.y = 0;

	// Creamos y asociamos el nuevo indicador de fuerza y efecto.	
	indicador_ = new Indicador(this, pantalla_);

	// Guardamos las secuencias de animación.
	int s0[] = {0, 1, 3, 3, 1, 2, 0, 4, 5, 6, 6, 0, -1};
	int s1[] = {9, 10, 11, 12, 10, 10, 13, 14, 15, 16, 16, 9, -1};
	int s2[] = {18, 19, 20, 20, 19, 21, 18, 22, 23, 23, 18, -1};
	int s3[] = {27, 28, 29, 30, 30, 29, 28, 27, 31, 32, 33, 27, -1};
	// *** Secuencias particulares
	int s4[] = {36, 37, 38, 38, 39, 36, 40, 41, 42, 42, -1};
	int s5[] = {45, 46, 47, 47, 48, 49, 50, 51, 51, -1};
	int s6[] = {54, 55, 56, 56, 55, 55, 57, 58, 59, 59, 59, -1 };
	// ***
	int s7[] = {63, 64, 65, 66, 65, 64, 63, 68, 69, 70, 70, 63, -1};	

	// Creamos cada una de las secuencias y las guardamos.
	animaciones_.push_back(new Animacion (rejilla_, s0, 70, pantalla_));
	animaciones_.push_back(new Animacion (rejilla_, s1, 70, pantalla_));
	animaciones_.push_back(new Animacion (rejilla_, s2, 70, pantalla_));		
	animaciones_.push_back(new Animacion (rejilla_, s3, 70, pantalla_));
	animaciones_.push_back(new Animacion (rejilla_, s4, 70, pantalla_));
	animaciones_.push_back(new Animacion (rejilla_, s5, 70, pantalla_));
	animaciones_.push_back(new Animacion (rejilla_, s6, 70, pantalla_));
	animaciones_.push_back(new Animacion (rejilla_, s7, 70, pantalla_));
}

int Ordenador::GolpearBola()
{

	// Variable para el retorno de la comprobación de eventos.
	int retorno;

	// Comprobamos si se ha cerrado la ventana o si tenemos que volver al
	// menú principal.
	retorno = partida_ -> ComprobarEventos();
	if(retorno == -1)
		return -1;
	else if(retorno == 1)
		return 1;

	// Esperamos.
	partida_ -> Esperar(250, true, true, true, false, false, 0, 0);
	
	// Guardamos el palo original para luego poder restaurarlo.
	std::string palo_original = PaloActual().Nombre();

	// Seleccionamos el palo que llega más lejos.
	while(PaloActual().Nombre() != "Madera 1"
		&& bola_ -> Estado() != Bola::BUNKER)
		PaloSiguiente();

	// Variables para el tiro en línea recta.
	// Fuerza óptima.
	int fuerza = 0;

	// Palo óptmo.
	std::string palo;

	// Ángulo óptimo. En este caso coincide con el ángulo del cursor orientado.
	float angulo;

	// Distancia mínima al hoyo.
	float distancia = 10000;

	// Distancia a comprobar.
	int d = 10000;

	// Destino óptimo.
	SDL_Rect destino_optimo;
	destino_optimo.x = 0;
	destino_optimo.y = 0;

	// Coordenadas de destino.
	int x, y;

	// Alcance máximo del palo.
	float alcance_maximo;

	// Establecemos el alcance máximo.
	alcance_maximo = PaloActual().AlcanceMaximo();

	// Establecemos el ángulo del cursor.
	angulo = cursor_ -> Angulo(); 

	// Número de palos.
	int numero_palos;

	// Obtenemos el número de palos en caso de que la bola no este en el bunker.
	if(bola_ -> Estado() != Bola::BUNKER)
		numero_palos = NumeroDePalos();
	// Si la bola está en el bunker sólo tenemos un palo.
	else
		numero_palos = 1;

	// Comprobamos si se ha cerrado la ventana o si tenemos que volver al
	// menú principal.	
	retorno = partida_ -> ComprobarEventos();
	if(retorno == -1)
		return -1;
	else if(retorno == 1)
		return 1;
	
	// Vectores con las soluciones para el tiro directo, el tiro a la izquierda
	// y el tiro a la derecha.
	std::vector<Solucion> s;
	std::vector<Solucion> si;
	std::vector<Solucion> sd;
	Solucion tmp;
	
	// Porcentaje aplicado sobre el mejor tiro para la inteligencia media
	// y baja.
	float porcentaje = 0;

	// Distancia a partir de la cual la inteligencia media y baja pueden
	// hacer hoyo directo.
	float dis_hoyo = 0;
	

	// Establecemos el porcentaje y la distancia al miníma de tiro directo
	// al hoyo en función de la inteligencia.
	if(tipo_ == IA_MEDIA)
	{
		porcentaje = 0.20;
		dis_hoyo = 120 / Constantes::_FACTOR_DE_CONVERSION_;
	}
	else if(tipo_ == IA_BAJA)
	{
		porcentaje = 0.50;
		dis_hoyo = 45 / Constantes::_FACTOR_DE_CONVERSION_;
	}
	
	Uint32 ta, tb;
	// Comprobamos todos los palos.

	ta = SDL_GetTicks();	
	for(int i = 0; i < numero_palos; i++)
	{
		// Probamos con todos los niveles de fuerza.
		for(float j = 1; j < 11; j++)
		{
			// Determinamos las coordenadas finales del tiro.
			// Coordenada del eje de abcisas.
			if(bola_ -> PosicionBola().x + bola_ -> Ancho()/2  < 
			cursor_ -> Posicion().x + cursor_ -> Posicion().w/2)
				x = bola_ -> PosicionSombra().x + alcance_maximo
					* j / 10 * cos(angulo);
			else
				x = bola_ -> PosicionSombra().x - alcance_maximo
					* j / 10 * cos(angulo);
			
			// Coordenada del eje de ordenadas.
			if(cursor_ -> Posicion().y + cursor_ -> Posicion().h/2<
			bola_ -> PosicionBola().y + bola_ -> Alto()/2)
				y = bola_ -> PosicionSombra().y - alcance_maximo * j / 10
					* sin(angulo);
			else
				y = bola_ -> PosicionSombra().y + alcance_maximo * j / 10
					* sin(angulo);

			tb = SDL_GetTicks();
			if(tb - ta > Constantes::_MS_)
			{
				partida_ -> CampoActivo().Dibujar();
				partida_ -> DibujarBolasInactivas();
				bola_ -> DibujarSombra();
				bola_ -> DibujarBola();
				this -> Dibujar();
				this -> CursorActual().Dibujar();
				partida_ -> CampoActivo().DibujarBandera();
				partida_ ->DibujarInterfaz();
				pantalla_ -> Actualizar();
				
				ta = SDL_GetTicks();
			}

			// Simulamos el lanzamiento para obtener la distancia a la que
			// que estará la bola del hoyo tras efectuar el golpe.
			d = SimularLanzamiento(x, y, PaloActual());

			// Si no se ha producido colisión en la simulación, lo procesamos.
			if(d != -1)
			{
				// Si la distancia obtenida en la simulación es menor que la mínima
				// actual, actualizamos la distancia mínima.
				if(d <= distancia)
				{
					// Si el golpe no es hoyo y el palo es distinto del putter,
					// establecemos los nuevos valores.
					// Esta comprobación la hacemos para que la inteligencia tire con
					// el mínimo de fuerza que haga hoyo con el putter.
					if(distancia != 0 || palo != "Putter")
					{
						destino_optimo.x = x;
						destino_optimo.y = y;
						palo = PaloActual().Nombre();
						fuerza = j;
						distancia = d;
						
						// Si la inteligencia es media o baja, añadimos la solución al
						// vector de soluciones.
						if((tipo_ == IA_MEDIA || tipo_ == IA_BAJA))
						{
							tmp.x = x;
							tmp.y = y;
							tmp.palo = PaloActual().Nombre();
							tmp.fuerza = j;
							tmp.distancia = d;
							tmp.angulo = cursor_ -> Angulo();
							s.push_back(tmp);
						}
					}
				}
			}				
		}

		// Si la bola no está en el bunker, cambiamos el palo.
		if(bola_ -> Estado() != Bola::BUNKER)
		{
			// Cambiamos al palo anterior.
			PaloAnterior();

			// Actualizamos el alcance máximo.
			alcance_maximo = PaloActual().AlcanceMaximo();
		}
	}
	
	// La inteligencia es media o baja.
	if((tipo_ == IA_MEDIA || tipo_ == IA_BAJA) && !s.empty())
	{
		// Obtenemos el mejor tiro.
		int minimo = s.back().distancia;

		// Si la distancia al hoyo es mayor que el mínimo establecido para que
		// pueda hacerse hoyo directo y el mejor tiro es hoyo directo, nos
		// quedamos con la alternativa mas próxima que no haga hoyo directo.
		if(DistanciaHoyo() > dis_hoyo && minimo == 0)
		{
			// Recorremos las soluciones y nos quedamos con la mejor alternativa.
			for(std::vector<Solucion>::iterator i = s.begin();
				(*i).distancia > 0 && i != s.end(); ++i)
			{
				destino_optimo.x = (*i).x;
				destino_optimo.y = (*i).y;
				palo = (*i).palo;
				fuerza = (*i).fuerza;
				distancia = (*i).distancia;
				angulo = (*i).angulo;
			}
		}
		
		// La distancia al hoyo es mayor que la distancia mínima para hacer hoyo
		// directo.
		else if(DistanciaHoyo() > dis_hoyo)
		{
			// Calculamos la distancia ideal en función de la inteligencia.
			int distancia_ideal = minimo + (s.front().distancia - minimo)
				* porcentaje;

			// Tomamos un valor suficientemente grande.
			int aproximacion = 10000;
			int solucion_mas_proxima = 0;

			// Buscamos la solución que más se aproxime a la ideal.
			for(std::vector<Solucion>::iterator i = s.begin(); i != s.end(); ++i)
			{
				if(abs((*i).distancia - distancia_ideal) < aproximacion)
				{
					aproximacion = abs((*i).distancia - distancia_ideal);
					solucion_mas_proxima = distance(s.begin(), i);
				}
			}

			// Seleccionamos la solución más próxima.
			std::vector<Solucion>::iterator i = s.begin();
			while(distance(s.begin(), i) != solucion_mas_proxima)
			{
				// Incrementamos.
				++i;

				// Si hemos llegado al final de las soluciones, salimos.
				if(distance(i, s.end()) == 1)
					break;
			}

			// Establecemos los nuevos datos del lanzamiento.
			destino_optimo.x = (*i).x;
			destino_optimo.y = (*i).y;
			palo = (*i).palo;
			fuerza = (*i).fuerza;
			distancia = (*i).distancia;
			angulo = (*i).angulo;
		}
	}

	
/*****************************************************************************/


	// Volvemos a seleccionar el palo que llega más lejos.
	while(PaloActual().Nombre() != "Madera 1"
		&& bola_ -> Estado() != Bola::BUNKER)
		PaloSiguiente();

	// Orientamos el cursor.
	OrientarCursor();
	
	// Variables para la simulación de lanzamiento a la izquierda.
	int fuerzai = 0;
	std::string paloi;
	float anguloi = 0;
	float distanciai = 10000;
	d = 10000;
	SDL_Rect destino_optimoi;
	destino_optimoi.x = 0;
	destino_optimoi.y = 0;
	int xi, yi;

	// Comprobamos si se ha cerrado la ventana o si tenemos que volver al
	// menú principal.
	retorno = partida_ -> ComprobarEventos();
	if(retorno == -1)
		return -1;
	else if(retorno == 1)
		return 1;

	ta = SDL_GetTicks();
		
	// Comprobamos todos los palos.
	for(int i = 0; i < numero_palos; i++)
	{
		// Probamos todos los niveles de fuerza.
		for(float j = 1; j < 11; j++)
		{
			// Todos los ángulos mayores que 10.
			while(cursor_ -> Angulo() * 180 / M_PI > 10)
			{
			

				// Giramos el cursor a la izquierda.
				CursorIzquierda();
				
				// Actualizamos el ángulo.
				angulo = cursor_ -> Angulo();

				// Determinamos las coordenadas finales del tiro.

				// Coordenada del eje de abcisas.
				if(bola_ -> PosicionBola().x + bola_ -> Ancho()/2  < 
				cursor_ -> Posicion().x + cursor_ -> Posicion().w/2)
					xi = bola_ -> PosicionSombra().x + alcance_maximo
						* j / 10 * cos(angulo);
				else
					xi = bola_ -> PosicionSombra().x - alcance_maximo
						* j / 10 * cos(angulo);

				// Coordenada del eje de ordenadas.		
				if(cursor_ -> Posicion().y + cursor_ -> Posicion().h/2<
				bola_ -> PosicionBola().y + bola_ -> Alto()/2)
					yi = bola_ -> PosicionSombra().y - alcance_maximo * j / 10
						* sin(angulo);
				else
					yi = bola_ -> PosicionSombra().y + alcance_maximo * j / 10
						* sin(angulo);

				tb = SDL_GetTicks();
				if(tb - ta > Constantes::_MS_)
				{
					partida_ -> CampoActivo().Dibujar();
					partida_ -> DibujarBolasInactivas();
					bola_ -> DibujarSombra();
					bola_ -> DibujarBola();
					this -> Dibujar();
					float angulo = cursor_ -> Angulo();
					cursor_ -> Orientar();
					this -> CursorActual().Dibujar();
					while(cursor_ -> Angulo() != angulo)
						CursorIzquierda();
					partida_ -> CampoActivo().DibujarBandera();
					partida_ ->DibujarInterfaz();
					pantalla_ -> Actualizar();
					
					ta = SDL_GetTicks();
				}

				// Simulamos el lanzamiento para obtener la distancia a la que
				// que estará la bola del hoyo tras efectuar el golpe.
				d = SimularLanzamiento(xi, yi, PaloActual());

				// Si no se ha producido colisión en la simulación, lo procesamos.
				if(d != -1)
				{
					// Si la distancia obtenida en la simulación es menor que la mínima
					// actual, actualizamos la distancia mínima.
					if(d <= distanciai)
					{
						// Si el golpe no es hoyo y el palo es distinto del putter,
						// establecemos los nuevos valores.
						// Esta comprobación la hacemos para que la inteligencia tire con
						// el mínimo de fuerza que haga hoyo con el putter.
						if(distanciai != 0 || paloi != "Putter")
						{
							destino_optimoi.x = xi;
							destino_optimoi.y = yi;
							paloi = PaloActual().Nombre();
							fuerzai = j;
							distanciai = d;
							anguloi = cursor_ -> Angulo();

							// Si la inteligencia es media o baja, añadimos la solución al
							// vector de soluciones.
							if((tipo_ == IA_MEDIA || tipo_ == IA_BAJA))
							{
								tmp.x = xi;
								tmp.y = yi;
								tmp.palo = PaloActual().Nombre();
								tmp.fuerza = j;
								tmp.distancia = d;
								tmp.angulo = cursor_ -> Angulo();
								si.push_back(tmp);
							}
						}
					}
				}			
			
			} // while
			
			// Orientamos el cursor.
			OrientarCursor();
			
			// Actualizamos el ángulo.
			angulo = cursor_ -> Angulo();		
		
		} // for fuerza
		
		// Si la bola no está en el bunker, cambiamos de palo.
		if(bola_ -> Estado() != Bola::BUNKER)
		{
			// Cambiamos al palo anterior.
			PaloAnterior();
			
			// Establecemos el alcance máximo.
			alcance_maximo = PaloActual().AlcanceMaximo();
		}
	} // for palos

	// La inteligencia es media o baja.
	if((tipo_ == IA_MEDIA || tipo_ == IA_BAJA) && !si.empty())
	{
		// Obtenemos el mejor tiro.
		int minimo = si.back().distancia;		

		// Si la distancia al hoyo es mayor que el mínimo establecido para que
		// pueda hacerse hoyo directo y el mejor tiro es hoyo directo, nos
		// quedamos con la alternativa mas próxima que no haga hoyo directo.
		if(DistanciaHoyo() > dis_hoyo && minimo == 0)
		{
			// Recorremos las soluciones y nos quedamos con la mejor alternativa.
			for(std::vector<Solucion>::iterator i = si.begin();
				(*i).distancia > 0 && i != si.end(); ++i)
			{
				destino_optimoi.x = (*i).x;
				destino_optimoi.y = (*i).y;
				paloi = (*i).palo;
				fuerzai = (*i).fuerza;
				distanciai = (*i).distancia;
				anguloi = (*i).angulo;
			}
		}

		// La distancia al hoyo es mayor que la distancia mínima para hacer hoyo
		// directo.
		else if(DistanciaHoyo() > dis_hoyo)
		{
			// Calculamos la distancia ideal en función de la inteligencia.
			int distancia_ideal = minimo + (si.front().distancia - minimo)
				* porcentaje;

			// Tomamos un valor suficientemente grande.
			int aproximacion = 10000;
			int solucion_mas_proxima = 0;

			// Buscamos la solución que más se aproxime a la ideal.
			for(std::vector<Solucion>::iterator i = si.begin(); i != si.end(); ++i)
			{
				if(abs((*i).distancia - distancia_ideal) < aproximacion)
				{
					aproximacion = abs((*i).distancia - distancia_ideal);
					solucion_mas_proxima = distance(si.begin(), i);
				}
			}
			
			// Seleccionamos la solución más próxima.			
			std::vector<Solucion>::iterator i = si.begin();
			while(distance(si.begin(), i) != solucion_mas_proxima)
			{
				// Incrementamos.
				++i;

				// Si hemos llegado al final de las soluciones, salimos.
				if(distance(i, si.end()) == 1)
					break;
			}

			// Establecemos los nuevos datos del lanzamiento.
			destino_optimoi.x = (*i).x;
			destino_optimoi.y = (*i).y;
			paloi = (*i).palo;
			fuerzai = (*i).fuerza;
			distanciai = (*i).distancia;
			anguloi = (*i).angulo;
		}
	}


/******************************************************************************/

	// Volvemos a seleccionar el palo que llega más lejos.
	while(PaloActual().Nombre() != "Madera 1"
		&& bola_ -> Estado() != Bola::BUNKER)
		PaloSiguiente();

	// Orientamos el cursor.
	OrientarCursor();

	// Variables para la simulación de lanzamiento a la izquierda.
	int fuerzad = 0;
	std::string palod;
	float angulod = 0;
	float distanciad = 10000;
	d = 10000;
	SDL_Rect destino_optimod;
	destino_optimod.x = 0;
	destino_optimod.y = 0;
	int xd, yd;


	// Comprobamos si se ha cerrado la ventana o si tenemos que volver al
	// menú principal.
	retorno = partida_ -> ComprobarEventos();
	if(retorno == -1)
		return -1;
	else if(retorno == 1)
		return 1;

	ta = SDL_GetTicks();

	// Comprobamos todos los palos.
	for(int i = 0; i < numero_palos; i++)
	{
		// Probamos todos los niveles de fuerza.
		for(float j = 1; j < 11; j++)
		{
			// Todos los ángulos mayores que 10.
			while(cursor_ -> Angulo() * 180 / M_PI > 10)
			{
				// Giramos el cursor a la izquierda.
				CursorDerecha();
				
				// Actualizamos el ángulo.
				angulo = cursor_ -> Angulo();

				// Determinamos las coordenadas finales del tiro.

				// Coordenada del eje de abcisas.
				if(bola_ -> PosicionBola().x + bola_ -> Ancho()/2  < 
				cursor_ -> Posicion().x + cursor_ -> Posicion().w/2)
					xd = bola_ -> PosicionSombra().x + alcance_maximo
						* j / 10 * cos(angulo);
				else
					xd = bola_ -> PosicionSombra().x - alcance_maximo
						* j / 10 * cos(angulo);

				// Coordenada del eje de ordenadas.	
				if(cursor_ -> Posicion().y + cursor_ -> Posicion().h/2<
				bola_ -> PosicionBola().y + bola_ -> Alto()/2)
					yd = bola_ -> PosicionSombra().y - alcance_maximo * j / 10
						* sin(angulo);
				else
					yd = bola_ -> PosicionSombra().y + alcance_maximo * j / 10
						* sin(angulo);

				tb = SDL_GetTicks();
				if(tb - ta > Constantes::_MS_)
				{
					partida_ -> CampoActivo().Dibujar();
					partida_ -> DibujarBolasInactivas();
					bola_ -> DibujarSombra();
					bola_ -> DibujarBola();
					this -> Dibujar();
					float angulo = cursor_ -> Angulo();
					cursor_ -> Orientar();
					this -> CursorActual().Dibujar();
					while(cursor_ -> Angulo() != angulo)
						CursorDerecha();
					partida_ -> CampoActivo().DibujarBandera();
					partida_ ->DibujarInterfaz();
					pantalla_ -> Actualizar();
					
					ta = SDL_GetTicks();
				}

				// Simulamos el lanzamiento para obtener la distancia a la que
				// que estará la bola del hoyo tras efectuar el golpe.
				d = SimularLanzamiento(xd, yd, PaloActual());

				// Si no se ha producido colisión en la simulación, lo procesamos.
				if(d != -1)
				{
					// Si la distancia obtenida en la simulación es menor que la mínima
					// actual, actualizamos la distancia mínima.
					if(d <= distanciad)
					{
						// Si el golpe no es hoyo y el palo es distinto del putter,
						// establecemos los nuevos valores.
						// Esta comprobación la hacemos para que la inteligencia tire con
						// el mínimo de fuerza que haga hoyo con el putter.
						if(distanciad != 0 || palod != "Putter")
						{
							destino_optimod.x = xd;
							destino_optimod.y = yd;
							palod = PaloActual().Nombre();
							fuerzad = j;
							distanciad = d;
							angulod = cursor_ -> Angulo();

							// Si la inteligencia es media o baja, añadimos la solución al
							// vector de soluciones.
							if((tipo_ == IA_MEDIA || tipo_ == IA_BAJA))
							{
								tmp.x = xd;
								tmp.y = yd;
								tmp.palo = PaloActual().Nombre();
								tmp.fuerza = j;
								tmp.distancia = d;
								tmp.angulo = cursor_ -> Angulo();
								sd.push_back(tmp);
							}

						}
					}
				}				
			} // while
			
			// Orientamos el cursor.
			OrientarCursor();
			
			// Actualizamos el ángulo.
			angulo = cursor_ -> Angulo();		
		
		} // for fuerza
		
		// Si la bola no está en el bunker, cambiamos de palo.
		if(bola_ -> Estado() != Bola::BUNKER)
		{
			// Seleccionamos el palo anterior.
			PaloAnterior();
			
			// Establecemos el alcance máximo.
			alcance_maximo = PaloActual().AlcanceMaximo();
		}
	} // for palos

	// La inteligencia es media o baja.
	if((tipo_ == IA_MEDIA || tipo_ == IA_BAJA) && !sd.empty())
	{
		// Obtenemos el mejor tiro.
		int minimo = sd.back().distancia;

		// Si la distancia al hoyo es mayor que el mínimo establecido para que
		// pueda hacerse hoyo directo y el mejor tiro es hoyo directo, nos
		// quedamos con la alternativa mas próxima que no haga hoyo directo.
		if(DistanciaHoyo() > dis_hoyo && minimo == 0)
		{
			// Recorremos las soluciones y nos quedamos con la mejor alternativa.
			for(std::vector<Solucion>::iterator i = sd.begin();
				(*i).distancia > 0 && i != sd.end(); ++i)
			{
				destino_optimod.x = (*i).x;
				destino_optimod.y = (*i).y;
				palod = (*i).palo;
				fuerzad = (*i).fuerza;
				distanciad = (*i).distancia;
				angulod = (*i).angulo;
			}
		}
		
		// La distancia al hoyo es mayor que la distancia mínima para hacer hoyo
		// directo.
		else if(DistanciaHoyo() > dis_hoyo)
		{
			// Calculamos la distancia ideal en función de la inteligencia.
			int distancia_ideal = minimo + (sd.front().distancia - minimo)
				* porcentaje;

			// Tomamos un valor suficientemente grande.
			int aproximacion = 10000;
			int solucion_mas_proxima = 0;

			// Buscamos la solución que más se aproxime a la ideal.
			for(std::vector<Solucion>::iterator i = sd.begin(); i != sd.end(); ++i)
			{
				if(abs((*i).distancia - distancia_ideal) < aproximacion)
				{
					aproximacion = abs((*i).distancia - distancia_ideal);
					solucion_mas_proxima = distance(sd.begin(), i);
				}
			}

			// Seleccionamos la solución más próxima.		
			std::vector<Solucion>::iterator i = sd.begin();
			while(distance(sd.begin(), i) != solucion_mas_proxima)
			{
				// Incrementamos.
				++i;

				// Si hemos llegado al final de las soluciones, salimos.
				if(distance(i, sd.end()) == 1)
					break;
			}

			// Establecemos los nuevos datos del lanzamiento.
			destino_optimod.x = (*i).x;
			destino_optimod.y = (*i).y;
			palod = (*i).palo;
			fuerzad = (*i).fuerza;
			distanciad = (*i).distancia;
			angulod = (*i).angulo;
		}
	}

/******************************************************************************/



	// Si la inteligencia es media o baja y la distancia al hoyo es mayor
	// que la distancia mínima para hacer hoyo directo y en las soluciones
	// hay un tiro que produce hoyo directo, comprobamos si hay alguna
	// alternativa. Si existe alternativa la tomamos como solución óptima.
	if((tipo_ == IA_MEDIA || tipo_ == IA_BAJA) && (DistanciaHoyo() > dis_hoyo)
		&& (distancia == 0 || distanciai == 0 || distanciad == 0))
	{
		// Se produce hoyo directo en el tiro recto.
		if(distancia == 0)
		{
			// Recorremos la lista de soluciones de forma inversa en busca de
			// la mejor alternativa.
			for(std::vector<Solucion>::reverse_iterator i = s.rbegin();
			i != s.rend(); ++i)
			{
				// Si la solución no produce hoyo directo, la tomamos como solución
				// definitiva y salimos del bucle.
				if((*i).distancia != 0)
				{
					// Establecemos los nuevos valores para la solución.
					destino_optimo.x = (*i).x;
					destino_optimo.y = (*i).y;
					palo = (*i).palo;
					fuerza = (*i).fuerza;
					distancia = (*i).distancia;
					angulo = (*i).angulo;
					
					break;		
				}
			}
		}

		// Se produce hoyo directo en el tiro a la izquierda.
		if(distanciai == 0)
		{
			// Recorremos la lista de soluciones de forma inversa en busca de
			// la mejor alternativa.
			for(std::vector<Solucion>::reverse_iterator i = si.rbegin();
			i != si.rend(); ++i)
			{
				// Si la solución no produce hoyo directo, la tomamos como solución
				// definitiva y salimos del bucle.
				if((*i).distancia != 0)
				{
					// Establecemos los nuevos valores para la solución.
					destino_optimoi.x = (*i).x;
					destino_optimoi.y = (*i).y;
					paloi = (*i).palo;
					fuerzai = (*i).fuerza;
					distanciai = (*i).distancia;
					anguloi = (*i).angulo;
					
					break;		
				}
			}
		}

		// Se produce hoyo directo en el tiro a la derecha.
		if(distanciad == 0)
		{
			// Recorremos la lista de soluciones de forma inversa en busca de
			// la mejor alternativa.
			for(std::vector<Solucion>::reverse_iterator i = sd.rbegin();
			i != sd.rend(); ++i)
			{
				// Si la solución no produce hoyo directo, la tomamos como solución
				// definitiva y salimos del bucle.
				if((*i).distancia != 0)
				{
					// Establecemos los nuevos valores para la solución.
					destino_optimod.x = (*i).x;
					destino_optimod.y = (*i).y;
					palod = (*i).palo;
					fuerzad = (*i).fuerza;
					distanciad = (*i).distancia;
					angulod = (*i).angulo;
					
					break;		
				}
			}
		}

	// Si con el tiro directo se produce hoyo directo y hay una alternativa,
	// anulamos la solución del tiro directo.
	if(distancia == 0 && ((distanciai != 0 && distanciai != 10000)
	|| (distanciad != 0 && distanciad != 10000)))
		distancia = 10000;

	// Si con el tiro a la izquierda se produce hoyo directo y hay una
	// alternativa, anulamos la solución del tiro a la izquierda.
	if(distanciai == 0 && ((distancia != 0 && distancia != 10000)
	|| (distanciad != 0 && distanciad != 10000)))
		distanciai = 10000;

	// Si con el tiro a la derecha se produce hoyo directo y hay una
	// alternativa, anulamos la solución del tiro a la derecha.
	if(distanciad == 0 && ((distancia != 0 && distancia != 10000)
	|| (distanciai != 0 && distanciai != 10000)))
		distanciad = 10000;
	}

	// Restauramos el palo original.
	while(PaloActual().Nombre() != palo_original
		&& bola_ -> Estado() != Bola::BUNKER)
		PaloAnterior();

	// Restauramos la posición del cursor.
	OrientarCursor();

	// En caso de que no se pueda evitar la colisión lanzamos con el Madera 1
	// (o Sand Wedge en caso de estar en el bunker) lo más lejos posible.
	if(fuerza == 0 && fuerzai == 0 && fuerzad == 0)
	{
		fuerza = 10;
		distancia = 1;

		// En caso de no estar en el bunker, seleccionamos Madera 1.
		while(PaloActual().Nombre() != "Madera 1"
		&& bola_ -> Estado() != Bola::BUNKER)
			PaloSiguiente();

		// Actualizamos el nombre del palo.
		palo = PaloActual().Nombre();
		
		// Establecemos el alcance máximo.
		alcance_maximo = PaloActual().AlcanceMaximo();
		
		// Modificamos el ángulo.
		angulo = cursor_ -> Angulo();

		// Coordenada del eje de abcisas.	
		if(bola_ -> PosicionBola().x + bola_ -> Ancho() / 2  < 
		cursor_ -> Posicion().x + cursor_ -> Posicion().w / 2)
			x = bola_ -> PosicionSombra().x + alcance_maximo
				* fuerza / 10 * cos(angulo);
		else
			x = bola_ -> PosicionSombra().x - alcance_maximo
				* fuerza / 10 * cos(angulo);

		// Coordenada del eje de ordenadas.		
		if(cursor_ -> Posicion().y + cursor_ -> Posicion().h / 2 <
		bola_ -> PosicionBola().y + bola_ -> Alto() / 2)
			y = bola_ -> PosicionSombra().y - alcance_maximo * fuerza / 10
				* sin(angulo);
		else
			y = bola_ -> PosicionSombra().y + alcance_maximo * fuerza / 10
				* sin(angulo);

		// Actualizamos el destino óptimo.
		destino_optimo.x = x;
		destino_optimo.y = y;
	}

	// Tras el procesamiento de los tres bucles, hemos determinado 3 palos.
	// Cada uno de ellos nos permite realizar un tiro lo más cercano posible
	// al hoyo. Vamos a quedarnos con el palo que nos permite realizar el tiro
	// más cercano al hoyo, junto con una fuerza y el ángulo determinado.
	
	// Tomamos un mínimo.
	int distancia_minima = distancia;
	int fuerza_final = fuerza;

	// Si el tiro a la izquierda es mejor, nos quedamos con él.
	if(distanciai < distancia_minima)
	{
		distancia_minima = distanciai;
		fuerza_final = fuerzai;
	}

	// Si el tiro a la derecha es mejor, nos quedamos con él.	
	if(distanciad < distancia_minima)
	{
		distancia_minima = distanciad;
		fuerza_final = fuerzad;
	}

	// No aplicamos ningún efecto.
	int efecto_final = 0;
	int cuadro_efecto = 0;

	// Actualizamos el valor de los indicadores.
	indicador_ -> Fuerza(fuerza_final);
	indicador_ -> Efecto(efecto_final);

	// Comprobamos si es más rápido cambiar al palo anterior o al siguiente.
	int anterior = 0;
	int siguiente = 0;

	std::string p;
	
	if(distancia_minima == distancia)
		p = palo;
	else if(distancia_minima == distanciai)
		p = paloi;
	else if(distancia_minima == distanciad)
		p = palod;
	
	// Guardamos cuantos palos hemos de recorrer eligiendo el palo anterior.
	while(PaloActual().Nombre() != p && bola_ -> Estado() != Bola::BUNKER)
	{
		PaloAnterior();
		++anterior;
	}

	// Reestablecemos el palo original
	while(PaloActual().Nombre() != palo_original
		&& bola_ -> Estado() != Bola::BUNKER)
		PaloAnterior();

	// Guardamos cuantos palos hemos de recorrer eligiendo el palo siguiente.
	while(PaloActual().Nombre() != p && bola_ -> Estado() != Bola::BUNKER)
	{
		PaloSiguiente();
		++siguiente;
	}			

	// Reestablecemos el palo original
	while(PaloActual().Nombre() != palo_original
		&& bola_ -> Estado() != Bola::BUNKER)
		PaloAnterior();	

	// Coordenadas de destino finales.
	int xfinal, yfinal;
	
	// Ahora, vamos a seleccionar el palo adecuado.
	if(distancia_minima == distancia)
	{
		// Seleccionamos el palo. En este caso no es necesario buscar el ángulo
		// ya que el tiro es en línea recta.
		while(PaloActual().Nombre() != palo && bola_ -> Estado() != Bola::BUNKER)
		{
			// Elegimos el palo de la forma más rápida posible.
			if(anterior <= siguiente)
				PaloAnterior();
			else
				PaloSiguiente();
			
			// Dibujamos.
			partida_ -> CampoActivo().Dibujar();
			partida_ -> DibujarBolasInactivas();
			bola_ -> DibujarSombra();
			bola_ -> DibujarBola();
			this -> Dibujar();
			this -> CursorActual().Dibujar();
			partida_ -> CampoActivo().DibujarBandera();
			partida_ -> ActualizarPalo();
			partida_ -> DibujarInterfaz();
			pantalla_ -> Actualizar();
			
			// Esperamos.
			partida_ -> Esperar(150, true, true, true, false, false, 0, 0);
		}
		
		// Actualizamos las coordenadas de destino finales.
		xfinal = destino_optimo.x;
		yfinal = destino_optimo.y;
	}
	
	// El mejor tiro es lanzando a la izquierda.
	else if(distancia_minima == distanciai)
	{
		// Seleccionamos el ángulo óptimo.
		while(cursor_ -> Angulo() != anguloi)
		{
			// Giramos el cursor a la izquierda.
			CursorIzquierda();
			
			// Dibujamos.
			this -> ActualizarCuadro();
			partida_ -> CampoActivo().Dibujar();
			partida_ -> DibujarBolasInactivas();
			bola_ -> DibujarSombra();
			bola_ -> DibujarBola();
			this -> Dibujar();
			this -> CursorActual().Dibujar();
			partida_ -> CampoActivo().DibujarBandera();
			partida_ -> DibujarInterfaz();
			pantalla_ -> Actualizar();
			
			// Esperamos.
			partida_ -> Esperar(20, true, true, true, false, false, 0, 0);
		}
		
		// Esperamos.
		partida_ -> Esperar(1000, true, true, true, false, false, 0, 0);
		
		// Seleccionamos el palo adecuado.
		while(PaloActual().Nombre() != paloi && bola_ -> Estado() != Bola::BUNKER)
		{
			// Elegimos el palo de la forma más rápida posible.
			if(anterior <= siguiente)
				PaloAnterior();
			else
				PaloSiguiente();
			
			// Dibujamos.
			partida_ -> CampoActivo().Dibujar();
			partida_ -> DibujarBolasInactivas();
			bola_ -> DibujarSombra();
			bola_ -> DibujarBola();
			this -> Dibujar();
			this -> CursorActual().Dibujar();
			partida_ -> CampoActivo().DibujarBandera();
			partida_ -> DibujarInterfaz();
			pantalla_ -> Actualizar();
			
			// Esperamos.
			partida_ -> Esperar(150, true, true, true, false, false, 0, 0);
		}
		
		// Actualizamos las coordenadas de destino finales.
		xfinal = destino_optimoi.x;
		yfinal = destino_optimoi.y;	
	}
	
	// El mejor tiro es lanzando a la derecha.
	else
	{
		// De igual forma, establecemos el ángulo necesario.
		while(cursor_ -> Angulo() != angulod)
		{
			// Giramos el cursor a la derecha.
			CursorDerecha();
			
			// Dibujamos.
			this -> ActualizarCuadro();
			partida_ -> CampoActivo().Dibujar();
			partida_ -> DibujarBolasInactivas();
			bola_ -> DibujarSombra();
			bola_ -> DibujarBola();
			this -> Dibujar();
			this -> CursorActual().Dibujar();
			partida_ -> CampoActivo().DibujarBandera();
			partida_ -> DibujarInterfaz();
			pantalla_ -> Actualizar();
			
			// Esperamos.
			partida_ -> Esperar(20, true, true, true, false, false, 0, 0);
		}
		
		partida_ -> Esperar(1000, true, true, true, false, false, 0, 0);

		// Elegimos el palo establecido.
		while(PaloActual().Nombre() != palod && bola_ -> Estado() != Bola::BUNKER)
		{
			// Elegimos el palo de la forma más rápida posible.
			if(anterior <= siguiente)
				PaloAnterior();
			else
				PaloSiguiente();
			
			// Dibujamos.
			partida_ -> CampoActivo().Dibujar();
			partida_ -> DibujarBolasInactivas();
			bola_ -> DibujarSombra();
			bola_ -> DibujarBola();
			this -> Dibujar();
			this -> CursorActual().Dibujar();
			partida_ -> CampoActivo().DibujarBandera();
			partida_ -> ActualizarPalo();
			partida_ -> DibujarInterfaz();
			pantalla_ -> Actualizar();
			
			// Esperamos.
			partida_ -> Esperar(150, true, true, true, false, false, 0, 0);
		}
		
		// Actualizamos las coordenadas de destino finales.
		xfinal = destino_optimod.x;
		yfinal = destino_optimod.y;		
	}

	// Comprobamos si se ha cerrado la ventana o si tenemos que volver al
	// menú principal.	
	retorno = partida_ -> ComprobarEventos();
	if(retorno == -1)
		return -1;
	else if(retorno == 1)
		return 1;

	// Control de tiempo para la animación del indicador de fuerza.
	Uint32 ta_fuerza, tb_fuerza;
	
	// Control de tiempo para el bucle.
	Uint32 ta_while, tb_while;
	
	// Guardamos el tiempo.
	ta_fuerza = SDL_GetTicks();
	
	// Cuando reiniciamos la animación, el cuadro procesado va a ser el número
	// 1. Si la fuerza coincide con dicho número 1, entonces no entraríamos
	// en el bucle y no se dibujaría el indicador de fuerza. Con esta variable
	// nos aseguramos de que al menos se haya entrado una vez.
	bool dibujado = false;
	
	// Animamos hasta seleccionar la fuerza adecuada.
	while(indicador_ -> AnimacionFuerza().CuadroProcesado() != fuerza_final
		|| !dibujado)
	{
		// Guardamos el tiempo para el control del bucle.
		ta_while = SDL_GetTicks();
		
		// Guardamos el tiempo para el control de la animación.
		tb_fuerza = SDL_GetTicks();
		
		// Si el tiempo transcurrido es mayor que el retardo de la animación,
		// la dibujamos.
		if(tb_fuerza - ta_fuerza > indicador_ -> AnimacionFuerza().Retardo())
		{
			// Hemos dibujado al menos una vez.
			if(!dibujado)
				dibujado = true;

			// Dibujamos.
			partida_ -> CampoActivo().Dibujar();
			partida_ -> DibujarBolasInactivas();
			bola_ -> DibujarSombra();
			bola_ -> DibujarBola();
			this -> Dibujar();
			this -> CursorActual().Dibujar();
			indicador_ -> AnimacionFuerza().Dibujar();
			partida_ -> CampoActivo().DibujarBandera();
			partida_ -> DibujarInterfaz();
			pantalla_ -> Actualizar();
			
			// Tomamos las nuevas referencias de tiempo.
			ta_fuerza = SDL_GetTicks();
			tb_fuerza = SDL_GetTicks();
		}
		
		// Guardamos el tiempo para el bucle.
		tb_while = SDL_GetTicks();
		
		// Si el tiempo transcurrido es menor que el establecido, esperamos.
		if(tb_while - ta_while < Constantes::_MS_)
			SDL_Delay(Constantes::_MS_ - (tb_while - ta_while));
	}

	// Control de tiempo para la animación del indicador de efecto.
	Uint32 ta_efecto = 0;
	Uint32 tb_efecto = 0;
	
	// Hay que dibujar al menos una vez.
	dibujado = false;

	while(indicador_ -> AnimacionEfecto().CuadroProcesado() != cuadro_efecto
		|| !dibujado)
	{
		// Guardamos el tiempo para el control del bucle.
		ta_while = SDL_GetTicks();
		
		// Guardamos el tiempo para el control de la animación.
		tb_efecto = SDL_GetTicks();

		// Si el tiempo transcurrido es mayor que el retardo de la animación,
		// la dibujamos.		
		if(tb_efecto - ta_efecto > indicador_ -> AnimacionEfecto().Retardo())
		{
			// Hemos dibujado al menos una vez.
			if(!dibujado)
				dibujado = true;
			
			// Dibujamos.
			partida_ -> CampoActivo().Dibujar();
			partida_ -> DibujarBolasInactivas();
			bola_ -> DibujarSombra();
			bola_ -> DibujarBola();
			this -> Dibujar();
			this -> CursorActual().Dibujar();
			indicador_ -> AnimacionFuerza().DibujarCuadro();
			indicador_ -> AnimacionEfecto().Dibujar();
			partida_ -> CampoActivo().DibujarBandera();
			partida_ -> DibujarInterfaz();
			pantalla_ -> Actualizar();
			
			// Tomamos las nuevas referencias de tiempo.
			ta_efecto = SDL_GetTicks();
			tb_efecto = SDL_GetTicks();
		}
		
		// Guardamos el tiempo para el bucle.
		tb_while = SDL_GetTicks();
		
		// Si el tiempo transcurrido es menor que el establecido, esperamos.
		if(tb_while - ta_while < Constantes::_MS_)
			SDL_Delay(Constantes::_MS_ - (tb_while - ta_while));
	}
	
	// Esperamos.
	partida_ -> Esperar(2000, true, true, true, true, true, 0, 0);

	// Reiniciamos las animaciones del indicador.
	indicador_ -> AnimacionFuerza().Reiniciar();
	indicador_ -> AnimacionEfecto().Reiniciar();

	// Realizamos la animación previa al golpe.
	this -> AnimacionPrevia();

	// Incrementamos el par del jugador.
	this -> IncrementarPar();
	
	// Dibujamos en pantalla.
	partida_ -> ActualizarPar();
	partida_ -> CampoActivo().Dibujar();
	partida_ -> DibujarBolasInactivas();
	bola_ -> DibujarBola();
	this -> Dibujar();
	partida_ -> CampoActivo().DibujarBandera();
	partida_ -> DibujarInterfaz();
	pantalla_ -> Actualizar();	

	// Realizamos el lanzamiento.
	return bola_ -> Desplazar(xfinal, yfinal);
}

int Ordenador::SimularLanzamiento(int x, int y, const Palo& palo)
{
	// Rectángulo de posición de la bola.
	SDL_Rect rbola = bola_ -> PosicionBola();
	
	// Rectángulo de posición de la sombra.
	SDL_Rect rsombra = bola_ -> PosicionSombra();
	
	// Coordenadas iniciales de la bola.
	int x_bola = rbola.x;
	int y_bola = rbola.y;
	
	// Coordenadas iniciales de la sombra.
	int x_sombra = rsombra.x;
	int y_sombra = rsombra.y;
	
	// Altura de la bola sobre la sombra.
	int altura_minima = 1;

	// Triángulo de destino.
	float cateto_x = abs(x - rsombra.x);
	float cateto_y = abs(y - rsombra.y);
	float hipotenusa = sqrt(pow(cateto_x, 2) + pow(cateto_y, 2));

	// Determinamos el ángulo.
	float angulo = atan(cateto_y / cateto_x);
	
	// Puntos que tenemos que generar
	float puntos_a_generar = hipotenusa;
	float puntos_generados = 0;
	
	// Establecemos la velocidad de generación de puntos del desplazamiento.
	int velocidad = sqrt(hipotenusa) * 1/3;	

	float angulo_de_desvio = 0;

	// Conversión a radianes del ángulo de desvío
	float angulo_desvio;
	
	// Posición del cursor.
	SDL_Rect rcursor = partida_ -> JugadorActivo().CursorActual().Posicion();

	// Determinamos la coordenada de destino x.
	if(rbola.x + rbola.w/2 <= rcursor.x + rcursor.w/2
		&& rbola.y + rbola.h/2 >= rcursor.y + rcursor.h/2)
			angulo_desvio = angulo - (angulo_de_desvio * M_PI) / 180;
		
	else if(rbola.x + rbola.w/2 >= rcursor.x + rcursor.w/2
		&& rbola.y + rbola.h/2 >= rcursor.y + rcursor.h/2)
			angulo_desvio = angulo + (angulo_de_desvio * M_PI) / 180;

	// Determinamos la coordenada de destino y.
	else if(rbola.x + rbola.w/2 >= rcursor.x + rcursor.w/2
		&& rbola.y + rbola.h/2 <= rcursor.y + rcursor.h/2)
			angulo_desvio = angulo - (angulo_de_desvio * M_PI) / 180;

	else if(rbola.x + rbola.w/2 <= rcursor.x + rcursor.w/2
		&& rbola.y + rbola.h/2  <= rcursor.y + rcursor.h/2)
			angulo_desvio = angulo + (angulo_de_desvio * M_PI) / 180;

	// Seno y coseno de el triángulo de efecto.
	float coseno_desvio = cos(angulo_desvio);
	float seno_desvio = sin(angulo_desvio);
	
	// Vamos a generar el efecto de desvío mediante la imagen de la función
	// seno. La función seno genera una curva perfectra entre 0 y el número
	// pi, por lo que dividiremos dicho número entre la longitud de la
	// para saber el incremento que hemos de aplicar a la curva en cada
	// iteración del bucle que genera los puntos del desplazamiento de la bola.
	float incremento_curva = M_PI / hipotenusa;

	// Todavía no se ha generado ningún punto de desplazamiento, por lo que
	// el valor de la curva es 0.
	float curva = 0;

	// Aplicamos el valor de la gravedad. Este valor es indiferente, puesto
	// que el tiempo viene determinado por la longitud del recorrido.
	float gravedad = 1;
	
	// Establecemos el tiempo a 0 dado que todavía no ha comenzado el
	// desplazamiento.
	float t = 0;
	
	// El ángulo de salida viene determinado por el palo que tenga seleccionado
	// el jugador activo.
	float angulo_salida = (palo.AnguloSalida() * M_PI) / 180;

	// Velocidad de salida del tiro parabólico.
	float velocidad_salida = sqrt((hipotenusa * gravedad)
		/ sin(2 * angulo_salida));
	
	// Tiempo que es necesario para realizar el tiro parabólico.
	float tiempo = hipotenusa / (velocidad_salida * cos(angulo_salida));

	// Dividimos el tiempo que necesitamos entre la hipotenusa para saber
	// en cuanto tenemos que incrementar el tiempo en cada iteración.	
	float incremento_tiempo = (tiempo * velocidad) / hipotenusa;

	Uint32 ta, tb;
	
	ta = SDL_GetTicks();

	// Realizamos el bucle hasta que no se hayan generado todos los puntos
	// de desplazamiento determinados por la longitud de la hipotenusa.
	while(puntos_generados < puntos_a_generar)
	{
		// En función de las coordenadas iniciales, vamos incrementando o
		// decrementando la posición en el eje de las x.
		if(x_sombra < x)
			rsombra.x = x_sombra + puntos_generados * coseno_desvio;
		else
			rsombra.x = x_sombra - puntos_generados * coseno_desvio;

		// Lo mismo para el eje de las y.
		if(y_sombra < y)
		{
			if(angulo_desvio < angulo)
			{				
				if(x_sombra < x)	
					rsombra.y = y_sombra + puntos_generados * seno_desvio
					- (hipotenusa / 168) * angulo_de_desvio * sin(curva);
				else
					rsombra.y = y_sombra + puntos_generados * seno_desvio
					+ (hipotenusa / 168) * angulo_de_desvio * sin(curva);			
			}
			else
			{
				if(x_sombra < x)
				{
					if(angulo_desvio > M_PI / 2)
						rsombra.y = y_sombra + puntos_generados * seno_desvio
							+ (hipotenusa / 168) * angulo_de_desvio
							* sin(curva);
					else
						rsombra.y = y_sombra + puntos_generados * seno_desvio
							- (hipotenusa / 168) * angulo_de_desvio
							* sin(curva);
				}
				else
				{
					if(angulo_desvio > M_PI / 2)
						rsombra.y = y_sombra + puntos_generados * seno_desvio
							- (hipotenusa / 168) * angulo_de_desvio
							* sin(curva);
					else
						rsombra.y = y_sombra + puntos_generados * seno_desvio
							+ (hipotenusa / 168) * angulo_de_desvio
							* sin(curva);				
				}				
			}				
		}
		else
		{		
			if(angulo_desvio < angulo)
			{
				if(x_sombra < x)
					rsombra.y = y_sombra - puntos_generados * seno_desvio
						- (hipotenusa / 168) * angulo_de_desvio
						* sin(curva);
				else
					rsombra.y = y_sombra - puntos_generados * seno_desvio
						+ (hipotenusa / 168) * angulo_de_desvio
						* sin(curva);
			}
			else
			{
				if(x_sombra < x)
				{
					if(angulo_desvio > M_PI / 2)
						rsombra.y = y_sombra - puntos_generados * seno_desvio
							+ (hipotenusa / 168) * angulo_de_desvio
							* sin(curva);
					else
						rsombra.y = y_sombra - puntos_generados * seno_desvio
							- (hipotenusa / 168) * angulo_de_desvio
							* sin(curva);
				}
				else
				{
					if(angulo_desvio > M_PI / 2)
						rsombra.y = y_sombra - puntos_generados * seno_desvio
						- (hipotenusa / 168) * angulo_de_desvio
						* sin(curva);
					else
						rsombra.y = y_sombra - puntos_generados * seno_desvio
						+ (hipotenusa / 168) * angulo_de_desvio
						* sin(curva);				
				}				
			}				
		}

		// Incrementamos el efecto de desvío.
		curva += velocidad * incremento_curva;

		// Establecemos la posición de la bola que viene descrita por el tiro
		// parabólico.
		rbola.x = rsombra.x;
		rbola.y = rsombra.y - t * velocidad_salida * sin(angulo_salida)
			+ pow(t, 2) * gravedad * 0.5;	

		// Incrementamos el tiempo.
		t += incremento_tiempo;

		// Incrementamos los puntos generados en función de la velocidad.
		puntos_generados += velocidad;

		// Comprobamos que las coordenadas estén dentro de los límites del campo.
		// Si no es así, indicamos que el tiro ha fallado.
		if(partida_ -> FueraDeLimites(&rbola, &rsombra))
			return -1;

		// Si la altura de la bola es menor que la de los árboles, comprobamos
		// si hay colisión. En caso de producirse indicamos que el tiro ha fallado.
		if(rsombra.y - rbola.y <= Constantes::_ALTURA_ARBOLES_)
		{
			if(palo.Nombre() != "Putter")
			{
				if(partida_ -> ComprobarColision(Partida::CENTRO, Campo::ARBOLES,
					&rbola, &rsombra))
					return -1;
			}
			else
			{
				if(partida_ -> ComprobarColision(Partida::CENTRO, Campo::ARBOLES,
					&rbola, &rsombra))
					return -1;
	
				if(partida_ -> ComprobarColision(Partida::CENTRO, Campo::AGUA,
					&rbola, &rsombra))
					return -1;
	
				if(partida_ -> ComprobarColision(Partida::CENTRO, Campo::BUNKER,
					&rbola, &rsombra))
					return -1;

				if(partida_ -> ComprobarColision(Partida::COMPLETO, Campo::HOYO,
					&rbola, &rsombra))
					return 0;
					
			}
		}

		tb = SDL_GetTicks();
		if(tb - ta > Constantes::_MS_)
		{
			partida_ -> CampoActivo().Dibujar();
			partida_ -> DibujarBolasInactivas();
			bola_ -> DibujarSombra();
			bola_ -> DibujarBola();
			this -> Dibujar();
			this -> CursorActual().Dibujar();
			partida_ -> CampoActivo().DibujarBandera();
			partida_ ->DibujarInterfaz();
			pantalla_ -> Actualizar();
			
			ta = SDL_GetTicks();
		}
	} // while


	// Comprobamos que las coordenadas estén dentro de los límites del campo.
	// Si no es así, indicamos que el tiro ha fallado.
	if(partida_ -> FueraDeLimites(&rbola, &rsombra))
		return -1;

	// Comprobamos si se produce alguna colisión.
	// En caso de producirse indicamos que el tiro ha fallado.
	if(partida_ -> ComprobarColision(Partida::CENTRO, Campo::ARBOLES,
		&rbola, &rsombra))
		return -1;
	
	if(partida_ -> ComprobarColision(Partida::CENTRO, Campo::AGUA,
		&rbola, &rsombra))
		return -1;
	
	if(partida_ -> ComprobarColision(Partida::CENTRO, Campo::BUNKER,
		&rbola, &rsombra))
		return -1;
	
	if(partida_ -> ComprobarColision(Partida::COMPLETO, Campo::HOYO,
		&rbola, &rsombra))	
		return 0;	

	// Reducimos la velocidad a la mitad.	
	velocidad = velocidad * 50/100;

	// La velocidad es 0.
	if(velocidad <= 0)
	{
		// Actualizamos la posición de la sombra.
		rsombra.x = rbola.x;
		rsombra.y = rbola.y + altura_minima;

		// Actualizamos las coordenadas iniciales.
		x_bola = rbola.x;
		y_bola = rbola.y;
		x_sombra = rsombra.x;
		y_sombra = rsombra.y;
		
		// El tiro ha tenido éxito, devolvemos la distancia.
		return Utilidades::DistanciaEuclidea(rsombra, partida_
			-> CampoActivo().PosicionHoyo());
	}

	// La velocidad ha sido mayor a 0, por lo tanto existirá rebote.
	// Establecemos la distancia del rebote a 0.
	float distancia_rebote = 0;
	
	if(palo.Nombre() != "Putter")
	{
		// Establecemos la distancia del rebote.
		distancia_rebote = puntos_a_generar * 8/100;
		
		// Establecemos la nueva velocidad de salida.
		velocidad_salida = sqrt((distancia_rebote*gravedad)
			/ sin(2 * angulo_salida));
			
		// El tiempo viene determinado por la nueva distancia.
		tiempo = distancia_rebote / (velocidad_salida * cos(angulo_salida));
		
		// Calculamos el incremento del tiempo.
		incremento_tiempo = (tiempo * velocidad) / distancia_rebote;
		
		// Ponemos a 0 la variable de tiempo transcurrido.
		t = 0;

		ta = SDL_GetTicks();
	
		// Este bucle genera el rebote de la bola.
		for(float i=puntos_a_generar; i < puntos_a_generar + distancia_rebote;
			i+=velocidad)
		{
			// En función de las coordenadas iniciales, vamos incrementando o
			// decrementando la posición en el eje de las x.
			if(x_sombra < x)
				rsombra.x = x_sombra + i * coseno_desvio;
			else
				rsombra.x = x_sombra - i * coseno_desvio;		

			// Lo mismo para el eje de las y.
			if(y_sombra < y)
			{
				if(angulo_desvio < angulo)
				{				
					if(x_sombra < x)	
						rsombra.y = y_sombra + i * seno_desvio
						- (hipotenusa / 168) * angulo_de_desvio
						* sin(curva);
					else
						rsombra.y = y_sombra + i * seno_desvio
						+ (hipotenusa / 168) * angulo_de_desvio
						* sin(curva);			
				}
				else
				{
					if(x_sombra < x)
					{
						if(angulo_desvio > M_PI / 2)
							rsombra.y = y_sombra + i * seno_desvio
								+ (hipotenusa / 168) * angulo_de_desvio
								* sin(curva);
						else
							rsombra.y = y_sombra + i * seno_desvio
								- (hipotenusa / 168) * angulo_de_desvio
								* sin(curva);
					}
					else
					{
						if(angulo_desvio > M_PI / 2)
							rsombra.y = y_sombra + i * seno_desvio
								- (hipotenusa / 168) * angulo_de_desvio
								* sin(curva);
						else
							rsombra.y = y_sombra + i * seno_desvio
								+ (hipotenusa / 168) * angulo_de_desvio
								* sin(curva);				
					}				
				}				
			}
			else
			{		
				if(angulo_desvio < angulo)
				{
					if(x_sombra < x)
						rsombra.y = y_sombra - i * seno_desvio
							- (hipotenusa / 168) * angulo_de_desvio
							* sin(curva);
					else
						rsombra.y = y_sombra - i * seno_desvio
							+ (hipotenusa / 168) * angulo_de_desvio
							* sin(curva);
				}
				else
				{
					if(x_sombra < x)
					{
						if(angulo_desvio > M_PI / 2)
							rsombra.y = y_sombra - i * seno_desvio
								+ (hipotenusa / 168) * angulo_de_desvio
								* sin(curva);
						else
							rsombra.y = y_sombra - i * seno_desvio
								- (hipotenusa / 168) * angulo_de_desvio
								* sin(curva);
					}
					else
					{
						if(angulo_desvio > M_PI / 2)
							rsombra.y = y_sombra - i * seno_desvio
							- (hipotenusa / 168) * angulo_de_desvio
							* sin(curva);
						else
							rsombra.y = y_sombra - i * seno_desvio
							+ (hipotenusa / 168) * angulo_de_desvio
							* sin(curva);				
					}				
				}				
			}
			
			// Determinamos la función de la bola para el tiro parabólico.
			rbola.x = rsombra.x;
			rbola.y = rsombra.y - t * velocidad_salida * sin(angulo_salida)
				+ pow(t, 2) * gravedad * 0.5;	

			// Incrementamos la curva del efecto de desvío.
			curva += velocidad * incremento_curva;

			// Incrementamos la variable de tiempo transcurrido.
			t += incremento_tiempo;	


			// Comprobamos que las coordenadas estén dentro de los límites del campo.
			// Si no es así, indicamos que el tiro ha fallado.
			if(partida_ -> FueraDeLimites(&rbola, &rsombra))
				return -1;

			// Si la altura de la bola es menor que la de los árboles, comprobamos
			// si hay colisión. En caso de haber, indicamos que el tiro ha fallado.
			if(rsombra.y - rbola.y <= 16)
			{
				if(partida_ -> ComprobarColision(Partida::CENTRO, Campo::ARBOLES,
					&rbola, &rsombra))
					return -1;		
			}

			tb = SDL_GetTicks();
			if(tb - ta > Constantes::_MS_)
			{
				partida_ -> CampoActivo().Dibujar();
				partida_ -> DibujarBolasInactivas();
				bola_ -> DibujarSombra();
				bola_ -> DibujarBola();
				this -> Dibujar();
				this -> CursorActual().Dibujar();
				partida_ -> CampoActivo().DibujarBandera();
				partida_ ->DibujarInterfaz();
				pantalla_ -> Actualizar();
				
				ta = SDL_GetTicks();
			}

		}	// for
	}
	
	// Comprobamos que las coordenadas estén dentro de los límites del campo.
	// Si no es así, indicamos que el tiro ha fallado.
	if(partida_ -> FueraDeLimites(&rbola, &rsombra))
		return -1;

	// Comprobamos si se produce alguna colisión.
	// En caso de producirse indicamos que el tiro ha fallado.
	if(partida_ -> ComprobarColision(Partida::CENTRO, Campo::ARBOLES,
		&rbola, &rsombra))
		return -1;
	
	if(partida_ -> ComprobarColision(Partida::CENTRO, Campo::AGUA,
		&rbola, &rsombra))
		return -1;
	
	if(partida_ -> ComprobarColision(Partida::CENTRO, Campo::BUNKER,
		&rbola, &rsombra))
		return -1;

	if(partida_ -> ComprobarColision(Partida::COMPLETO, Campo::HOYO,
		&rbola, &rsombra))	
		return 0;

	// Hemos terminado el rebote. La bola vuelve a golpear contra la superficie
	// del campo, así que volvemos a reducir la velocidad.	
	velocidad = velocidad * 60/100;
	
	// Establecemos la distancia rodando a 0.
	float distancia_rodando = 0;

	// Si la velocidad es nula, guardamos las coordenadas y terminamos.	
	if(velocidad <= 0)
	{
		// Actualizamos las coordenadas de la sombra.
		rsombra.x = rbola.x;
		rsombra.y = rbola.y + altura_minima;

		// Actualizamos las coordenadas iniciales.
		x_bola = rbola.x;
		y_bola = rbola.y;
		x_sombra = rsombra.x;
		y_sombra = rsombra.y;

		// El tiro ha tenido éxito, devolvemos la distancia.
		return Utilidades::DistanciaEuclidea(rsombra, partida_
			-> CampoActivo().PosicionHoyo());		
	}
	
	if(palo.Nombre() != "Putter")
	{
		// Establecemos la distancia en la que la bola rodará.
		distancia_rodando = puntos_a_generar * 2/100;

		ta = SDL_GetTicks();

		for(float i=puntos_a_generar + distancia_rebote;
			i < puntos_a_generar + distancia_rebote + distancia_rodando;
			i+=velocidad)
		{

			// En función de las coordenadas iniciales, vamos incrementando o
			// decrementando la posición en el eje de las x.	
			if(x_sombra < x)
				rsombra.x = x_sombra + i * coseno_desvio;
			else
				rsombra.x = x_sombra - i * coseno_desvio;		

			// Lo mismo para el eje de las y.
			if(y_sombra < y)
			{
				if(angulo_desvio < angulo)
				{				
					if(x_sombra < x)	
						rsombra.y = y_sombra + i * seno_desvio
						- (hipotenusa / 168) * angulo_de_desvio
						* sin(curva);
					else
						rsombra.y = y_sombra + i * seno_desvio
						+ (hipotenusa / 168) * angulo_de_desvio
						* sin(curva);			
				}
				else
				{
					if(x_sombra < x)
					{
						if(angulo_desvio > M_PI / 2)
							rsombra.y = y_sombra + i * seno_desvio
								+ (hipotenusa / 168) * angulo_de_desvio
								* sin(curva);
						else
							rsombra.y = y_sombra + i * seno_desvio
								- (hipotenusa / 168) * angulo_de_desvio
								* sin(curva);
					}
					else
					{
						if(angulo_desvio > M_PI / 2)
							rsombra.y = y_sombra + i * seno_desvio
								- (hipotenusa / 168) * angulo_de_desvio
								* sin(curva);
						else
							rsombra.y = y_sombra + i * seno_desvio
								+ (hipotenusa / 168) * angulo_de_desvio
								* sin(curva);				
					}				
				}				
			}
			else
			{		
				if(angulo_desvio < angulo)
				{
					if(x_sombra < x)
						rsombra.y = y_sombra - i * seno_desvio
							- (hipotenusa / 168) * angulo_de_desvio
							* sin(curva);
					else
						rsombra.y = y_sombra - i * seno_desvio
							+ (hipotenusa / 168) * angulo_de_desvio
							* sin(curva);
				}
				else
				{
					if(x_sombra < x)
					{
						if(angulo_desvio > M_PI / 2)
							rsombra.y = y_sombra - i * seno_desvio
								+ (hipotenusa / 168) * angulo_de_desvio
								* sin(curva);
						else
							rsombra.y = y_sombra - i * seno_desvio
								- (hipotenusa / 168) * angulo_de_desvio
								* sin(curva);
					}
					else
					{
						if(angulo_desvio > M_PI / 2)
							rsombra.y = y_sombra - i * seno_desvio
							- (hipotenusa / 168) * angulo_de_desvio
							* sin(curva);
						else
							rsombra.y = y_sombra - i * seno_desvio
							+ (hipotenusa / 168) * angulo_de_desvio
							* sin(curva);				
					}				
				}				
			}

			// La posición de la bola será igual a la de la sombra, ya que está
			// rodando.
			rbola.x = rsombra.x;
			rbola.y = rsombra.y;

			// Incrementamos el efecto de desvío.						
			curva += velocidad * incremento_curva;		

			// Incrementamos el tiempo transcurrido.
			t += incremento_tiempo;	

			// Comprobamos que las coordenadas estén dentro de los límites del campo.
			// Si no es así, indicamos que el tiro ha fallado.
			if(partida_ -> FueraDeLimites(&rbola, &rsombra))
				return -1;

			// Comprobamos si se produce alguna colisión.
			// En caso de producirse indicamos que el tiro ha fallado.
			if(partida_ -> ComprobarColision(Partida::CENTRO, Campo::ARBOLES,
				&rbola, &rsombra))
				return -1;
	
			if(partida_ -> ComprobarColision(Partida::CENTRO, Campo::AGUA,
				&rbola, &rsombra))
				return -1;
	
			if(partida_ -> ComprobarColision(Partida::CENTRO, Campo::BUNKER,
				&rbola, &rsombra))
				return -1;

			if(partida_ -> ComprobarColision(Partida::COMPLETO, Campo::HOYO,
				&rbola, &rsombra))	
				return 0;
				
			tb = SDL_GetTicks();
			if(tb - ta > Constantes::_MS_)
			{
				partida_ -> CampoActivo().Dibujar();
				partida_ -> DibujarBolasInactivas();
				bola_ -> DibujarSombra();
				bola_ -> DibujarBola();
				this -> Dibujar();
				this -> CursorActual().Dibujar();
				partida_ -> CampoActivo().DibujarBandera();
				partida_ ->DibujarInterfaz();
				pantalla_ -> Actualizar();
				
				ta = SDL_GetTicks();
			}
		}	// for
	}

	// Actualizamos la posición de la sombra.
	rsombra.x = rbola.x;
	rsombra.y = rbola.y + altura_minima;

	// Actualizamos las coordenadas iniciales de la bola.
	x_bola = rbola.x;
	y_bola = rbola.y;
	
	// Actualizamos las coordenadas iniciales de la bola.
	x_sombra = rsombra.x;
	y_sombra = rsombra.y;

	// Devolvemos la distancia a la que estaría la bola del hoyo en caso de
	// realizar el tiro.
	return Utilidades::DistanciaEuclidea(rsombra, partida_
		-> CampoActivo().PosicionHoyo());
}
