/// \file ordenador.h Inteligencia artificial.
/*
	Copyright 2010 Andrés Francisco Aparicio

	This file is part of SDLgolf.

	SDLgolf is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	SDLgolf is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with SDLgolf.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef _ORDENADOR_H_
#define _ORDENADOR_H_

#include "jugador.h"

/*! \class Ordenador
    \brief Jugador controlado por el ordenador.

		Esta clase se encarga de la implementación de un jugador controlado
		por el ordenador, es decir, en ella se desarrolla la inteligencia
		artificial del juego.
		
		Todos los jugadores tienen un nombre, un color y
		pueden pertenecer a un equipo cuando se está jugando una partida por
		parejas. De la misma forma, los jugadores de un mismo equipo podrán
		compartir la bola de golf. A parte de esto, el jugador cuenta con una
		lista de palos de golf que podrá utilizar durante la partida.
 */
class Ordenador: public Jugador
{
	public:

		/*! Realiza las asociaciones con la Partida y la Pantalla.\n
				Establece el nombre y el color del jugador. Carga las imágenes
				y crea la lista de palos. Genera las animaciones del jugador.
				\brief Constructor.
				\param nombre Nombre del Jugador.
				\param color Color del Jugador.
				\param tipo Tipo de Jugador.
				\param bola Bola del Jugador.
				\param partida Partida que se está desarrollando.
				\param pantalla Pantalla donde se dibujará la Bola.
				\see Jugador::Tipos.
		 */
		Ordenador(std::string nombre, int color, int tipo,
			Bola& bola, Partida *partida, Pantalla *pantalla);

		/*! El ordenador realiza el lanzamiento de la Bola.\n
				Busca el mejor lanzamiento en función del nivel de inteligencia.
				\brief Golpea la Bola.
				\return \b 0 cuando el lanzamiento ha finalizado.
				\see SimularLanzamiento.
				\see Jugador::Tipos.
				\see Palo::AlcanceMaximo.
				\see Palo::AnguloSalida.
				\see Cursor::Angulo.
		 */	
		int GolpearBola();
		
		/*! \brief Simula el lanzamiento de la Bola.
				\return \arg \b -1 Si se ha producido colisión en el lanzamiento.
								\arg \b	Distancia a la que estaría la Bola del hoyo del
												Campo, en otro caso.
		 */	
		int SimularLanzamiento(int x, int y, const Palo& palo);

	private:

		/*!	Datos necesarios para guardar los parámetros que nos permitan
			 	realizar un tiro válido.
			 	\brief Tiro válido a realizar por el ordenador.
		 */
		struct Solucion
		{
			int x;	///< Coordenada x de destino.
			int y;	///< Coordenada y de destino.
			std::string palo;	///< Palo con el que se golpearía la Bola.
			int fuerza;	///< Fuerza que se aplicaría a la Bola.
			int distancia;	///< Distancia a la que estaría la Bola del hoyo.
			float angulo;	///< Ángulo entre el Cursor y la Bola.
		};
};

#endif
