/// \file pantalla.cpp Pantalla del juego.
/*
	Copyright 2010 Andrés Francisco Aparicio

	This file is part of SDLgolf.

	SDLgolf is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	SDLgolf is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with SDLgolf.  If not, see <http://www.gnu.org/licenses/>.
*/

#include <SDL/SDL.h>
#include <iostream>
#include "pantalla.h"
#include "camara.h"
#include "utilidades.h"
#include "constantes.h"

Pantalla::Pantalla()
{
	// Si la pantalla no ha sido creada, la creamos.
	if(SDL_GetVideoSurface() == 0)
		pantalla_ = SDL_SetVideoMode(800, 480, 0, SDL_HWSURFACE | SDL_DOUBLEBUF |
			SDL_HWACCEL | SDL_HWPALETTE);

	// Comprobamos que no existan problemas.
	if(pantalla_ == 0)
	{
		std::cerr << "pantalla.cpp: Error al crear la pantalla." << std::endl;
		exit(1);
	}
	
	// Establecemos el título de la ventana.
	SDL_WM_SetCaption("SDLgolf", 0);
	
	// Desactivamos el cursor.
	SDL_ShowCursor(SDL_DISABLE);
	
	// Guardamos el ancho y el alto de la pantalla.
	rpantalla_.w = pantalla_ -> w;
	rpantalla_.h = pantalla_ -> h;
	
	// Guardamos las coordenadas.
	rpantalla_.x = 0;
	rpantalla_.y = 0;
	
	// Creamos la cámara de juego.
	camara_ = new Camara(Ancho(), Alto());
}

Pantalla::~Pantalla()
{

	// Destruimos la superficie de la pantalla.
	SDL_FreeSurface(pantalla_);
	
	// Destruimos la cámara.
	delete camara_;
}


void Pantalla::Actualizar()
{
	// Actualizamos la pantalla.
	if (SDL_Flip(pantalla_) != 0)
	{
		std::cerr << "pantalla.cpp: Actualizar() Error al intercambiar los"
		<< " buffers." << std::endl;
		exit(1);
	}
    	
}

void Pantalla::PantallaCompleta()
{
//	if(SDL_ShowCursor(SDL_QUERY) == SDL_ENABLE)
//		SDL_ShowCursor(SDL_DISABLE);
//	else
//		SDL_ShowCursor(SDL_ENABLE);

//	Uint32 flags;
//	
//	flags = pantalla_ -> flags;

//	pantalla_ = SDL_SetVideoMode(0, 0, 0, pantalla_ -> flags ^ SDL_FULLSCREEN);
//	
//	if(pantalla_ == NULL)
//		pantalla_ = SDL_SetVideoMode(0, 0, 0, flags);
//		
//	if(pantalla_ == NULL)
//		exit(1);
//		
//	Actualizar();
	
	// Cambiamos el modo de pantalla completa.
	if(SDL_WM_ToggleFullScreen(pantalla_) != 1)
	{
		std::cerr << "pantalla.cpp: PantallaCompleta() Error al establecer el"
			<< " modo de pantalla completa." << std::endl;
		exit(1);
	}
}

int Pantalla::Alto() const
{
	// Devolvemos el alto de la pantalla.
	return pantalla_ -> h;
}

int Pantalla::Ancho() const
{
	// Devolvemos el ancho de la pantalla.
	return pantalla_ -> w;
}

void Pantalla::Dibujar(const SDL_Surface *origen, const SDL_Rect *rdestino)
{
	// Dibujamos.
	if(SDL_BlitSurface(const_cast<SDL_Surface*>(origen), NULL, pantalla_,
		const_cast<SDL_Rect*>(&(camara_ -> Posicion(*rdestino)))) == -1)
	{
		std::cerr << "pantalla.cpp: Dibujar(origen, rdestino)"
			<<  " error al realizar blit." << std::endl;
		exit(1);
	}
}

void Pantalla::CentrarCamara(const SDL_Rect& objetivo,
	const SDL_Rect& superficie)
{
	// Centramos la cámara en el objetivo.
	camara_ -> Centrar(objetivo, superficie);
}

void Pantalla::Dibujar(const SDL_Surface *origen, const SDL_Rect *rorigen,
	const SDL_Rect *rdestino)
{
	// Dibujamos.
	if(SDL_BlitSurface(const_cast<SDL_Surface*>(origen),
	const_cast<SDL_Rect*>(rorigen), pantalla_,
	const_cast<SDL_Rect*>(&(camara_ -> Posicion(*rdestino)))) == -1)
	{
		std::cerr << "pantalla.cpp: Dibujar(origen, rorigen, rdestino)"
			<< " Error al realizar blit." << std::endl;
		exit(1);
	}
}

void Pantalla::Dibujar(const SDL_Surface *origen)
{
	// Dibujamos.
	if(SDL_BlitSurface(const_cast<SDL_Surface*>(origen),
	const_cast<SDL_Rect*>(&(camara_ ->  Posicion())), pantalla_, 0) == -1)
	{
		std::cerr << "pantalla.cpp: Dibujar(origen) Error al realizar blit."
			<< std::endl;
		exit(1);
	}
}

void Pantalla::DibujarDirectamente(const SDL_Surface *origen,
	const SDL_Rect *destino)
{
	// Dibujamos.
	if(SDL_BlitSurface(const_cast<SDL_Surface*>(origen), NULL, pantalla_,
	const_cast<SDL_Rect*>(destino)) == -1)
	{
		std::cerr << "pantalla.cpp: Dibujar(origen, destino) Error"
			<< " al realizar blit." << std::endl;
		exit(1);
	}
}

void Pantalla::DibujarDirectamente(const SDL_Surface *origen,
	const SDL_Rect *rorigen, const SDL_Rect *rdestino)
{
	// Dibujamos.
	if(SDL_BlitSurface(const_cast<SDL_Surface*>(origen),
	const_cast<SDL_Rect*>(rorigen), pantalla_,
	const_cast<SDL_Rect*>(rdestino)) == -1)
	{
		std::cerr << "pantalla.cpp: Dibujar(origen, destino) Error"
			<< " al realizar blit." << std::endl;
		exit(1);
	}
}

void Pantalla::Copiar(SDL_Surface **destino)
{
	// Rectángulo de posición.
	SDL_Rect rdestino;

	// Establecemos las coordenadas.
	rdestino.x = 0;
	rdestino.y = 0;
	
	// Generamos la ruta a la imagen transparente.
	std::string tmp;
	tmp.append(Constantes::_IMG_MENU_);
	tmp += "alpha.png";
	
	// Cargamos la imagen transparente.
	*destino = Utilidades::CargarImagen(tmp);
	
	// Dibujamos la pantalla en la superficie de la imagen.
	if(SDL_BlitSurface(pantalla_, NULL, *destino, &rdestino) != 0)
	{
		std::cerr << "pantalla.cpp: Copiar() Error"
			<< " al copiar la superficie." << std::endl;
		exit(1);
		
	}	
}

