/// \file pantalla.h Pantalla del juego.
/*
	Copyright 2010 Andrés Francisco Aparicio

	This file is part of SDLgolf.

	SDLgolf is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	SDLgolf is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with SDLgolf.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef _PANTALLA_H_
#define _PANTALLA_H_

#include <SDL/SDL_image.h>

class Camara;

/*! \class Pantalla.
    \brief Pantalla del juego.

    En la pantalla del juego se representarán todos los elementos gráficos
    del mismo. Es uno de los elementos vitales del juego.
 */
class Pantalla
{
	public:

		/*! Crea la superficie de la pantalla del juego, oculta el cursor,
				muestra el nombre del juego en el borde de la ventana y crea
				la Camara.
				\brief Constructor.
		 */
		Pantalla();

		/*! Destruye la pantalla y la Camara.
				\brief Destructor.
		 */
		~Pantalla();

		/*! \brief Establece el modo de pantalla completa.
		 */
		void PantallaCompleta();
		
		/*! \brief Actualiza la pantalla.
		 */
		void Actualizar();

		/*! \brief Centra la Camara en un determinado objetivo.
				\see Camara::Centrar.
		 */		
		void CentrarCamara(const SDL_Rect& objetivo, const SDL_Rect& superficie);

		/*! Dibuja sobre la superficie de la pantalla tomando las coordenadas
				relativas de la Camara.
				\brief Dibuja en pantalla.
				\param origen Imagen que se va a dibujar.
		 */		
		void Dibujar(const SDL_Surface *origen);

		/*! Dibuja sobre la superficie de la pantalla tomando las coordenadas
				relativas de la Camara.
				\brief Dibuja en pantalla.
				\param origen Imagen que se va a dibujar.
				\param rdestino Coordenadas donde se va a dibujar la imagen.
				\see Camara::Posicion.
		 */		
		void Dibujar(const SDL_Surface *origen, const SDL_Rect *rdestino);
		
		/*! Dibuja sobre la superficie de la pantalla tomando las coordenadas
				relativas de la Camara.
				\brief Dibuja en pantalla.
				\param origen Imagen que se va a dibujar.
				\param rorigen Rectángulo de la imagen que se va a dibujar.
				\param rdestino Coordenadas donde se va a dibujar la imagen.
				\see Camara::Posicion.
		 */
		void Dibujar(const SDL_Surface *origen, const SDL_Rect *rorigen,
			const SDL_Rect *rdestino);

		/*! Dibuja sobre la superficie de la pantalla tomando las coordenadas
				directamente.
				\brief Dibuja directamente en pantalla.
				\param origen Imagen que se va a dibujar.
				\param destino Coordenadas donde se va a dibujar la imagen.
				\see Camara::Posicion.
		 */			
		void DibujarDirectamente(const SDL_Surface *origen,
			const SDL_Rect *destino);
			
		/*! Dibuja sobre la superficie de la pantalla tomando las coordenadas
				directamente.
				\brief Dibuja directamente en pantalla.
				\param origen Imagen que se va a dibujar.
				\param rorigen Rectángulo de la imagen que se va a dibujar.
				\param rdestino Coordenadas donde se va a dibujar la imagen.
		 */
		void DibujarDirectamente(const SDL_Surface *origen,
			const SDL_Rect *rorigen, const SDL_Rect *rdestino);
			
		/*! \brief Realiza una copia de la superficie de la pantalla.
				\param destino Superficie donde se va a copiar la pantalla.
		 */
		void Copiar(SDL_Surface **destino);

		/*! \brief Ancho de la pantalla.
				\return Ancho de la pantalla.
		 */
		int Ancho() const;

		/*! \brief Alto de la pantalla.
				\return Alto de la pantalla.
		 */		
		int Alto() const;
		
	private:
	
		SDL_Surface *pantalla_;	///< Superficie de la pantalla.
		
		SDL_Rect rpantalla_;	///< Posición de la pantalla.
		
		Camara *camara_;	///< Cámara de la pantalla.
};

#endif
