/// \file partida.cpp Partida.
/*
	Copyright 2010 Andrés Francisco Aparicio

	This file is part of SDLgolf.

	SDLgolf is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	SDLgolf is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with SDLgolf.  If not, see <http://www.gnu.org/licenses/>.
*/

#include <iostream>
#include <string>
#include <sstream>
#include <cmath>
#include <cstdlib>
#include "partida.h"
#include "bola.h"
#include "campo.h"
#include "pantalla.h"
#include "jugador.h"
#include "camara.h"
#include "utilidades.h"
#include "constantes.h"
#include "menu.h"
#include "interfaz.h"


Partida::~Partida()
{

	// Destruimos las bolas de la partida.
	for(std::vector<Bola*>::iterator ibola = bolas_.begin();
		ibola != bolas_.end(); ibola++)
		delete *ibola;	
	
	// Destruimos los jugadores de la partida.
	for(std::vector<Jugador*>::iterator ijugador = jugadores_.begin();
		ijugador != jugadores_.end(); ijugador++)
		delete *ijugador;
	
	// Destruimos el campo.
	delete campo_activo_;
	
	// Destruimos la animación del agua.
	delete rejilla_agua_;
	delete animacion_agua_;

	// Destruimos la animación del bunker.
	delete rejilla_bunker_;
	delete animacion_bunker_;

	// Destruimos la animación de las hojas.
	delete rejilla_hojas_;
	delete animacion_hojas_;

	// Destruimos la interfaz.
	delete interfaz_;
}

void Partida::DibujarNombreCampo()
{
	// Imagen para el nombre del campo.
	SDL_Surface *nombre_campo;
	
	// Posición del nombre del campo.
	SDL_Rect rnombre_campo;
	
	// Imagen para el número de hoyo.
	SDL_Surface *numero_hoyo;
	
	// Posición para el número de hoyo.
	SDL_Rect rnumero_hoyo;
	
	// Cadena del nombre del campo.
	std::string nombre;
	
	// Cadena del número de hoyo.
	std::string hoyo;
	
	// Guardamos el nombre.
	nombre = campo_activo_ -> Nombre();
	
	// Generamos la cadena del número de hoyo.
	hoyo = "Hoyo " + campo_activo_ -> NumeroHoyo();
	
	// Creamos el texto.
	nombre_campo = Utilidades::CrearTexto(nombre.c_str(), Constantes::_FUENTE_,
		70, Utilidades::Color(Utilidades::BLANCO));
		
	// Guardamos el ancho y el alto.
	rnombre_campo.w = nombre_campo -> w;
	rnombre_campo.h = nombre_campo -> h;
	
	//Guardamos las coordenadas.
	rnombre_campo.x = pantalla_ -> Ancho() / 2 - rnombre_campo.w / 2;

	// Creamos el texto.	
	numero_hoyo = Utilidades::CrearTexto(hoyo.c_str(), Constantes::_FUENTE_, 40,
		Utilidades::Color(Utilidades::BLANCO));
	
	// Guardamos el ancho y el alto.
	rnumero_hoyo.w = numero_hoyo -> w;
	rnumero_hoyo.h = numero_hoyo -> h;
	
	//Guardamos las coordenadas.
	rnumero_hoyo.x = pantalla_ -> Ancho() / 2 - rnumero_hoyo.w / 2;

	rnombre_campo.y = pantalla_ -> Alto() / 2
		- (rnombre_campo.h + rnumero_hoyo.h) / 2;
		
	rnumero_hoyo.y = rnombre_campo.y + rnombre_campo.h;

	// Dibujamos.
	pantalla_ -> DibujarDirectamente(nombre_campo, &rnombre_campo);
	pantalla_ -> DibujarDirectamente(numero_hoyo, &rnumero_hoyo);

	// Destruimos las imágenes.	
	SDL_FreeSurface(nombre_campo);
	SDL_FreeSurface(numero_hoyo);
		
}

Campo& Partida::CampoActivo()
{
	// Devolvemos el campo activo.
	return *campo_activo_;
}

Jugador& Partida::JugadorActivo()
{
	// Devolvemos el jugador activo.
	return *jugador_activo_;
}

void Partida::ActivarJugador(Jugador *jugador)
{
	// Cambiamos el estado.
	jugador -> EstablecerEstado(Jugador::ACTIVO);
	
	// Actualizamos el puntero.
	jugador_activo_ = jugador;

	// Activamos la bola.
	if(jugador -> BolaEnJuego().Estado() == Bola::INACTIVA)
		jugador -> BolaEnJuego().Estado(Bola::ACTIVA);

	else if(jugador -> BolaEnJuego().Estado() == Bola::BUNKER_INACTIVA)
	{
		jugador -> BolaEnJuego().Estado(Bola::BUNKER);
		jugador -> SeleccionarPalo("Sand Wedge");
	}

	// Actualizamos el puntero.
	bola_activa_ = &(jugador -> BolaEnJuego());
	
	// Actualizamos el cuadro y el indicador.
	jugador -> ActualizarCuadro();
	jugador -> IndicadorActual().ActualizarPosicion();
	
	if(jugador -> DistanciaHoyo() < 30)
		campo_activo_ -> DesactivarBandera();
}


void Partida::OrientarCursor()
{
	// Orientamos el cursor al hoyo.
	jugador_activo_ -> OrientarCursor();
}


void Partida::CentrarCamara()
{
	// Centramos la cámara en la bola activa.
	pantalla_ -> CentrarCamara(bola_activa_ -> PosicionSombra(),
		campo_activo_ -> Posicion());
}

void Partida::DibujarBolasInactivas()
{
	// Hay bolas inactivas.
	if(!bolas_inactivas_.empty())
	{
		// Recorremos la lista de bolas inactivas y las dibujamos.
		for(std::vector<Bola*>::iterator i = bolas_inactivas_.begin();
			i != bolas_inactivas_.end(); i++)
			(*i) -> DibujarBola();
	}
}

void Partida::ActualizarBolasInactivas()
{
	// Vaciamos la lista de bolas inactivas.
	if(!bolas_inactivas_.empty())
		bolas_inactivas_.clear();
	
	// Recorremos los jugadores.
	for(std::vector<Jugador*>::iterator i = jugadores_.begin();
		i != jugadores_.end(); i++)
		// El jugador está inactivo.
		if((*i) -> EstadoActual() == Jugador::INACTIVO
			&& bola_activa_ != &(*i) -> BolaEnJuego())
			{
				// La bola está en el bunker.
				if((*i) -> BolaEnJuego().Estado() == Bola::BUNKER)
					(*i) -> BolaEnJuego().Estado(Bola::BUNKER_INACTIVA);

				// La bola está activa.
				if((*i) -> BolaEnJuego().Estado() == Bola::ACTIVA)				
					(*i) -> BolaEnJuego().Estado(Bola::INACTIVA);

				// En caso de no estar finalizada, la insertamos.
				if((*i) -> BolaEnJuego().Estado() != Bola::FINALIZADA)
					bolas_inactivas_.push_back(&(*i) -> BolaEnJuego());
			}
}
bool Partida::SiguienteJugador()
{
	std::vector<Jugador*>::iterator i;
	i = jugadores_.begin();
	int x = 0;
	
	// Comprobamos si hay jugadores inactivos.
	while(i != jugadores_.end())
	{
		if((*i) -> EstadoActual() != Jugador::FINALIZADO)
			x++;

		i++;
	}
	
	// En caso de no haber la partida ha terminado.
	if(!x)
		return false;

	// Pasamos al siguiente jugador.
	if(*ijugador == jugadores_.back())
		ijugador = jugadores_.begin();
	else
		ijugador++;

	// Si al jugador que hemos pasado no se encuentra inactivo seguimos pasando
	// hasta encontrar uno que esté inactivo.
	while((*ijugador) -> EstadoActual() != Jugador::INACTIVO)
	{
		if(*ijugador == jugadores_.back())
			ijugador = jugadores_.begin();
		else
			ijugador++;
	}
	
	return true;
}

void Partida::DesactivarJugador(Jugador *jugador)
{
	// Cambiamos el estado del jugador.
	jugador -> EstablecerEstado(Jugador::INACTIVO);
	
	// Cambiamos el estado de la bola.
	if(jugador -> BolaEnJuego().Estado() == Bola::ACTIVA)
		jugador -> BolaEnJuego().Estado(Bola::INACTIVA);

	else if(jugador -> BolaEnJuego().Estado() == Bola::BUNKER)
		jugador -> BolaEnJuego().Estado(Bola::BUNKER_INACTIVA);
	
	// Anulamos los punteros.
	jugador_activo_ = 0;
	bola_activa_ = 0;	
}

void Partida::FinalizarJugador(Jugador *jugador)
{
	// Cambiamos el estado de la bola y el jugador.
	jugador -> EstablecerEstado(Jugador::FINALIZADO);
	jugador -> BolaEnJuego().Estado(Bola::FINALIZADA);

	// Anulamos los punteros.
	jugador_activo_ = 0;
	bola_activa_ = 0;
}


void Partida::ColocarBolas()
{
	// Variable para controlar si la bola ha sido colocada.
	bool colocada = false;
	
	// Vector de bolas colocadas.
	std::vector<Bola*> colocadas;
	
	// Iterador de jugadores.
	std::vector<Jugador*>::iterator j;
	j = jugadores_.begin();
	
	// Desplazamiento fijo entre bolas.
	int desplazamiento = 0;
	
	// Desplazamiento aleatorio en el eje de las x.
	int desplazamiento_x = 0;
	
	// Desplazamiento aleatorio en el eje de las y.
	int desplazamiento_y = 0;
	
	// Signo aleatorio en el desplazamiento de las y.
	int signo = 0;
	
	// Recorremos los jugadores.
	while(j != jugadores_.end())
	{
		// Recorremos la lista de bolas.
		for(std::vector<Bola*>::iterator i = colocadas.begin();
			i != colocadas.end(); i++)
		{
			// Si la bola actual está en juego, ya ha sido colocada.
			if(&((*j) -> BolaEnJuego()) == *i)
			{
				colocada = true;
				break;
			}
		}
		
		// La bola no ha sido colocada.
		if(!colocada)
		{
			SDL_Rect tmp;

			// Destino para el eje de abcisas.
			tmp.x = campo_activo_ -> LugarSalida().x + desplazamiento
				+ desplazamiento_x;

			// Elegimos el signo y establecemos el destino para el eje de ordenadas.
			if(signo == 0)
				tmp.y = campo_activo_ -> LugarSalida().y + desplazamiento_y;
			else
				tmp.y = campo_activo_ -> LugarSalida().y + desplazamiento_y;
		
			// Actualizamos la posición.
			(*j) -> BolaEnJuego().Posicion(tmp);
			
			// Insertamos la bola en la lista.
			colocadas.push_back(&((*j) -> BolaEnJuego()));
			
			// Incrementamos el desplazamiento.
			desplazamiento += 25;
			
			// Desplazamiento aleatorio para las x.
			desplazamiento_x = Utilidades::NumeroAleatorio(3, 8);
			
			// Desplazamiento aleatorio para las y.
			desplazamiento_y = Utilidades::NumeroAleatorio(3, 8);
			
			// Signo aleatorio.
			signo = Utilidades::NumeroAleatorio(0, 1);			
		}
		else
			// La bola no ha sido colocada.
			colocada = false;
		
		// Pasamos al siguiente jugador.
		j++;
	}
}

bool Partida::FueraDeLimites(SDL_Rect *bola, SDL_Rect *sombra)
{
	// Definimos un incremento para las coordenadas.
	int inc = 20;
	
	// Coordenadas para la bola.
	int xbola1, xbola2, ybola1, ybola2;
	
	// Coordenadas para la sombra.
	int xsombra1, xsombra2, ysombra1, ysombra2;

	// Si el puntero a la posición de la bola es nulo o el puntero de la sombra
	// es nulo, tomamos las posiciones de la bola activa.
	if(bola == 0 || sombra == 0)
	{
		xbola1 = bola_activa_ -> PosicionBola().x - inc;
		xbola2 = bola_activa_ -> PosicionBola().x + inc;
		ybola1 = bola_activa_ -> PosicionBola().y - inc;
		ybola2 = bola_activa_ -> PosicionBola().y + inc;

		xsombra1 = bola_activa_ -> PosicionSombra().x - inc;
		xsombra2 = bola_activa_ -> PosicionSombra().x + inc;
		ysombra1 = bola_activa_ -> PosicionSombra().y - inc;
		ysombra2 = bola_activa_ -> PosicionSombra().y + inc;
	}
	
	// En caso contrario, empleamos las coordenadas de los rectángulos de
	// posición pasados a la función.
	else
	{
		xbola1 = bola -> x - inc;
		xbola2 = bola -> x + inc;
		ybola1 = bola -> y - inc;
		ybola2 = bola -> y + inc;

		xsombra1 = sombra -> x - inc;
		xsombra2 = sombra -> x + inc;
		ysombra1 = sombra -> y - inc;
		ysombra2 = sombra -> y + inc;	
	}
	
	// Comprobamos si alguna de las coordenadas supera los límites de la
	// pantalla.
	if(xbola1 <= 0 || xbola2 >= campo_activo_ -> Ancho() || ybola1 <= 0
		|| ybola2 >= campo_activo_ -> Alto()
		|| xsombra1 <= 0 || xsombra2 >= campo_activo_ -> Ancho() || ysombra1 <= 0
		|| ysombra2 >= campo_activo_ -> Alto())
		return true;
	else
		return false;
}


bool Partida::ComprobarColision(int modo, int elemento,
	SDL_Rect *bola, SDL_Rect *sombra)
{
	// Definimos los valores máximos para las coordenadas.
	int xmax, ymax;
	xmax = campo_activo_ -> Ancho();
	ymax = campo_activo_ -> Alto();
	
	// Creamos los punteros.
	const SDL_Rect *rbola;
	const SDL_Rect *rsombra;

	// Si los punteros son nulos usamos la bola activa.
	if(bola == 0 || sombra == 0)
	{
		rbola = &(bola_activa_ -> PosicionBola());
		rsombra = &(bola_activa_ -> PosicionSombra());
	}
	else
	{
		rbola = bola;
		rsombra = sombra;	
	}

	// Comprobamos el pixel del centro de la bola.
	if(modo == CENTRO)
	{
		// Pixel del centro de la bola.
		Uint32 pixel_bola;
		
		// Pixel del centro de la sombra.
		Uint32 pixel_sombra;

		// Obtenemos el color del pixel.
		pixel_bola = campo_activo_ -> Pixel(rbola -> x
			+ (rbola -> w / 2) , rbola -> y
			+	(rbola -> h / 2));
		
		// Si el palo es el putter, el pixel de la sombra es el mismo que el de
		// la bola.
		if(jugador_activo_ -> PaloActual().Nombre() == "Putter")
			pixel_sombra = pixel_bola;
			
		// En caso contrario obtenemos el pixel de la bola.
		else
			pixel_sombra = campo_activo_ -> Pixel(rsombra -> x
			 + (rsombra -> w / 2) , rsombra -> y
			 + (rsombra -> h / 2));

		// Iterador de elementos del campo.
		std::map< int, std::vector<Uint32> >::const_iterator im;
		
		// Iterador de colores de un determinado elemento.
		std::vector<Uint32>::const_iterator iv;
		
		// No se han producido colisiones todavía.
		bool colision_bola = false;
		bool colision_sombra = false;
		
		// Seleccionamos el elemento a analizar.
		switch(elemento)
		{
			case Campo::ARBOLES:
			{
				// Recorremos los colores.
				for(iv = ((campo_activo_ -> Elementos().find(Campo::ARBOLES))
				-> second).begin();
				iv != ((campo_activo_ -> Elementos().find(Campo::ARBOLES))
				-> second).end(); ++iv)
				{
					// El color procesado coincide con el del pixel del centro
					// de la bola.
					if(pixel_bola == *iv)
						colision_bola = true;

					// El color procesado coincide con el del pixel del centro
					// de la sombra.					
					if(pixel_sombra == *iv)
						colision_sombra = true;
				}
				
				// Devolvemos el valor de la colisión.
				return colision_bola && colision_sombra;
				
				break;
			}
			case Campo::AGUA:
			{
				// Recorremos los colores.
				for(iv = ((campo_activo_ -> Elementos().find(Campo::AGUA))
				-> second).begin();
				iv != ((campo_activo_ -> Elementos().find(Campo::AGUA))
				-> second).end(); ++iv)
				{
					// El color procesado coincide con el del pixel del centro
					// de la bola.
					if(pixel_bola == *iv)
						colision_bola = true;
						
					// El color procesado coincide con el del pixel del centro
					// de la sombra.	
					if(pixel_sombra == *iv)
						colision_sombra = true;
				}
				
				// Devolvemos el valor de la colisión.
				return colision_bola && colision_sombra;
			
				break;
			}

			case Campo::BUNKER:
			{
				// Recorremos los colores.
				for(iv = ((campo_activo_ -> Elementos().find(Campo::BUNKER))
				-> second).begin();
				iv != ((campo_activo_ -> Elementos().find(Campo::BUNKER))
				-> second).end(); ++iv)
				{
					// El color procesado coincide con el del pixel del centro
					// de la bola.
					if(pixel_bola == *iv)
						colision_bola = true;
						
					// El color procesado coincide con el del pixel del centro
					// de la sombra.	
					if(pixel_sombra == *iv)
						colision_sombra = true;
				}
				
				// Devolvemos el valor de la colisión.
				return colision_bola && colision_sombra;
			
				break;
			}

			case Campo::HOYO:
			{
				// Recorremos los colores.
				for(iv = ((campo_activo_ -> Elementos().find(Campo::HOYO))
				-> second).begin();
				iv != ((campo_activo_ -> Elementos().find(Campo::HOYO))
				-> second).end(); ++iv)
				{
					// El color procesado coincide con el del pixel del centro
					// de la bola.
					if(pixel_bola == *iv)
						colision_bola = true;

					// El color procesado coincide con el del pixel del centro
					// de la sombra.	
					if(pixel_sombra == *iv)
						colision_sombra = true;
				}

				// Devolvemos el valor de la colisión.
				return colision_bola && colision_sombra;
			
				break;
			}
			
			case Campo::TODOS:
			{
				// Recorremos los elementos..
				for(im = (campo_activo_ ->Elementos()).begin();
					im != (campo_activo_ ->Elementos()).end() ; im++)
				{
					// Recorremos los colores.
					for(iv = (*im).second.begin(); iv != (*im).second.end(); iv++)
					{
						// El color procesado coincide con el del pixel del centro
						// de la bola.
						if(pixel_bola == *iv)
							colision_bola = true;

						// El color procesado coincide con el del pixel del centro
						// de la sombra.
						if(pixel_sombra == *iv)
							colision_sombra = true;
					}
					
					// Devolvemos el valor de la colisión.
					return colision_bola && colision_sombra;	
				}	
			
				break;
			}
			
			default: return false;
		}
	}
	else if(modo == COMPLETO || modo == RESTAURACION)
	{
		// Vector de píxeles.
		std::vector<Uint32> pixels;
		
		// Incremento sobre las coordenadas.
		int inc;
		
		// En el modo de restauración el incremento es mayor para poder
		// situar la bola a una distancia suficiente de una zona de colisión.
		if(modo == RESTAURACION)
			inc = 20;
		else
			inc = 1;

		// Cuadrado a comprobar.
		int x, y, w, h;

		// Establecemos las coordenas.
		x = rbola -> x - inc;
		y = rbola -> y - inc;
		
		// Establecemos el ancho y el alto.
		w = rbola -> w + 2 * inc;
		h = rbola -> h + 2 * inc;

		int i, j;
		
		for(i = y; i < y + h; i++)
		{
			for(j = x; j < x + w; j++)
			{
				// En caso de que estemos restaurando la bola a una zona libre de
				// colisiones y los píxeles que vayamos a comprobar estén fuera de
				// límites, devolvemos verdadero para que la función de restaurar
				// siga iterando.
				if(modo == RESTAURACION && (j >= xmax || i >=ymax || j <= 0 || i <= 0))
					return true;

				pixels.push_back(campo_activo_ -> Pixel(j, i));
			}
		}


		// En el modo de restauración también tenemos que comprobar la posición
		// de la sombra de la bola (Fuera De Límites abajo...)
		if(modo == RESTAURACION)
		{
			int xs, ys, ws, hs;
			xs = rsombra -> x - inc;
			ys = rsombra -> y - inc;
			ws = rsombra -> w + 2 * inc;
			hs = rsombra -> h + 2 * inc;

			int i, j;
			for(i = ys; i < ys + hs; i++)
			{
				for(j = xs; j < xs + ws; j++)
				{
					// En caso de que estemos restaurando la bola a una zona libre de
					// colisiones y los píxeles que vayamos a comprobar estén fuera de
					// límites, devolvemos verdadero para que la función de restaurar
					// siga iterando.
					if(modo == RESTAURACION && (j >= xmax || i >=ymax
					|| j <= 0 || i <= 0))
						return true;

				}
			}
		}
		
		// Iterador de elementos.
		std::map< int, std::vector<Uint32> >::const_iterator im;
		
		// Iterador de color y píxel.
		std::vector<Uint32>::const_iterator iv, ip;
		
		// Seleccionamos elemento.
		switch(elemento)
		{
			case Campo::ARBOLES:
			{
				
				// Recorremos los colores.
				for(iv = ((campo_activo_ -> Elementos().find(Campo::ARBOLES))
				-> second).begin();
				iv != ((campo_activo_ -> Elementos().find(Campo::ARBOLES))
				-> second).end(); ++iv)
				{
						// Comprobamos los píxeles del cuadrado.
						for(ip = pixels.begin(); ip != pixels.end(); ip++)
						{
							// Determinamos si coincide el color de algún píxel del cuadrado
							// con el color de colisión.
							if(*(ip) == *(iv))
								return true;
						}
				}
			
				break;
			}
			case Campo::AGUA:
			{
				// Recorremos los colores.
				for(iv = ((campo_activo_ -> Elementos().find(Campo::AGUA))
				-> second).begin();
				iv != ((campo_activo_ -> Elementos().find(Campo::AGUA))
				-> second).end(); ++iv)
				{
						// Comprobamos los píxeles del cuadrado.
						for(ip = pixels.begin(); ip != pixels.end(); ip++)
						{
							// Determinamos si coincide el color de algún píxel del cuadrado
							// con el color de colisión.
							if(*(ip) == *(iv))
								return true;
						}
				}
			
				break;
			}
			case Campo::BUNKER:
			{
				// Recorremos los colores.
				for(iv = ((campo_activo_ -> Elementos().find(Campo::BUNKER))
				-> second).begin();
				iv != ((campo_activo_ -> Elementos().find(Campo::BUNKER))
				-> second).end(); ++iv)
				{
						// Comprobamos los píxeles del cuadrado.
						for(ip = pixels.begin(); ip != pixels.end(); ip++)
						{
							// Determinamos si coincide el color de algún píxel del cuadrado
							// con el color de colisión.
							if(*(ip) == *(iv))
								return true;
						}
				}
			
				break;
			}
			case Campo::HOYO:
			{
				// Recorremos los colores.
				for(iv = ((campo_activo_ -> Elementos().find(Campo::HOYO))
				-> second).begin();
				iv != ((campo_activo_ -> Elementos().find(Campo::HOYO))
				-> second).end(); ++iv)
				{
						// Comprobamos los píxeles del cuadrado.
						for(ip = pixels.begin(); ip != pixels.end(); ip++)
						{
							// Determinamos si coincide el color de algún píxel del cuadrado
							// con el color de colisión.
							if(*(ip) == *(iv))
								return true;
						}
				}
			
				break;
			}
			
			case Campo::TODOS:
			{
				// Recorremos los elementos.
				for(im = (campo_activo_ ->Elementos()).begin();
				im != (campo_activo_ ->Elementos()).end() ; im++)
				{
					// Recorremos los colores.
					for(iv = (*im).second.begin(); iv != (*im).second.end(); iv++)
					{
						// Comprobamos los píxeles del cuadrado.
						for(ip = pixels.begin(); ip != pixels.end(); ip++)
						{
							// Determinamos si coincide el color de algún píxel del cuadrado
							// con el color de colisión.
							if(*(ip) == *(iv))
								return true;
						}
					}

				}	
			
				break;
			}
			default: return false;
		}
	}
	return false;
}



int Partida::Pausar()
{
	// Pausamos el juego.
	return menu_ -> Pausa();
}

void Partida::ProcesarColision(int elemento)
{
	// Imagen para el texto a generar.
	SDL_Surface *texto;
	
	// Posición del texto.
	SDL_Rect rtexto;

	// Reiniciamos la animación del jugador.
	jugador_activo_ -> AnimacionActual().Reiniciar();

	// En caso de ser una cuadro particular, lo seleccionamos.
	if(jugador_activo_ -> AnimacionActual().CuadroSiguiente() / 9 == 6)
		jugador_activo_ -> SeleccionarCuadro(60);

	if(jugador_activo_ -> AnimacionActual().CuadroSiguiente() / 9 == 5)
		jugador_activo_ -> SeleccionarCuadro(52);
	
	if(jugador_activo_ -> AnimacionActual().CuadroSiguiente() / 9 == 4)
		jugador_activo_ -> SeleccionarCuadro(43);

	// Elegimos el tipo de colisión.
	switch(elemento)
	{
		case Campo::ARBOLES:
		{		
			SDL_Rect tmp;
			
			// Coordenadas para la animación.
			tmp.x = bola_activa_ -> PosicionBola().x - 13;
			tmp.y = bola_activa_ -> PosicionBola().y - 7;

			// Establecemos la posición de la animación.
			animacion_hojas_ -> Posicion(tmp);

			// Variables de tiempo para el control del bucle.
			Uint32 ta, tb;
			
			// Variables de tiempo para el control de la animación.
			Uint32 ta_animacion, tb_animacion;
			
			// Guardamos el tiempo para la animación.
			ta_animacion = SDL_GetTicks();
			
			// La animación no se ha relizado.
			while(!animacion_hojas_ -> Realizada())
			{
			
				// Guardamos el tiempo para el control del bucle.
				ta = SDL_GetTicks();

				// Guardamos el tiempo para el control de la animación.			
				tb_animacion = SDL_GetTicks();
				
				// Si el tiempo transcurrido es mayor que el retardo de la animación,
				// la dibujamos.
				if(tb_animacion - ta_animacion > animacion_hojas_ -> Retardo())
				{
					// Dibujamos.
					ActualizarBolasInactivas();
					campo_activo_ -> Dibujar();
					DibujarBolasInactivas();
					animacion_hojas_ -> Dibujar();
					jugador_activo_ -> Dibujar();
					campo_activo_ -> DibujarBandera();
					DibujarInterfaz();
					pantalla_ -> Actualizar();
					
					// Tomamos las nuevas referencias de tiempo.
					ta_animacion = SDL_GetTicks();
					tb_animacion = SDL_GetTicks();
				}
				
				// Guardamos el tiempo para el control del bucle.
				tb = SDL_GetTicks();
				
				// Si el tiempo transcurrido es menor que el establecido, esperamos.
				if(tb - ta < Constantes::_MS_)
					SDL_Delay(Constantes::_MS_ - (tb - ta));
			}
			
			// Reiniciamos la animación.
			animacion_hojas_ -> Reiniciar();

			// Esperamos.
			this -> Esperar(500, false, false, false, false, false, 0, 0);

			// Creamos el texto.
			texto = Utilidades::CrearTexto("Fuera", Constantes::_FUENTE_, 70,
				Utilidades::Color(Utilidades::BLANCO));
				
			// Establecemos las coordenadas del texto.
			rtexto.x = (pantalla_ -> Ancho() / 2) - (texto -> w / 2);
			rtexto.y = (pantalla_ -> Alto() / 2) - (texto -> h / 2);
			
			// Guardamos el ancho y el alto del texto.
			rtexto.w = texto -> w;
			rtexto.h = texto -> h;
			
			// Incrementamos el par.
			jugador_activo_ -> IncrementarPar();
			
			// Dibujamos.
			ActualizarBolasInactivas();
			campo_activo_ -> Dibujar();
			DibujarBolasInactivas();
			jugador_activo_ -> Dibujar();
			campo_activo_ -> DibujarBandera();
			DibujarInterfaz();
			pantalla_ -> DibujarDirectamente(texto, &rtexto);
			pantalla_ -> Actualizar();
						
			// Esperamos.
			this -> Esperar(3000, false, false, false, false, false,
				texto, &rtexto);

			// Destruimos la imagen del texto.
			SDL_FreeSurface(texto);
			
			// Restauramos la posición de la bola a una zona fuera de colisión.
			bola_activa_ -> RestaurarPosicion();

			break;		
		}		
		case Campo::AGUA:
		{
			SDL_Rect tmp;

			// Coordenadas para la animación.
			tmp.x = bola_activa_ -> PosicionBola().x - 13;
			tmp.y = bola_activa_ -> PosicionBola().y - 21;

			// Establecemos la posición de la animación.
			animacion_agua_ -> Posicion(tmp);

			// Variables de tiempo para el control del bucle.
			Uint32 ta, tb;
			
			// Variables de tiempo para el control de la animación.
			Uint32 ta_animacion, tb_animacion;
			
			// Guardamos el tiempo para la animación.
			ta_animacion = SDL_GetTicks();
			
			while(!animacion_agua_ -> Realizada())
			{
				// Guardamos el tiempo para el control del bucle.
				ta = SDL_GetTicks();
				
				// Guardamos el tiempo para el control de la animación.	
				tb_animacion = SDL_GetTicks();
				
				
				// Si el tiempo transcurrido es mayor que el retardo de la animación,
				// la dibujamos.
				if(tb_animacion - ta_animacion > animacion_agua_ -> Retardo())
				{
					// Dibujamos.
					ActualizarBolasInactivas();
					campo_activo_ -> Dibujar();
					DibujarBolasInactivas();
					animacion_agua_ -> Dibujar();
					jugador_activo_ -> Dibujar();
					campo_activo_ -> DibujarBandera();
					DibujarInterfaz();
					pantalla_ -> Actualizar();
					
					// Tomamos las nuevas referencias de tiempo.
					ta_animacion = SDL_GetTicks();
					tb_animacion = SDL_GetTicks();
				}
				
				// Guardamos el tiempo para el control del bucle.
				tb = SDL_GetTicks();
				
				// Si el tiempo transcurrido es menor que el establecido, esperamos.
				if(tb - ta < Constantes::_MS_)
					SDL_Delay(Constantes::_MS_ - (tb - ta));
			}

			// Reiniciamos la animación.
			animacion_agua_ -> Reiniciar();

			// Esperamos.
			this -> Esperar(500, false, false, false, false, false, 0, 0);

			// Creamos el texto.
			texto = Utilidades::CrearTexto("Agua", Constantes::_FUENTE_, 70,
				Utilidades::Color(Utilidades::BLANCO));
				
			// Establecemos las coordenadas del texto.
			rtexto.x = (pantalla_ -> Ancho() / 2) - (texto -> w / 2);
			rtexto.y = (pantalla_ -> Alto() / 2) - (texto -> h / 2);
			
			// Guardamos el ancho y el alto del texto.
			rtexto.w = texto -> w;
			rtexto.h = texto -> h;
			
			// Incrementamos el par.
			jugador_activo_ -> IncrementarPar();
			
			// Dibujamos.
			ActualizarBolasInactivas();
			campo_activo_ -> Dibujar();
			DibujarBolasInactivas();
			jugador_activo_ -> Dibujar();
			campo_activo_ -> DibujarBandera();
			DibujarInterfaz();
			pantalla_ -> DibujarDirectamente(texto, &rtexto);
			pantalla_ -> Actualizar();
						
			// Esperamos.
			this -> Esperar(3000, false, false, false, false, false,
				texto, &rtexto);
							
			// Destruimos la imagen del texto.
			SDL_FreeSurface(texto);

			// Restauramos la posición de la bola a una zona fuera de colisión.
			bola_activa_ -> RestaurarPosicion();
			
			break;
		}
		case Campo::BUNKER:
		{
			SDL_Rect tmp;

			// Coordenadas para la animación.
			tmp.x = bola_activa_ -> PosicionBola().x - 13;
			tmp.y = bola_activa_ -> PosicionBola().y - 21;

			// Establecemos la posición de la animación.
			animacion_bunker_ -> Posicion(tmp);

			// Variables de tiempo para el control del bucle.
			Uint32 ta, tb;
			
			// Variables de tiempo para el control de la animación.
			Uint32 ta_animacion, tb_animacion;
			
			// Guardamos el tiempo para la animación.
			ta_animacion = SDL_GetTicks();
			
			while(!animacion_bunker_ -> Realizada())
			{
				// Guardamos el tiempo para el control del bucle.
				ta = SDL_GetTicks();

				// Guardamos el tiempo para el control de la animación.	
				tb_animacion = SDL_GetTicks();
				
				// Si el tiempo transcurrido es mayor que el retardo de la animación,
				// la dibujamos.
				if(tb_animacion - ta_animacion > animacion_bunker_ -> Retardo())
				{
					// Dibujamos.
					ActualizarBolasInactivas();
					campo_activo_ -> Dibujar();
					DibujarBolasInactivas();
					animacion_bunker_ -> Dibujar();
					jugador_activo_ -> Dibujar();
					campo_activo_ -> DibujarBandera();
					DibujarInterfaz();
					pantalla_ -> Actualizar();

					// Tomamos las nuevas referencias de tiempo.
					ta_animacion = SDL_GetTicks();
					tb_animacion = SDL_GetTicks();
				}

				// Guardamos el tiempo para el control del bucle.
				tb = SDL_GetTicks();

				// Si el tiempo transcurrido es menor que el establecido, esperamos.
				if(tb - ta < Constantes::_MS_)
					SDL_Delay(Constantes::_MS_ - (tb - ta));
			}

			// Reiniciamos la animación.
			animacion_bunker_ -> Reiniciar();
			
			// Esperamos.
			this -> Esperar(500, true, true, false, false, false, 0, 0);

			// Creamos el texto.
			texto = Utilidades::CrearTexto("Bunker", Constantes::_FUENTE_, 70,
				Utilidades::Color(Utilidades::BLANCO));

			// Establecemos las coordenadas del texto.
			rtexto.x = (pantalla_ -> Ancho() / 2) - (texto -> w / 2);
			rtexto.y = (pantalla_ -> Alto() / 2) - (texto -> h / 2);
			
			// Guardamos el ancho y el alto del texto.
			rtexto.w = texto -> w;
			rtexto.h = texto -> h;

			// Incrementamos el par.
			jugador_activo_ -> IncrementarPar();
			
			// Cambiamos el estado de la bola.
			jugador_activo_ -> BolaEnJuego().Estado(Bola::BUNKER);
			
			// Dibujamos.
			ActualizarBolasInactivas();
			campo_activo_ -> Dibujar();
			DibujarBolasInactivas();
			jugador_activo_ -> BolaEnJuego().DibujarBola();
			jugador_activo_ -> Dibujar();
			campo_activo_ -> DibujarBandera();
			DibujarInterfaz();
			pantalla_ -> DibujarDirectamente(texto, &rtexto);
			pantalla_ -> Actualizar();
						
			// Esperamos.
			this -> Esperar(3000, true, true, false, false, false,
				texto, &rtexto);

			// Destruimos la imagen del texto.
			SDL_FreeSurface(texto);
			
			break;
		}
		case Campo::HOYO:
		{
			// Dibujamos.
			ActualizarBolasInactivas();
			campo_activo_ -> Dibujar();
			jugador_activo_ -> Dibujar();
			DibujarBolasInactivas();
			campo_activo_ -> DibujarBandera();
			DibujarInterfaz();
			pantalla_ -> Actualizar();

			// Esperamos.
			this -> Esperar(500, false, false, false, false, false, 0, 0);

			// Creamos el texto.
			texto = Utilidades::CrearTexto("Hoyo", Constantes::_FUENTE_, 70,
				Utilidades::Color(Utilidades::BLANCO));
				
			// Establecemos las coordenadas del texto.
			rtexto.x = (pantalla_ -> Ancho() / 2) - (texto -> w / 2);
			rtexto.y = (pantalla_ -> Alto() / 2) - (texto -> h / 2);
			
			// Guardamos el ancho y el alto del texto.
			rtexto.w = texto -> w;
			rtexto.h = texto -> h;

			// Dibujamos.
			ActualizarBolasInactivas();
			campo_activo_ -> Dibujar();
			jugador_activo_ -> Dibujar();
			DibujarBolasInactivas();
			campo_activo_ -> DibujarBandera();
			DibujarInterfaz();
			pantalla_ -> DibujarDirectamente(texto, &rtexto);
			pantalla_ -> Actualizar();
						
			// Esperamos.
			this -> Esperar(3000, false, false, false, false, false,
				texto, &rtexto);
	
			// Finalizamos el jugador.
			FinalizarJugador(*ijugador);

			// Destruimos la imagen del texto.
			SDL_FreeSurface(texto);
						
			break;

		}
		case Campo::FDL:
		{
			// Creamos el texto.
			texto =Utilidades:: CrearTexto("Fuera", Constantes::_FUENTE_, 70,
				Utilidades::Color(Utilidades::BLANCO));
				
			// Establecemos las coordenadas del texto.
			rtexto.x = (pantalla_ -> Ancho() / 2) - (texto -> w / 2);
			rtexto.y = (pantalla_ -> Alto() / 2) - (texto -> h / 2);
			
			// Guardamos el ancho y el alto del texto.
			rtexto.w = texto -> w;
			rtexto.h = texto -> h;

			// Incrementamos el par del jugador.
			jugador_activo_ -> IncrementarPar();
			
			// Dibujamos.
			ActualizarBolasInactivas();
			campo_activo_ -> Dibujar();
			DibujarBolasInactivas();
			jugador_activo_ -> Dibujar();
			DibujarInterfaz();
			pantalla_ -> DibujarDirectamente(texto, &rtexto);
			pantalla_ -> Actualizar();

			// Esperamos.
			this -> Esperar(3000, false, false, false, false, false,
				texto, &rtexto);
			
			// Destruimos la imagen del texto.
			SDL_FreeSurface(texto);
			
			// Restauramos la posición de la bola a una zona fuera de colisión.
			bola_activa_ -> RestaurarPosicion();		
			
			break;
		}
	} // switch
}

void Partida::DibujarInterfaz()
{
	// Dibujamos la interfaz de la partida.
	interfaz_ -> Dibujar();
}
void Partida::GenerarInterfaz()
{
	// Actualizamos la interfaz con los nuevos datos.
	interfaz_ -> Generar();
}

int Partida::ComprobarEventos()
{
	// Evento a procesar.
	SDL_Event evento;
	
	// Tecla del evento.
	SDLKey &tecla = evento.key.keysym.sym;
	
	while(SDL_PollEvent(&evento))
	{
		// Hemos cerrado la ventana. Terminamos el juego.
		if(evento.type == SDL_QUIT)
			return 1;

		if(evento.key.type == SDL_KEYDOWN)
		{
			// Pantalla completa.
			if(tecla == teclado_.Tecla(Teclado::PANTALLA_COMPLETA))
				pantalla_ -> PantallaCompleta();

			// Pausamos.
			if(tecla == teclado_.Tecla(Teclado::PAUSA)
				|| tecla == teclado_.Tecla(Teclado::ESCAPE))
			{
				int retorno;
				retorno = this -> Pausar();
				campo_activo_ -> Dibujar();
				DibujarBolasInactivas();
				bola_activa_ -> DibujarSombra();
				bola_activa_ -> DibujarBola();
				jugador_activo_ -> Dibujar();
				jugador_activo_ -> CursorActual().Dibujar();
				DibujarInterfaz();
				pantalla_ -> Actualizar();

				// -1 si hay que salir, 0 si hay que continuar.
				return retorno;
			}		
		}
	}
	
	return 0;
}

void Partida::CargarMusica()
{
	// Generamos un número aleatorio.
	std::stringstream ss;
	ss << Utilidades::NumeroAleatorio(1, 7);
	
	// Generamos la ruta.
	std::string ruta = "./snd/musica/" + ss.str() + ".ogg";
	
	// Caragmos la música.
	ambiente_.Cargar(ruta.c_str());
}

bool Partida::EfectosActivos() const
{
	// Devolvemos el estado de los efectos.
	return efectos_;
}

const std::vector<int>& Partida::Puntuaciones() const
{
	// Devolvemos las puntuaciones de la partida.
	return puntuaciones_;
}

const std::vector<Jugador*>& Partida::Jugadores() const
{
	// Devolvemos la lista de jugadores.
	return jugadores_;
}

void Partida::Esperar(Uint32 ms, bool bola, bool sombra, bool cursor,
	bool fuerza, bool efecto, SDL_Surface *texto, SDL_Rect *rtexto)
{
	Uint32 ta_for, tb_for;
	Uint32 ta_control, tb_control;
	
	ta_control = SDL_GetTicks();
	
	for(;;)
	{
		ta_for = SDL_GetTicks();
		
		campo_activo_ -> Dibujar();
		DibujarBolasInactivas();
		
		if(sombra)
			bola_activa_ -> DibujarSombra();
			
		if(bola)
			bola_activa_ -> DibujarBola();
			
		jugador_activo_ -> Dibujar();
		
		if(cursor)
			jugador_activo_ -> CursorActual().Dibujar();

		if(fuerza)
			jugador_activo_
				-> IndicadorActual().AnimacionFuerza().DibujarCuadro();

		if(efecto)	
			jugador_activo_ -> IndicadorActual().AnimacionEfecto().DibujarCuadro();
			
		campo_activo_ -> DibujarBandera();
		DibujarInterfaz();
		
		if(texto != 0 && rtexto != 0)
			pantalla_ -> DibujarDirectamente(texto, rtexto);
			
		pantalla_ -> Actualizar();		

		tb_for = SDL_GetTicks();

		if(tb_for - ta_for < Constantes::_MS_)
			SDL_Delay(Constantes::_MS_ - (tb_for - ta_for));
			
		
		tb_control = SDL_GetTicks();

		if(tb_control - ta_control > ms)
			break;	
	}
}

void Partida::ActualizarPalo()
{
	interfaz_ -> ActualizarPalo();
}

void Partida::ActualizarPar()
{
	interfaz_ -> ActualizarPar();
}
