/// \file partida.h Partida.
/*
	Copyright 2010 Andrés Francisco Aparicio

	This file is part of SDLgolf.

	SDLgolf is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	SDLgolf is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with SDLgolf.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef _PARTIDA_H_
#define _PARTIDA_H_

#include <iostream>
#include <vector>
#include <string>
#include "teclado.h"
#include "musica.h"
#include "datos.h"

class Campo;
class Jugador;
class Pantalla;
class Camara;
class Bola;
class Menu;
class Rejilla;
class Animacion;
class Interfaz;

/*! \class Partida
    \brief Clase virtual.

		La clase partida engloba todos las funciones compartidas por los
		diferentes modos de juego.
		
		Desde esta clase se podrá controlar y acceder todos los elementos que
		participan en una partida.
				
		La clase partida también se encarga de detectar y procesar las colisiones
		de la bola de golf con los elementos del campo.
 */
class Partida
{
	public:

		/*! Destruye las bolas, los jugadores, el campo, las animaciones y
				la interfaz.
				\brief Destructor virtual de Partida.
		 */
		virtual ~Partida();
		
		/*! Inicia la Partida.
				\brief Función virtual
				\return \arg \b -1 Si hemos cerrado la ventana.
								\arg \b 0 En otro caso.
		 */		
		virtual int Iniciar() = 0;
		
		/*! \brief Dibuja la Interfaz de la partida.
				\see Interfaz::Dibujar.
		 */		
		void DibujarInterfaz();
		
		/*! Actualiza la Interfaz de la partida con los nuevos valores.
				\brief Actualiza la Interfaz.
				\see Interfaz::Generar.
		 */
		void GenerarInterfaz();
		
		/*! Muestra un rótulo con el nombre del Campo y el número de hoyo que
				se va a jugar.
				\brief Muestra el nombre del Campo.
				\see Campo::Nombre.
				\see Campo::NumeroHoyo.
		 */		
		void DibujarNombreCampo();

		/*! \brief Procesa una determinada colisión.
				\param elemento Elemento con el que se ha producido la colisión.
				\see Campo::ElementosCampo.
		 */	
		void ProcesarColision(int elemento);
		
		/*! \brief Comprueba que la Bola esté dentro de los límites del Campo.
				\param bola Posición de la Bola.
				\param sombra Posición de la sombra de la Bola.
				\return \arg \b Verdadero Si la Bola se encuentra fuera de los
												límites de la Pantalla.
								\arg \b Falso Si la Bola se encuentra dentro de los límites
												de la Pantalla.
				\see Bola::PosicionBola.
				\see Bola::PosicionSombra.
				\see Pantalla::Ancho.
				\see Pantalla::Alto.
		 */
		bool FueraDeLimites(SDL_Rect *bola, SDL_Rect *sombra);

		/*! \brief Activa un determinado Jugador.
				\param jugador Jugador que se quiere activar.
				\see Jugador::Estados.
		 */		
		void ActivarJugador(Jugador *jugador);

		/*! \brief Devuelve el Campo activo.
				\return Campo en el que se está desarrollando la Partida actualmente.
		 */	
		Campo& CampoActivo();

		/*! \brief Devuelve el Jugador activo.
				\return Jugador que está activo en la Partida actualmente.
				\see Jugador::Estados.
		 */
		Jugador& JugadorActivo();

		/*!	\brief Centra la Camara en la Bola.
				\see Pantalla::CentrarCamara.
		 */
		void CentrarCamara();

		/*! \brief Dibuja en la Pantalla las bolas que esten inactivas.
				\see Bola::Estados.
		 */
		void DibujarBolasInactivas();

		/*! \brief Actualiza la lista de bolas inactivas.
				\see Bola::Estados.
		 */
		void ActualizarBolasInactivas();

		/*! \brief Orienta el Cursor al hoyo del Campo.
				\see Cursor::Orientar.
				\see Campo::PosicionHoyo.
		 */
		void OrientarCursor();

		/*! \brief Coloca las bolas de la Partida partiendo de la posición inicial.
				\see Campo::LugarSalida.
		 */
		void ColocarBolas();

		/*! Cambia el puntero del jugador activo al siguiente jugador que esté
				en estado inactivo.
				\brief Activa el siguiente jugador.
				\return \arg \b Verdadero Si se ha realizado el cambio puesto que
												existían jugadores inactivos.
								\arg \b Falso Si no se ha producido el cambio dado que
												no existían jugadores inactivos.
				\see Jugador::Estados.
		 */
		bool SiguienteJugador();

		/*!	\brief Desactiva a un determinado Jugador.
				\param jugador Jugador que se quiere desactivar.
				\see Jugador::Estados.
		 */
		void DesactivarJugador(Jugador *jugador);

		/*!	\brief Finaliza a un determinado Jugador.
				\param jugador Jugador que se quiere finalizar.
				\see Jugador::Estados.
		 */
		void FinalizarJugador(Jugador *jugador);

		/*!	\brief Comprueba la colisión de la Bola con los elementos del Campo.
				\param modo Modo de comprobación de colisión.
				\param elemento Elemento del campo a comprobar.
				\param bola Posición de la Bola con la que se llevará a cabo
										la comprobación.
				\param sombra Posición de la sombra de la Bola.
				\note Si \e bola o \e sombra son nulos, se tomará la posición de
							la Bola del Jugador activo para la comprobación de la colisión.
				\see Partida::JugadorActivo.
				\see Jugador::BolaEnJuego.
				\see Bola::PosicionBola.
				\see Bola::PosicionSombra.
		 */		
		bool ComprobarColision(int modo, int elemento,
			SDL_Rect *bola, SDL_Rect *sombra);

		/*! \brief Pausa la partida actual.
				\return \arg \b -1 Si se sale de la Partida.
								\arg \b 1 Si se ha cerrado la ventana del Juego.
								\arg \b 0 Si se continua la Partida.
		 */
		int Pausar();

		/*! Comprueba los eventos que genera el usuario mientras está jugando el
				Ordenador y los procesa en caso de que se cierre la ventana o el
				juego sea pausado.
				\brief Comprueba los eventos del usuario.
				\return \arg \b -1 Si se sale de la Partida.
								\arg \b 1 Si se ha cerrado la ventana del Juego.
								\arg \b 0 Si se continua la Partida.
		 */		
		int ComprobarEventos();

		/*! Se elige una pista aleatoria entre las disponibles.
				\brief Carga la música ambiente de la partida.
				\see Utilidades::NumeroAleatorio.
		 */
		void CargarMusica();

		/*!	Función observadora. Devuelve si los efectos de sonido están
				activados o no.
				\brief Estado de los efectos de sonido.
				\return \arg \b Verdadero Si los efectos de sonido están activados.
								\arg \b Falso Si los efectos de sonido no están activados.
		 */
		bool EfectosActivos() const;


		/*!	Función observadora. Devuelve las puntuaciones de los jugadores
				que participan en la partida.
				\brief Puntuaciones de los jugadores.
				\return Puntuaciones de los jugadores.
		 */
		const std::vector<int>& Puntuaciones() const;
		

		/*!	Función observadora. Devuelve los jugadores que participan en la
				partida.
				\brief Jugadores de la partida.
				\return Jugadores que participan en la partida.
		 */	
		const std::vector<Jugador*>& Jugadores() const;

		/*!	Pone la partida en espera un determinado tiempo permitiendo que la
				animación de la bandera se siga dibujando.\n
				También permite realizar la espera cuando se dibuja el rótulo del
				resultado de una colisión.\n
				Cuando la partida está en espera el juego no responde a los eventos
				que pueda producir el usuario.
				\brief Espera un determinado intervalo de tiempo.
				\param ms Tiempo en milisegundos que se desea esperar.
				\param bola Indica si se ha de dibujar la bola.
				\param sombra Indica si se ha de dibujar la sombra.
				\param cursor Indica si se ha de dibujar el cursor.
				\param fuerza Indica si se ha de dibujar el indicador de fuerza.
				\param efecto Indica si se ha de dibujar el indicador de efecto.
				\param texto Imagen del texto que se desea dibujar.
				\param rtexto Posición del texto.
		 */	
		void Esperar(Uint32 ms, bool bola, bool sombra, bool cursor,
			bool fuerza, bool efecto, SDL_Surface *texto, SDL_Rect *rtexto);

		/*! \brief Actualiza el palo de la interfaz.
				\see Jugador::PaloActual.
		 */	
		void ActualizarPalo();

		/*! \brief Actualiza el par del Jugador en la interfaz.
				\see Jugador::Par.
		 */	
		void ActualizarPar();

		/*! Enumeración de los modos de comprobación de colisión.
		 */

		/*! \enum ModosComprobacion
				\brief Modos de comprobación de colisión.

				\var CENTRO
		 		Comprueba la colisión en el píxel del centro de la Bola.
		 		
		 		\var COMPLETO
		 		Comprueba la colisión en todos los píxeles de la Bola.
		 		
		 		\var RESTAURACION
		 		Comprueba la colisión en todos los píxeles de la Bola y en una
		 		distancia determinada a su alrededor.

				\see ComprobarColision.
				\see Campo::ElementosCampo.
		 */	
		enum ModosComprobacion { CENTRO, COMPLETO, RESTAURACION };
		
		
	protected:
	
		Teclado teclado_;	///< Teclado de la partida.
		
		Musica ambiente_;	///< Música de fondo.
		
		std::vector<Bola*> bolas_;	///< Bolas de la partida.
		
		std::vector<Jugador*> jugadores_;	///< Jugadores que participan en la
																			///< partida.

		std::vector<Bola*> bolas_inactivas_;	///< Bolas en estado inactivo.

		std::vector<Jugador*>::iterator ijugador;	///< Iterador de jugadores.

		Jugador *jugador_activo_;	///< Jugador que está activo.

		Campo *campo_activo_;	///< Campo que está activo.

		Bola *bola_activa_;	///< Bola del jugador activo.

		Pantalla *pantalla_;	///< Pantalla del Juego.
		
		Menu *menu_;	///< Menú del Juego.
		
		Interfaz *interfaz_;	///< Interfaz de la partida.
		
		std::vector<int> puntuaciones_;	///< Puntuaciones de los jugadores.
		
		Rejilla *rejilla_agua_;	///< Rejilla de la animación del agua.
		
		Animacion *animacion_agua_;	///< Animación del agua.

		Rejilla *rejilla_bunker_;	///< Rejilla de la animación del bunker.

		Animacion *animacion_bunker_;	///< Animación del bunker.

		Rejilla *rejilla_hojas_;	///< Rejilla de la animación de las hojas.
		
		Animacion *animacion_hojas_;	///< Animación de las hojas.
		
		bool efectos_;	///< Variable de control de los efectos de sonido.
		
		bool musica_;	///< Variable de control de la música del juego.

};

#endif
