/// \file rejilla.h Rejilla de imágenes.
/*
	Copyright 2010 Andrés Francisco Aparicio

	This file is part of SDLgolf.

	SDLgolf is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	SDLgolf is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with SDLgolf.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef _REJILLA_H_
#define _REJILLA_H_

#include <SDL/SDL.h>

/*! \class Rejilla.
    \brief Rejilla de imágenes.

    La clase rejilla permite el acceso a los diferentes cuadros que componen
    una rejilla de imágenes.
 */
class Rejilla
{
	public:

		/*! Carga la rejilla de imágenes, establece el número de filas y de
				columnas y selecciona un cuadro nulo.
				\brief Constructor.
				\param ruta Ruta de la rejilla que se quiere cargar.
				\param filas Número de filas que posee la rejilla.
				\param columnas Número de columnas que posee la rejilla.
		 */
		Rejilla(std::string ruta, int filas, int columnas);

		/*! Destruye la imagen de la rejilla.
				\brief Destructor.
		 */
		~Rejilla();

		/*! Selecciona un determinado cuadro en la rejilla de imágenes.
				\brief Selecciona un cuadro de la rejilla.
				\param cuadro Número de cuadro que se desea seleccionar.
		 */
		void SeleccionarCuadro(int cuadro);

		/*! \brief Devuelve el total de cuadros que posee la rejilla.
				\return Número de cuadros de la rejilla.
		 */
		int TotalCuadros() const;

		/*! \brief Devuelve la imagen del cuadro de la rejilla seleccionado.
				\return Imagen del cuadro seleccionado actualmente.
				\see CuadroActual.
				\see SeleccionarCuadro.
		 */
		SDL_Surface * Imagen();

		/*! \brief Devuelve la posición del cuadro de la rejilla seleccionado.
				\return Posicion del cuadro seleccionado actualmente.
				\see CuadroActual.
				\see SeleccionarCuadro.
		 */
		SDL_Rect& Posicion();

		/*! \brief Devuelve el cuadro de la rejilla que está seleccionado
							 actualmente.
				\return Cuadro de la rejilla.
				\see SeleccionarCuadro.
		 */
		int CuadroActual() const;
	
	private:
	
		int cuadro_;	///< Cuadro de la rejilla seleccionado.
		
		int filas_;	///< Número de filas de la rejilla.
		
		int columnas_;	///< Número de columnas de la rejilla.
		
		SDL_Surface *rejilla_;	///< Imagen de la rejilla.
		
		SDL_Rect rrejilla_;	///< Posición de la rejilla.
};

#endif
