/// \file strokeplay.cpp Modo de juego Strokeplay.
/*
	Copyright 2010 Andrés Francisco Aparicio

	This file is part of SDLgolf.

	SDLgolf is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	SDLgolf is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with SDLgolf.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "strokeplay.h"
#include "bola.h"
#include "jugador.h"
#include "campo.h"
#include "rejilla.h"
#include "animacion.h"
#include "humano.h"
#include "ordenador.h"
#include "interfaz.h"
#include "pantalla.h"
#include "constantes.h"
#include "menu.h"

Strokeplay::Strokeplay(std::vector<Datos::Jugador>& jugadores,
	std::vector<Datos::Campo>& campos, Menu *menu, Pantalla *pantalla)
{
	// Asociamos con la pantalla.
	pantalla_ = pantalla;
	
	// Asociamos con el menú del juego.
	menu_ = menu;
	
	// Asignamos 0 a las puntuaciones de los jugadores.
	puntuaciones_.assign(4, 0);
	
	// Copiamos los datos de los campos.
	datos_campos_ = campos;

	// Establecemos el estado de los efectos.
	efectos_ = menu_ -> EfectosActivos();
	
	// Establecemos el estado de la música.
	musica_ = menu_ -> MusicaActiva();
	
	// Equipo.
	int x = 0;
	
	// Iterador de jugadores.
	std::vector<Datos::Jugador>::iterator i;
	i = jugadores.begin();
	
	// Recorremos los datos de los jugadores.
	while(i != jugadores.end())
	{
		// Creamos una nueva bola.
		bolas_.push_back(new Bola(this, pantalla_));

		// Creamos el nuevo jugador dependiendo del tipo.		
		if(i -> tipo_ == Jugador::HUMANO)
			jugadores_.push_back(new Humano(i -> nombre_, i -> color_,
				*bolas_.back(), this, pantalla_));
		else if(i -> tipo_ == Jugador::IA_BAJA
			|| i -> tipo_ == Jugador::IA_MEDIA
			|| i -> tipo_ == Jugador::IA_ALTA)
			jugadores_.push_back(new Ordenador(i -> nombre_, i -> color_,
				i -> tipo_, *bolas_.back(), this, pantalla_));

		// Establecemos el equipo.		
		jugadores_.back() -> Equipo(x);
		
		// Establecemos el color de la bola inactiva.
		bolas_.back() -> Color(i -> color_);
		
		// Siguiente jugador.
		++i;
		
		// Incrementamos el equipo.
		++x;
	}

	// La partida no ha comenzado. Establecemos el hoyo actual a 0.
	hoyo_actual_ = 0;

	// Creamos el primer hoyo.
	campo_activo_ = new Campo(campos[0].nombre_, campos[0].hoyo_,
		campos[0].par_, &(campos[0].posicion_salida_),
		&(campos[0].posicion_hoyo_), campos[0].colores_, campos[0].ruta_,
		pantalla_);

	// Colocamos las bolas en el campo.	
	ColocarBolas();
	
	// Activamos el primer jugador.	
	ActivarJugador(jugadores_[0]);
	
	// Actulizamos la lista de bolas inactivas.	
	ActualizarBolasInactivas();

	// Ruta a las animaciones.
	std::string ruta_animaciones =
		Constantes::_IMG_ANIMACIONES_ + campos[0].animaciones_ + "/";

	// Cargamos la rejilla de la animación del agua.	
	rejilla_agua_ = new Rejilla(ruta_animaciones + "agua.png", 1, 5);
	
	// Generamos la secuencia de animación.	
	int sagua[] = {1, 2, 3, 4, 0, -1};
	
	// Creamos la animación del agua.	
	animacion_agua_ = new Animacion(rejilla_agua_, sagua, 40, pantalla_);
	
	// Cargamos la rejilla de la animación del bunker.
	rejilla_bunker_ = new Rejilla(ruta_animaciones + "bunker.png", 1, 5);

	// Generamos la secuencia de animación.
	int sbunker[] = {1, 2, 3, 4, 0, -1};

	// Creamos la animación del bunker.
	animacion_bunker_ = new Animacion(rejilla_bunker_, sbunker, 40, pantalla_);

	// Cargamos la rejilla de la animación de las hojas.
	rejilla_hojas_ = new Rejilla(ruta_animaciones + "hojas.png", 1, 5);

	// Generamos la secuencia de animación.
	int shojas[] = {1, 2, 3, 4, 0, -1};

	// Creamos la animación de las hojas.
	animacion_hojas_ = new Animacion(rejilla_hojas_, shojas, 40, pantalla_);

	// Creamos la interfaz de la partida.
	interfaz_ = new Interfaz(this, pantalla_);
}

int Strokeplay::Iniciar()
{
	// Si la música está activada y no hay música de fondo reproducíendose
	// cargamos una canción.
	if(musica_)
		if(!ambiente_.Reproduciendo())
		{
			this -> CargarMusica();
			ambiente_.Reproducir(1);
		}
			
	// Establecemos el filtrado de eventos (utilidades.cpp)
	SDL_SetEventFilter(Utilidades::FiltroDeEventos);

	// Centramos la cámara.
	CentrarCamara();
	
	// Dibujamos el campo.	
	campo_activo_ -> Dibujar();
	
	// Mostramos el rótulo con el nombre del campo.	
	this -> DibujarNombreCampo();
	
	// Actualizamos la pantalla.
	pantalla_ -> Actualizar();
	
	// Esperamos.	
	SDL_Delay(2500);

	// Dibujamos todo.
	CentrarCamara();
	campo_activo_ -> Dibujar();
	DibujarBolasInactivas();
	bola_activa_ -> DibujarSombra();
	bola_activa_ -> DibujarBola();
	OrientarCursor();
	jugador_activo_ -> ActualizarCuadro();
	jugador_activo_ -> IndicadorActual().ActualizarPosicion();
	jugador_activo_ -> Dibujar();
	jugador_activo_ -> CursorActual().Dibujar();
	campo_activo_ -> DibujarBandera();

	// Generamos la interfaz y la dibujamos.	
	GenerarInterfaz();
	DibujarInterfaz();

	// Actualizamos la pantalla.
	pantalla_ -> Actualizar();

	// Variable que controla si hemos terminado un tiro.
	bool terminado = true;

	// Variable de control para el establecimiento de fuerza.
	bool fuerza_establecida = false;

	// Varaible de control para el establecimiento de efecto.
	bool efecto_establecido = false;

	// El iterador de la lista de jugadores apunta al primer jugador.	
	ijugador = jugadores_.begin();
	
	// Variable para controlar si la tecla ha sido liberada.
	bool tecla_soltada = false;

	// Variable que controla si la tecla del mapa está activa.	
	bool tecla_mapa = false;

	// Variable para controla si teneos que dibujar el mapa.	
	bool dibujar_mapa = false;

	// Evento para tratar.
	SDL_Event evento;
	
	// Tecla del evento.
	SDLKey &tecla = evento.key.keysym.sym;
	
	// Control de tiempo para el bucle principal.
	Uint32 ta_for, tb_for;
	
	// Control de tiempo para la cola de eventos.
	Uint32 ta_while, tb_while;
	
	for(;;)
	{
		// Guardamos el tiempo para el control del bucle de la partida.
		ta_for = SDL_GetTicks();

		// Si la música está activa y no se está reproduciendo ninguna pista
		// cargamos una.
		if(musica_)
			if(!ambiente_.Reproduciendo())
			{
				this -> CargarMusica();
				ambiente_.Reproducir(1);
			}

		// Bucle para la cola de eventos.		
		while(SDL_PollEvent(&evento))
		{
			// Guardamos el tiempo para el control del bucle de la cola de eventos.
			ta_while = SDL_GetTicks();
	
			if(evento.type == SDL_QUIT)
				return -1;

			// Se ha pulsado una tecla.
			if(evento.key.type == SDL_KEYDOWN)
			{
				// Pantalla completa.
				if(tecla == teclado_.Tecla(Teclado::PANTALLA_COMPLETA))
					pantalla_ -> PantallaCompleta();

				// Menú de pausa.
				if(tecla == teclado_.Tecla(Teclado::PAUSA)
					|| tecla == teclado_.Tecla(Teclado::ESCAPE))
				{
					int retorno;
					retorno = this -> Pausar();
					// Hemos seleccionado Salir desde el menú de Pausa.
					// Volvemos al Menú Principal.
					if(retorno == -1)
						return 0;
					// Hemos pulsado la x de cerrar ventana. Salimos del juego.
					else if(retorno == 1)
						return -1;

					// Dibujamos todo y actualizamos.
					campo_activo_ -> Dibujar();
					DibujarBolasInactivas();
					bola_activa_ -> DibujarSombra();
					bola_activa_ -> DibujarBola();
					jugador_activo_ -> Dibujar();
					jugador_activo_ -> CursorActual().Dibujar();
					campo_activo_ -> DibujarBandera();
					DibujarInterfaz();
					pantalla_ -> Actualizar();
				}

				// Tecla de golpeo.
				if(tecla == teclado_.Tecla(Teclado::ESPACIO))
				{
					// Vaciamos la cola de eventos.
				  SDL_Event tmp;
				  while(SDL_PollEvent(&tmp)){}

					// Establecemos el tipo de evento a 0 para evitar solapamientos.					
					evento.type = 0;

					// Ocultamos el mapa.					
					dibujar_mapa = false;
					interfaz_ -> DesactivarMapa();
					
					// Control de tiempo para la animación de fuerza.
					Uint32 ta_fuerza, tb_fuerza;

					// Control de tiempo para el bucle.
					Uint32 ta, tb;

					// Guardamos el tiempo para la animación del indicador de fuerza.
					ta_fuerza = SDL_GetTicks();

					while(1)
					{
						// Guardamos el tiempo para el control del while.
						ta = SDL_GetTicks();

						// Establecemos el tipo de evento a 0 para evitar solapamientos.					
						evento.type = 0;

						// Extraemos el evento.
						SDL_PollEvent(&evento);

						// Si el evento es de salida, terminamos el juego.
						if(evento.type == SDL_QUIT)
							return -1;																		

						// Cancelamos el tiro.			
						if(tecla == teclado_.Tecla(Teclado::ESCAPE))
						{
							// Dibujamos todo.
							campo_activo_ -> Dibujar();
							DibujarBolasInactivas();
							bola_activa_ -> DibujarSombra();
							bola_activa_ -> DibujarBola();
							jugador_activo_ -> Dibujar();
							jugador_activo_ -> CursorActual().Dibujar();
							campo_activo_ -> DibujarBandera();
							DibujarInterfaz();
							pantalla_ -> Actualizar();

							// Reiniciamos la animación del indicador de fuerza.
							jugador_activo_
								-> IndicadorActual().AnimacionFuerza().Reiniciar();

						// La tecla no ha sido soltada.
							tecla_soltada = false;
						
							break;
						}
						
						// Se libera la tecla.
						if(evento.type == SDL_KEYUP
							&& tecla == teclado_.Tecla(Teclado::ESPACIO)
							&& !tecla_soltada)
							tecla_soltada = true;

						// Se vuelve a pulsar la tecla espacio.				
						if(evento.type == SDL_KEYDOWN
							&& tecla == teclado_.Tecla(Teclado::ESPACIO)
							&& tecla_soltada)
						{
							// Valor de la fuerza.
							int fuerza;
						
							// Leemos la fuerza.
							fuerza = jugador_activo_
								-> IndicadorActual().AnimacionFuerza().CuadroProcesado();

							// La guardamos.							
							jugador_activo_ -> IndicadorActual().Fuerza(fuerza);

							// La fuerza ha sido establecida.						
							fuerza_establecida = true;

							// La tecla nos e ha soltado.
							tecla_soltada = false;

							break;							
						}

						// Guardamos el tiempo para la animación.				
						tb_fuerza = SDL_GetTicks();

						// Si el tiempo transcurrido es mayor que el retardo de la
						// animación del indicador de fuerza, lo dibujamos.			
						if(tb_fuerza - ta_fuerza > jugador_activo_
							-> IndicadorActual().AnimacionFuerza().Retardo())
						{
							// Dibujamos todo.
							campo_activo_ -> Dibujar();
							DibujarBolasInactivas();
							bola_activa_ -> DibujarSombra();
							bola_activa_ -> DibujarBola();
							jugador_activo_ -> Dibujar();
							jugador_activo_ -> CursorActual().Dibujar();
							jugador_activo_ -> IndicadorActual().AnimacionFuerza().Dibujar();
							campo_activo_ -> DibujarBandera();
							DibujarInterfaz();
							pantalla_ -> Actualizar();

							// Tomamos la nueva referencia de tiempo.
							ta_fuerza = SDL_GetTicks();
						}

						// Guardamos el tiempo.			
						tb = SDL_GetTicks();

						// Si el tiempo transcurrido es menor que el establecido en el
						// juego, esperamos.
						if(tb - ta < Constantes::_MS_)
							SDL_Delay(Constantes::_MS_ - (tb-ta));
					}

					// Vaciamos la cola de eventos.
				  while(SDL_PollEvent(&tmp)){}

					// Control de tiempo para la animación de efecto.
					Uint32 ta_efecto, tb_efecto;

					// Guardamos el tiempo.					
					ta_efecto = SDL_GetTicks();			
			
					// La fuerza se ha establecido.			
					while(fuerza_establecida)
					{
						// Guardamos el tiempo para el control del bucle.
						ta = SDL_GetTicks();

						// Establecemos el tipo de evento a 0 para evitar solapamientos.					
						evento.type = 0;

						// Extraemos el evento.
						SDL_PollEvent(&evento);

						// Si el evento es de salida, terminamos el juego.						
						if(evento.type == SDL_QUIT)
							return -1;		

						// Hemos liberado la tecla.
						if(evento.type == SDL_KEYUP
							&& tecla == teclado_.Tecla(Teclado::ESPACIO)
							&& !tecla_soltada)
							tecla_soltada = true;

						// Menú de pausa.
						if(evento.type == SDL_KEYDOWN &&
							(tecla == teclado_.Tecla(Teclado::PAUSA)
							|| tecla == teclado_.Tecla(Teclado::ESCAPE)))
						{
							// Retorno del menú.
							int retorno;
							retorno = this -> Pausar();
					
							// Hemos seleccionado Salir desde el menú de Pausa.
							// Volvemos al Menú Principal.
							if(retorno == -1)
								return 0;
							// Hemos pulsado la x de cerrar ventana. Salimos del juego.
							else if(retorno == 1)
								return -1;

						}

						// Hemos pulsado espacio.			
						if(evento.type == SDL_KEYDOWN
							&& tecla == teclado_.Tecla(Teclado::ESPACIO)
							&& tecla_soltada)
						{
							// Valor para el efecto.
							int efecto;

							// Establecemos el efecto en función del cuadro de la animación.
							switch(jugador_activo_
								-> IndicadorActual().AnimacionEfecto().CuadroProcesado())
							{
								case 0: efecto = 0; break;
								case 1: efecto = 5; break;
								case 2: efecto = 10; break;
								case 3: efecto = 15; break;
								case 4: efecto = 25; break;
								case 5: efecto = 30; break;
								case 6: efecto = -5; break;
								case 7: efecto = -10; break;
								case 8: efecto = -15; break;
								case 9: efecto = -25; break;
								case 10: efecto = -30; break;
								default: efecto = 0; break;
							}

							// Establecemos el efecto.					
							jugador_activo_ -> IndicadorActual().Efecto(efecto);

							// El efecto se ha establecido.
							efecto_establecido = true;

							// No se ha soltado la tecla.
							tecla_soltada = false;
							
							// Salimos del bucle.
							break;
						}

						// Guardamos el tiempo para el control de la animación
						// de efecto.			
						tb_efecto = SDL_GetTicks();

						// Si el tiempo transcurrido es mayor que el retardo de la
						// animación del efecto, dibujamos.				
						if(tb_efecto - ta_efecto > jugador_activo_
							-> IndicadorActual().AnimacionEfecto().Retardo())
						{
							// Dibujamos todo.
							campo_activo_ -> Dibujar();
							DibujarBolasInactivas();
							bola_activa_ -> DibujarSombra();
							bola_activa_ -> DibujarBola();
							jugador_activo_ -> Dibujar();
							jugador_activo_ -> CursorActual().Dibujar();
							jugador_activo_
								-> IndicadorActual().AnimacionFuerza().DibujarCuadro();
							jugador_activo_ -> IndicadorActual().AnimacionEfecto().Dibujar();
							campo_activo_ -> DibujarBandera();
							DibujarInterfaz();
							pantalla_ -> Actualizar();

							// Tomamos la nueva referencia de tiempo.				
							ta_efecto = SDL_GetTicks();
						}

						// Guardamos el tiempo del bucle.
						tb = SDL_GetTicks();

						// Si el tiempo transcurrido es mayor que el establecido en el
						// juego, esperamos.
						if(tb - ta < Constantes::_MS_)
							SDL_Delay(Constantes::_MS_ - (tb-ta));
					}

					// Se ha terminado de definir el golpe.
					if(terminado && fuerza_establecida && efecto_establecido)
					{
						this -> Esperar(2000, true, true, true, true, true, 0, 0);

						// Reiniciamos la animación de fuerza.
						jugador_activo_
							-> IndicadorActual().AnimacionFuerza().Reiniciar();

						// Reiniciamos la animación de efecto.					
						jugador_activo_
							-> IndicadorActual().AnimacionEfecto().Reiniciar();
						
						// Reiniciamos las variables de control.
						terminado = false;
						fuerza_establecida = false;
						efecto_establecido = false;
						
						// Realizamos la animación previa al golpe.
						jugador_activo_ -> AnimacionPrevia();
						
						// Golpeamos la bola.
						int retorno;
						retorno = JugadorActivo().GolpearBola();

						// Si el retorno es 0 continuamos con la partida.						
						if(retorno == 0)
							terminado = true;
						// Si es -1 terminamos el juego.
						else if(retorno == -1)
							return 0;

						// Si el jugador no ha finalizado, lo desactivamos.						
						if((*ijugador) -> EstadoActual() != Jugador::FINALIZADO)
							DesactivarJugador(*ijugador);
						
						
						// Si ya no quedan más jugadores inactivos, la partida ha acabado.
						if(!SiguienteJugador())
						{
							// No hay más hoyos por jugar. Terminamos.
							if(!SiguienteHoyo())
							{
								// Paramos la música.
								Mix_HaltMusic();

								// Actualizamos las puntuaciones.
								ActualizarPuntuaciones();

								// Invocamos la pantalla de finalización.
								return(menu_ -> Finalizar(Menu::STROKEPLAY));
							}
							else
							{
								// Actualizamos las puntuaciones.
								ActualizarPuntuaciones();

								// Reiniciamos los jugadores.
								ReiniciarJugadores();

								// Volvemos a colocar las bolas.
								ColocarBolas();

								// Activamos el primer jugador.
								ActivarJugador(jugadores_[0]);

								// Actualizamos las bolas inactivas.
								ActualizarBolasInactivas();
								
								// Generamos la nueva interfaz.
								GenerarInterfaz();

								// El iterador de jugadores apunta al primer jugador.
								ijugador = jugadores_.begin();

								// Centramos la cámara.
								CentrarCamara();

								// Dibujamos el campo.
								campo_activo_ -> Dibujar();

								// Dibujamos el rótulo con el nombre del campo.
								this -> DibujarNombreCampo();

								// Actualizamos la pantalla.
								pantalla_ -> Actualizar();
	
								// Esperamos.
								SDL_Delay(2500);

							}
						}

						// Activamos al jugador.						
						ActivarJugador(*ijugador);

						// Actualizamos las bolas inactivas.
						ActualizarBolasInactivas();

						// Dibujamos todo.				
						CentrarCamara();
						campo_activo_ -> Dibujar();
						DibujarBolasInactivas();
						bola_activa_ -> DibujarSombra();
						bola_activa_ -> DibujarBola();
						OrientarCursor();
						jugador_activo_ -> ActualizarCuadro();
						jugador_activo_ -> Dibujar();
						jugador_activo_ -> CursorActual().Dibujar();
						campo_activo_ -> DibujarBandera();
						interfaz_ -> Actualizar();
						DibujarInterfaz();
						pantalla_ -> Actualizar();
						
						// Vaciamos la cola de eventos.
						while(SDL_PollEvent(&tmp)){}
					}
				} // Teclado::Espacio

				// Elegimos el palo siguiente.			
				if(tecla == teclado_.Tecla(Teclado::ARRIBA))
				{
			
					JugadorActivo().PaloSiguiente();
					campo_activo_ -> Dibujar();
					DibujarBolasInactivas();
					bola_activa_ -> DibujarSombra();
					bola_activa_ -> DibujarBola();
					jugador_activo_ -> Dibujar();
					jugador_activo_ -> CursorActual().Dibujar();
					campo_activo_ -> DibujarBandera();
					interfaz_ -> ActualizarPalo();
					DibujarInterfaz();
					pantalla_ -> Actualizar();
					this -> Esperar(150, true, true, true, false, false, 0, 0);
				}

				// Elegimos el palo anterior.				
				if(tecla == teclado_.Tecla(Teclado::ABAJO))
				{
					JugadorActivo().PaloAnterior();
					campo_activo_ -> Dibujar();
					DibujarBolasInactivas();
					bola_activa_ -> DibujarSombra();
					bola_activa_ -> DibujarBola();
					jugador_activo_ -> Dibujar();
					jugador_activo_ -> CursorActual().Dibujar();
					campo_activo_ -> DibujarBandera();
					interfaz_ -> ActualizarPalo();
					DibujarInterfaz();
					pantalla_ -> Actualizar();	
					this -> Esperar(150, true, true, true, false, false, 0, 0);
				}

				// Giramos el cursor a la derecha.		
				if(tecla == teclado_.Tecla(Teclado::DERECHA))
				{
					jugador_activo_ -> CursorDerecha();
					jugador_activo_ -> ActualizarCuadro();
					campo_activo_ -> Dibujar();
					DibujarBolasInactivas();
					bola_activa_ -> DibujarSombra();
					bola_activa_ -> DibujarBola();
					jugador_activo_ -> Dibujar();
					jugador_activo_ -> CursorActual().Dibujar();
					campo_activo_ -> DibujarBandera();
					DibujarInterfaz();
					pantalla_ -> Actualizar();
					SDL_Delay(1);
				}

				// Giramos el cursor a la izquierda.		
				if(tecla == teclado_.Tecla(Teclado::IZQUIERDA))
				{
					jugador_activo_ -> CursorIzquierda();
					jugador_activo_ -> ActualizarCuadro();
					campo_activo_ -> Dibujar();
					DibujarBolasInactivas();
					bola_activa_ -> DibujarSombra();
					bola_activa_ -> DibujarBola();
					jugador_activo_ -> Dibujar();
					jugador_activo_ -> CursorActual().Dibujar();
					campo_activo_ -> DibujarBandera();
					DibujarInterfaz();
					pantalla_ -> Actualizar();	
					SDL_Delay(1);
				}

				// Mostramos u ocultamos el mapa.				
				if(tecla == teclado_.Tecla(Teclado::MAPA))
				{
					// La tecla de mapa no estaba activa.				
					if(!tecla_mapa)
					{
						// Activamos la tecla.
						tecla_mapa = true;

						// Dibujamos el mapa.	
						if(!dibujar_mapa)
						{
							dibujar_mapa = true;
							interfaz_ -> ActivarMapa();
						}
						// Ocultamos el mapa.
						else
						{
							dibujar_mapa = false;
							interfaz_ -> DesactivarMapa();					
						}
						
						// Dibujamos todo.
						campo_activo_ -> Dibujar();
						DibujarBolasInactivas();
						bola_activa_ -> DibujarSombra();
						bola_activa_ -> DibujarBola();
						jugador_activo_ -> Dibujar();
						jugador_activo_ -> CursorActual().Dibujar();
						campo_activo_ -> DibujarBandera();
						DibujarInterfaz();
						pantalla_ -> Actualizar();	
					}
				}
			} // if SDL_KEYDOWN

			// La tecla ha sido liberada.
			if(evento.key.type == SDL_KEYUP)
			{
				if(tecla == teclado_.Tecla(Teclado::MAPA))
				{
					// Si la tecla estaba activa, la desactivamos.
					if(tecla_mapa == true)
						tecla_mapa = false;
				}			
			}

			// Guardamos el tiempo para el control de la cola de eventos.			
			tb_while = SDL_GetTicks();

			// Si el tiempo es menor, esperamos.
			if(tb_while - ta_while < Constantes::_MS_)
				SDL_Delay(Constantes::_MS_ - (tb_while - ta_while));
			
		} // while PollEvent
		
		// El jugador es el ordenador.
		if((*ijugador) -> Tipo() != Jugador::HUMANO && terminado)
		{
			terminado = false;
			
			int retorno;
			retorno = JugadorActivo().GolpearBola();

			// Se ha realizado el tiro.
			if(retorno == 0)
				terminado = true;
			// Se ha elegido Salir desde el menú de pausa.
			else if(retorno == -1)
				return 0;
			// Se ha pulsado la x de cerrar ventana. Terminamos el juego.
			else if(retorno == 1)
				return -1;

			// Si el jugador no ha finalizado, lo desactivamos.
			if((*ijugador) -> EstadoActual() != Jugador::FINALIZADO)
				DesactivarJugador(*ijugador);

			if(!SiguienteJugador())
			{
				// No hay más hoyos por jugar. Terminamos.
				if(!SiguienteHoyo())
				{
					// Paramos la música.
					Mix_HaltMusic();

					// Actualizamos las puntuaciones.
					ActualizarPuntuaciones();

					// Invocamos la pantalla de finalización.
					return(menu_ -> Finalizar(Menu::STROKEPLAY));
				}
				else
				{
					// Actualizamos las puntuaciones.
					ActualizarPuntuaciones();

					// Reiniciamos los jugadores.
					ReiniciarJugadores();

					// Volvemos a colocar las bolas.
					ColocarBolas();

					// Activamos el primer jugador.
					ActivarJugador(jugadores_[0]);

					// Actualizamos la lista de bolas inactivas.
					ActualizarBolasInactivas();
					
					// Generamos la nueva interfaz.
					GenerarInterfaz();

					// El iterador de jugadores apunta al primer jugador.
					ijugador = jugadores_.begin();

					// Centramos la cámara.
					CentrarCamara();

					// Dibujamos el campo.
					campo_activo_ -> Dibujar();

					// Dibujamos el rótulo del campo.
					this -> DibujarNombreCampo();

					// Actualizamos la pantalla.
					pantalla_ -> Actualizar();

					// Esperamos.
					SDL_Delay(2500);

				}
			}

			// Activamos el jugador.
			ActivarJugador(*ijugador);

			// Actualizamos las bolas inactivas.
			ActualizarBolasInactivas();

			// Dibujamos todo.
			CentrarCamara();
			campo_activo_ -> Dibujar();
			DibujarBolasInactivas();
			bola_activa_ -> DibujarSombra();
			bola_activa_ -> DibujarBola();
			OrientarCursor();
			jugador_activo_ -> ActualizarCuadro();
			jugador_activo_ -> Dibujar();
			jugador_activo_ -> CursorActual().Dibujar();
			campo_activo_ -> DibujarBandera();
			interfaz_ -> Actualizar();
			DibujarInterfaz();
			pantalla_ -> Actualizar();

			// Vaciamos la cola de eventos.		
			SDL_Event tmp;
			while(SDL_PollEvent(&tmp));			
		}

		// Dibujamos todo.
		campo_activo_ -> Dibujar();
		DibujarBolasInactivas();
		bola_activa_ -> DibujarSombra();
		bola_activa_ -> DibujarBola();
		jugador_activo_ -> Dibujar();
		jugador_activo_ -> CursorActual().Dibujar();
		campo_activo_ -> DibujarBandera();
		DibujarInterfaz();
		pantalla_ -> Actualizar();

		// Guradamos el tiempo para el control del bucle.		
		tb_for = SDL_GetTicks();

		// Si el tiempo transcurrido es menor al establecido, esperamos.
		if(tb_for - ta_for < Constantes::_MS_)
			SDL_Delay(Constantes::_MS_ - (tb_for - ta_for));
		
	} //for GameLoop
}

bool Strokeplay::SiguienteHoyo()
{
	// Incrementamos el número de hoyo.
	++hoyo_actual_;

	// Ya no hay más campos. Terminamos.
	if(hoyo_actual_ > datos_campos_.size() - 1)
		return false;

	// Eliminamos el campo actual.
	delete campo_activo_;

	// Cargamos el nuevo campo.	
	campo_activo_ = new Campo(datos_campos_[hoyo_actual_].nombre_,
		datos_campos_[hoyo_actual_].hoyo_, datos_campos_[hoyo_actual_].par_,
		&(datos_campos_[hoyo_actual_].posicion_salida_),
		&(datos_campos_[hoyo_actual_].posicion_hoyo_),
		datos_campos_[hoyo_actual_].colores_, datos_campos_[hoyo_actual_].ruta_,
		pantalla_);

	// Volvemos.		
	return true;
}

void Strokeplay::ReiniciarJugadores()
{
	// Reiniciamos los jugadores.
	for(std::vector<Jugador*>::iterator i = jugadores_.begin();
		i != jugadores_.end(); i++)
		(*i) -> Reiniciar();

	// Establecemos todas las bolas al estado inactivo.		
	for(std::vector<Bola*>::iterator i = bolas_.begin();
		i != bolas_.end(); i++)
		(*i) -> Estado(Bola::INACTIVA);
}

void Strokeplay::ActualizarPuntuaciones()
{
	// Equipos.
	int x = 0;
	
	// Recorremos la lista de jugadores y actualizamos las puntuaciones.
	for(std::vector<Jugador*>::iterator i = jugadores_.begin();
		i != jugadores_.end(); ++x, ++i)
	{
		puntuaciones_[x] = puntuaciones_[x] +
			(*i) -> Par() - campo_activo_ -> Par();
	}
}

