/// \file strokeplay.h Modo de juego Strokeplay.
/*
	Copyright 2010 Andrés Francisco Aparicio

	This file is part of SDLgolf.

	SDLgolf is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	SDLgolf is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with SDLgolf.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef _STROKEPLAY_H_
#define _STROKEPLAY_H_

#include "partida.h"
#include "datos.h"

/*! \class Strokeplay.
    \brief Modo de juego Strokeplay.

    Implementación del modo de juego Strokeplay.
    
    En el modo de juego Strokeplay, uno o más jugadores se enfrentan en
    todos los hoyos de un campo.
    
		Al final de cada hoyo, cada jugador obtendrá una puntuación determinada
		por la resta del par del hoyo menos el par del propio jugador.
    
    Ganará la partida el jugador que obtenga la puntuación más baja, es decir,
    el jugador que haya completado todos los hoyos de un campo haciendo
    menor par que sus contrincantes.
 */
class Strokeplay: public Partida
{
	public:

		/*! Realiza las asociaciones con el Menu y la Pantalla.\n
				Crea y asigna las bolas de golf y los jugadores.\n
				Inicializa las puntuaciones a 0 y carga el primer Campo.\n
				Coloca las Bolas sobre el Campo, actualiza la lista de bolas inactivas
				y activa el primer Jugador.\n
				Crea las animaciones de las colisiones de la Bola con los elementos
				del Campo.
				
				\brief Constructor.
				\param jugadores Datos de los jugadores que participan en la Partida.
				\param campos Datos de los campos en los que se desarrollará la
							 Partida.
				\param menu Menú del Juego.
				\param pantalla Pantalla del Juego.
				\see Modos.
				\see Campo::LugarSalida.
				\see Campo::ElementosCampo.
				\see Datos::Jugador.
				\see Datos::Campo.
		 */
		Strokeplay(std::vector<Datos::Jugador>& jugadores,
			std::vector<Datos::Campo>& campos, Menu *menu, Pantalla *pantalla);

		/*! \brief Inicia la partida.
				\return \arg \b 0 hemos salido de la Partida desde el Menu de pausa.
								\arg \b -1 hemos cerrado la ventana del Juego.
				\see Menu::Pausa.
		 */
		int Iniciar();

		/*! Pasa al siguiente hoyo, destruyendo el actual y modificando el
				campo activo.
				\brief Pasa al siguiente hoyo.
				\return \arg \b Verdadero en caso de haber más hoyos.
								\arg \b Falso en caso de no haber más hoyos.
		 */
		bool SiguienteHoyo();

		/*! \brief Reinicia todos los jugadores de la Partida.
				\see Jugador::Reiniciar()
		 */
		void ReiniciarJugadores();

		/*! Actualiza las puntuaciones de los jugadores. \n
				La puntuación del hoyo se obtiene restando al par del Jugador el
				par del Campo.
				\brief Actualiza las puntuaciones.
				\see Jugador::Par.
				\see Campo::Par.
		 */
		void ActualizarPuntuaciones();

	private:

		unsigned int hoyo_actual_;	///< Hoyo en el que se está jugando.

		std::vector<Datos::Campo>
			datos_campos_;	///< Datos de los campos de la Partida.
};

#endif
