/// \file utilidades.cpp Funciones y utilidades.
/*
	Copyright 2010 Andrés Francisco Aparicio

	This file is part of SDLgolf.

	SDLgolf is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	SDLgolf is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with SDLgolf.  If not, see <http://www.gnu.org/licenses/>.
*/

#include <SDL/SDL.h>
#include <SDL/SDL_image.h>
#include <SDL/SDL_ttf.h>
#include <iostream>
#include <ctime>
#include "utilidades.h"


SDL_Surface * Utilidades::CargarImagen(std::string ruta)
{
	// Imágenes que vamos a utilizar.
	SDL_Surface *t1, *t2;
	
	// Cargamos la imagen.
	t1 = IMG_Load(ruta.c_str());

	// Comprobamos que la carga ha sido correcta.
	if(t1 == 0)
	{
		std::cerr << "utilidades.cpp CargarImagen: Error al abrir la imagen."
			<< std::endl << "> " << ruta << " <" << std::endl;
		exit(1);
	}
	
	// Convertimos la imagen al formato de la pantalla.
	t2 = SDL_DisplayFormatAlpha(t1);
	
	// Comprobamos que la imagen sea correcta.
	if(t2 == 0)
	{
		std::cerr
		<< "utilidades.cpp CargarImagen: Error al intercambiar la imagen."
		<< SDL_GetError() << std::endl;
		exit(1);
	}
	
	// Liberamos la imagen.
	SDL_FreeSurface(t1);
	
	// Devolvemos la imagen convertida al formato de pantalla.
	return t2;
}

SDL_Surface * Utilidades::CrearTexto(const char *texto, const char *fuente,
	int tam, SDL_Color color)
{
	// El texto está vacío.
	if(!strlen(texto))
	{
		std::cerr << "utilidades.cpp CrearTexto: Error, texto vacío."
		<< std::endl;
		exit(1);
	}

	// Abrimos la fuente.
	TTF_Font *f = TTF_OpenFont(fuente, tam);

	// La fuente nos e ha abierto correctamente.
	if(!fuente)
	{
		std::cerr << "TTF_OpenFont: " << std::endl << TTF_GetError();
		exit(1);
	}

	// Imágenes que vamos a usar.
	SDL_Surface *renderizado;
	SDL_Surface *conversion;

	// Creamos el texto.
	renderizado = TTF_RenderText_Blended(f, texto, color);
	
	// Lo convertimos al formato de pantalla.
	conversion = SDL_DisplayFormatAlpha(renderizado);

	// Cerramos la fuente.
	TTF_CloseFont(f);
	
	// Liberamos la imagen.
	SDL_FreeSurface(renderizado);

	// Devolvemos la imagen del texto convertida al formato de pantalla.
	return conversion;
}

SDL_Color Utilidades::Color(int color)
{
	SDL_Color c;
	
	// Definiciones RGB para los colores.
	switch(color)
	{
		case AMARILLO: c.r = 255; c.g = 255; c.b = 0; return c;
		case AZUL: c.r = 0; c.g = 0; c.b = 255; return c;
		case AZUL_MARINO: c.r = 0; c.g = 0; c.b = 128; return c;
		case BLANCO: c.r = 255; c.g = 255; c.b = 255; return c;
		case CIAN: c.r = 0; c.g = 255; c.b = 255; return c;
		case FUCSIA: c.r = 255; c.g = 0; c.b = 255; return c;
		case GRIS: c.r = 128; c.g = 128; c.b = 128;	return c;
		case LIMA: c.r = 0; c.g = 255; c.b = 0; return c;
		case MARRON: c.r = 128; c.g = 0; c.b = 0; return c;
		case NEGRO: c.r = 0; c.g = 0; c.b = 0; return c;
		case OLIVA:	c.r = 128; c.g = 128; c.b = 0; return c;		
		case PLATA: c.r = 192; c.g = 192; c.b = 192; return c;
		case PURPURA: c.r = 128; c.g = 0; c.b = 128; return c;	
		case ROJO: c.r = 255; c.g = 0; c.b = 0; return c;
		case VERDE: c.r = 0; c.g = 128; c.b = 0; return c;
		case VERDE_AZULADO: c.r = 0; c.g = 128; c.b = 128; return c;
		default: break;
	}
	std::cerr << "utilidades.cpp Color: color inválido." << std::endl;
	exit(1);
}

int Utilidades::DistanciaEuclidea(const SDL_Rect& origen,
	const SDL_Rect& destino)
{
	// Devolvemos la distancia euclídea.
	return sqrt(pow(destino.x - origen.x, 2) + pow(destino.y - origen.y, 2));
}

int Utilidades::FiltroDeEventos(const SDL_Event *evento)
{
	// Filtramos los eventos del ratón.
	if(evento -> type == SDL_MOUSEMOTION
	|| evento -> type == SDL_MOUSEBUTTONDOWN
	|| evento -> type == SDL_MOUSEBUTTONUP)
		return 0;
	else
		return 1;
}

int Utilidades::NumeroAleatorio(int a, int b)
{
	// Generamos un número aleatorio en el intervalo [a, b] y lo devolvemos.	
	return(rand() % (b - a + 1) + a); 
}

void Utilidades::PonerPixel(SDL_Surface *superficie, int x, int y,
	Uint32 pixel)
{
	// Comprobamos que las coordenadas estén dentro de los límites de la
	// superficie.
	if(x < 0 || y < 0 || x >= superficie -> w || y >= superficie -> h)
	{
		std::cerr << "pixel.cpp: PonerPixel() Error, coordenadas (" << x << ", "
		<< y << ") fuera de límites" << std::endl;
		exit(1);
	}

	// Obtenemos la profundidad de color.
	int bpp = superficie->format->BytesPerPixel;

	// Obtenemos la posición del píxel a sustituir.
	Uint8 *p = (Uint8 *)superficie->pixels + y * superficie->pitch + x*bpp;

	// Elegimos la profundidad del color.
	switch (bpp)
	{
		// 8 bits (256 colores).
		case 1: *p = pixel; break;
		
		// 16 bits (65536 colores).
		case 2: *(Uint16 *)p = pixel; break;
		
		// 24 bits
		case 3:
		{
			// Determinamos la naturaleza del sistema. Big Endian o Little Endian.
			if(SDL_BYTEORDER == SDL_BIG_ENDIAN)
			{ 
				// Calculamos cada una de las componentes de color
				// 3 bytes, 3 posiciones.
				p[R] = (pixel >> 16) & 0xFF;
				p[G] = (pixel >> 8) & 0xFF;
				p[B] = pixel & 0xFF;
			}
			else
			{
				// Calculamos cada una de las componentes de color
				// 3 byes, 3 posiciones.
				p[R] = pixel & 0xFF;
				p[G] = (pixel >> 8) & 0xFF;
				p[B] = (pixel >> 16) & 0xFF;
			}
			break;
		}
		
		// 32 bits (True Color + Alpha).
		case 4: *(Uint32 *) p = pixel; break;
	}
}

Uint32 Utilidades::ObtenerPixel(int x, int y, SDL_Surface *superficie)
{
	// Comprobamos que las coordenadas estén dentro de los límites de la
	// superficie
	if(x < 0 || y < 0 || x >= superficie -> w || y >= superficie -> h)
	{
		std::cerr << "ObtenerPixel(): Error, coordenadas (" << x << ", " << y
			<< ") fuera de límites" << std::endl;
		exit(1);
	}

	// Obtenemos la profundidad de color.
	int bpp = superficie -> format -> BytesPerPixel;

	// Obtenemos la posición del píxel a consultar.
	Uint8 *p = (Uint8 *)superficie -> pixels + y
		* superficie -> pitch + x * bpp;

	// Elegimos la profundidad del color.
	switch (bpp)
	{
		// 256 colores.
		case 1: return *p;
		
		// 65536 colores.
		case 2: return *(Uint16 *)p;

		// 24 bits.
		case 3: 
			// Según la naturaleza del sistema
			if (SDL_BYTEORDER == SDL_BIG_ENDIAN)
				// OR entre los distintos componentes del color
				return p[R] << 16 | p[G] << 8 | p[B];
			else
				// OR entre los distintos componentes del color
				return p[R] | p[G] << 8 | p[B] << 16;

		// 32 bits (True Color + Alpha).
		case 4:
			return *(Uint32 *)p;
		default:
			return 0;
	}
}
