/// \file utilidades.h Funciones y utilidades.
/*
	Copyright 2010 Andrés Francisco Aparicio

	This file is part of SDLgolf.

	SDLgolf is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	SDLgolf is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with SDLgolf.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef _UTILIDADES_H_
#define _UTILIDADES_H_

#include <SDL/SDL_image.h>
#include <cmath>
#include "datos.h"

/*!	\namespace Utilidades.
		\brief Conjunto de utilidades.
 */
namespace Utilidades
{

	/*! \enum Colores
			\brief Lista de colores.
	 */	
	enum Colores { AMARILLO, AZUL, AZUL_MARINO, BLANCO, CIAN, FUCSIA, GRIS,
								 LIMA, MARRON, NEGRO, OLIVA, PLATA, PURPURA, ROJO, VERDE,
								 VERDE_AZULADO };

	/*! \enum RGB
			Valor en RGB del color de un píxel.
			\brief Colores.

			\var R
	 		Rojo.
	 		
	 		\var G
	 		Verde.
	 		
	 		\var B
	 		Azul.
	 */	
	enum RGB {R, G, B};

	/*! Carga una imagen de un fichero y la guarda en un superficie temporal.\n
			Convierte la superficie temporal al formato de Pantalla para mejorar el
			rendimiento.\n
			Destruye la superficie temporal y devuelve la conversión.
			\brief Carga una imagen desde un fichero.
			\param ruta Ruta a la imagen.
			\return Superficie de la imagen convertida al formato de la Pantalla.
	 */
	SDL_Surface * CargarImagen(std::string ruta);

	/*! Genera la imagen de un texto a partir de una cadena.\n
			Abre una determinada fuente con un determinado tamaño y crea una
			superficie temporal.\n
			Convierte la superficie temporal al formato de Pantalla para mejorar el
			rendimiento.\n
			Destruye la superficie temporal y devuelva la conversión. 
			\brief Crea un texto.
			\param texto Texto que se desea crear.
			\param fuente Ruta a la fuente a emplear.
			\param tam Tamaño de la fuente.
			\param color Color que se quiere aplicar al texto.
			\return Texto convertido al formato de la Pantalla.
			\see Colores.
			\see Color.
	 */
	SDL_Surface * CrearTexto(const char *texto, const char *fuente, int tam,
		SDL_Color color);
	
	/*! Devuelve el color asociado a un entero.
			\brief Devuelve un color.
			\param color Color.
			\brief Color asociado al entero.
			\see Colores.
	 */
	SDL_Color Color(int color);

	/*! Calcula la distancia euclídea que existe entre dos puntos.
			\brief Calcula la distancia entre dos puntos.
			\param origen Posición del origen.
			\param destino Posición del destino.
			\return Distancia euclídea entre los dos puntos.
	 */
	int DistanciaEuclidea(const SDL_Rect& origen, const SDL_Rect& destino);

	/*! Filtra los eventos generados por el usuario.
			\brief Filtra los eventos.
			\param evento Evento a filtrar.
			\return \arg \b 0 Si el evento debe ser filtrado.
							\arg \b 1 En caso contrario.
	 */
	int FiltroDeEventos(const SDL_Event *evento);


	/*! Genera un número pseudo-aleatorio en un rango <em>[a, b]</em>
			determinado.
			\brief Genera un número aleatorio.
			\param a Comienzo del intervalo.
			\param b Final del intervalo. 
			\return Número aleatorio en el rango especificado.
	 */
	int NumeroAleatorio(int a, int b);
	
	/*! \brief Sustituye el color de un píxel.
			\param superficie Superficie en la que vamos a cambiar el píxel.
			\param x Coordenada x del píxel.
			\param y Coordenada y del píxel.
			\param pixel Color del píxel.
	*/
	void PonerPixel(SDL_Surface *superficie, int x, int y, Uint32 pixel);


	/*! \brief Obtiene el color de un píxel.
			\param x Coordenada x del píxel.
			\param y Coordenada y del píxel.
			\param superficie Superficie de la que vamos a obtener el píxel.
	*/
	Uint32 ObtenerPixel(int x, int y, SDL_Surface *superficie);
}

#endif
